@@ -3,7 +3,7 @@ use std::path::PathBuf;
 use base64::Engine;
 use iced_core::Color;
 use palette::rgb::{Rgb, Rgba};
-use palette::{FromColor, Hsva, Okhsl, Srgb, Srgba};
+use palette::{FromColor, Hsva, Okhsl, Srgba};
 use rand::prelude::*;
 use rand_chacha::ChaChaRng;
 use serde::{Deserialize, Serialize};
@@ -160,6 +160,12 @@ pub struct ServerMessages {
     pub monitored_online: Option<Color>,
     #[serde(default, with = "color_serde_maybe")]
     pub monitored_offline: Option<Color>,
+    #[serde(default, with = "color_serde_maybe")]
+    pub standard_reply_fail: Option<Color>,
+    #[serde(default, with = "color_serde_maybe")]
+    pub standard_reply_warn: Option<Color>,
+    #[serde(default, with = "color_serde_maybe")]
+    pub standard_reply_note: Option<Color>,
     #[serde(default = "default_transparent", with = "color_serde")]
     pub default: Color,
 }
@@ -222,20 +228,20 @@ pub fn color_to_hex(color: Color) -> String {
     let [r, g, b, a] = color.into_rgba8();
 
     let _ = write!(&mut hex, "#");
-    let _ = write!(&mut hex, "{:02X}", r);
-    let _ = write!(&mut hex, "{:02X}", g);
-    let _ = write!(&mut hex, "{:02X}", b);
+    let _ = write!(&mut hex, "{r:02X}");
+    let _ = write!(&mut hex, "{g:02X}");
+    let _ = write!(&mut hex, "{b:02X}");
 
     if a < u8::MAX {
-        let _ = write!(&mut hex, "{:02X}", a);
+        let _ = write!(&mut hex, "{a:02X}");
     }
 
     hex
 }
 
 /// Adjusts the transparency of the foreground color based on the background color's lightness.
-pub fn alpha_color(min_alpha: f32, max_alpha: f32, background: Color, foreground: Color) -> Color {
-    alpha(
+pub fn alpha_color_calculate(min_alpha: f32, max_alpha: f32, background: Color, foreground: Color) -> Color {
+    alpha_color(
         foreground,
         min_alpha + to_hsl(background).lightness * (max_alpha - min_alpha),
     )
@@ -253,7 +259,7 @@ pub fn randomize_color(original_color: Color, seed: &str) -> Color {
     let original_hsl = to_hsl(original_color);
 
     // Randomize the hue value using the random number generator
-    let randomized_hue: f32 = rng.gen_range(0.0..=360.0);
+    let randomized_hue: f32 = rng.random_range(0.0..=360.0);
     let randomized_hsl = Okhsl::new(
         randomized_hue,
         original_hsl.saturation,
@@ -265,7 +271,7 @@ pub fn randomize_color(original_color: Color, seed: &str) -> Color {
 }
 
 pub fn to_hsl(color: Color) -> Okhsl {
-    let mut hsl = Okhsl::from_color(Rgb::from(color));
+    let mut hsl = Okhsl::from_color(to_rgb(color));
     if hsl.saturation.is_nan() {
         hsl.saturation = Okhsl::max_saturation();
     }
@@ -274,18 +280,19 @@ pub fn to_hsl(color: Color) -> Okhsl {
 }
 
 pub fn to_hsva(color: Color) -> Hsva {
-    Hsva::from_color(Rgba::from(color))
+    Hsva::from_color(to_rgba(color))
 }
 
 pub fn from_hsva(color: Hsva) -> Color {
-    Srgba::from_color(color).into()
+    to_color(Srgba::from_color(color))
+
 }
 
 pub fn from_hsl(hsl: Okhsl) -> Color {
-    Srgb::from_color(hsl).into()
+    to_color(Srgba::from_color(hsl))
 }
 
-pub fn alpha(color: Color, alpha: f32) -> Color {
+pub fn alpha_color(color: Color, alpha: f32) -> Color {
     Color { a: alpha, ..color }
 }
 
@@ -293,6 +300,31 @@ fn default_transparent() -> Color {
     Color::TRANSPARENT
 }
 
+fn to_rgb(color: Color) -> Rgb {
+    Rgb {
+        red: color.r,
+        green: color.g,
+        blue: color.b,
+        ..Rgb::default()
+    }
+}
+
+fn to_rgba(color: Color) -> Rgba {
+    Rgba {
+        alpha: color.a,
+        color: to_rgb(color),
+    }
+}
+
+fn to_color(rgba: Rgba) -> Color {
+    Color {
+        r: rgba.color.red,
+        g: rgba.color.green,
+        b: rgba.color.blue,
+        a: rgba.alpha,
+    }
+}
+
 mod color_serde {
     use iced_core::Color;
     use serde::{Deserialize, Deserializer, Serialize, Serializer};
@@ -418,6 +450,9 @@ mod binary {
         ButtonsSecondaryBackgroundHover = 35,
         ButtonsSecondaryBackgroundSelected = 36,
         ButtonsSecondaryBackgroundSelectedHover = 37,
+        BufferServerMessagesStandardReplyFail = 38,
+        BufferServerMessagesStandardReplyWarn = 39,
+        BufferServerMessagesStandardReplyNote = 40,
     }
 
     impl Tag {
@@ -456,6 +491,15 @@ mod binary {
                 Tag::BufferServerMessagesMonitoredOffline => {
                     colors.buffer.server_messages.monitored_offline?
                 }
+                Tag::BufferServerMessagesStandardReplyFail => {
+                    colors.buffer.server_messages.standard_reply_fail?
+                }
+                Tag::BufferServerMessagesStandardReplyWarn => {
+                    colors.buffer.server_messages.standard_reply_warn?
+                }
+                Tag::BufferServerMessagesStandardReplyNote => {
+                    colors.buffer.server_messages.standard_reply_note?
+                }
                 Tag::BufferServerMessagesDefault => colors.buffer.server_messages.default,
                 Tag::ButtonsPrimaryBackground => colors.buttons.primary.background,
                 Tag::ButtonsPrimaryBackgroundHover => colors.buttons.primary.background_hover,
@@ -515,6 +559,15 @@ mod binary {
                 Tag::BufferServerMessagesMonitoredOffline => {
                     colors.buffer.server_messages.monitored_offline = Some(color);
                 }
+                Tag::BufferServerMessagesStandardReplyFail => {
+                    colors.buffer.server_messages.standard_reply_fail = Some(color);
+                }
+                Tag::BufferServerMessagesStandardReplyWarn => {
+                    colors.buffer.server_messages.standard_reply_warn = Some(color);
+                }
+                Tag::BufferServerMessagesStandardReplyNote => {
+                    colors.buffer.server_messages.standard_reply_note = Some(color);
+                }
                 Tag::BufferServerMessagesDefault => colors.buffer.server_messages.default = color,
                 Tag::ButtonsPrimaryBackground => colors.buttons.primary.background = color,
                 Tag::ButtonsPrimaryBackgroundHover => {
