@@ -2,7 +2,8 @@ use core::fmt;
 
 use serde::{Deserialize, Serialize};
 
-use crate::user::Nick;
+use crate::config::buffer::NicknameClickAction;
+use crate::target::{self, Target};
 use crate::{channel, config, message, Server};
 
 #[derive(Debug, Clone, PartialEq, Eq, Hash, Serialize, Deserialize)]
@@ -15,8 +16,8 @@ pub enum Buffer {
 #[derive(Debug, Clone, PartialEq, Eq, Hash, Serialize, Deserialize)]
 pub enum Upstream {
     Server(Server),
-    Channel(Server, String),
-    Query(Server, Nick),
+    Channel(Server, target::Channel),
+    Query(Server, target::Query),
 }
 
 #[derive(Debug, Clone, Copy, PartialEq, Eq, Hash, Serialize, Deserialize, strum::Display)]
@@ -28,6 +29,13 @@ pub enum Internal {
 }
 
 impl Buffer {
+    pub fn key(&self) -> String {
+        match self {
+            Buffer::Upstream(upstream) => upstream.key(),
+            Buffer::Internal(internal) => internal.key(),
+        }
+    }
+
     pub fn upstream(&self) -> Option<&Upstream> {
         if let Self::Upstream(upstream) = self {
             Some(upstream)
@@ -46,23 +54,35 @@ impl Buffer {
 }
 
 impl Upstream {
+    pub fn key(&self) -> String {
+        match self {
+            Upstream::Server(server) => format!("server:{server}"),
+            Upstream::Channel(server, channel) => {
+                format!("server:{server}:{}", channel.as_str())
+            }
+            Upstream::Query(server, query) => {
+                format!("server:{server}:{}", query.as_str())
+            }
+        }
+    }
+
     pub fn server(&self) -> &Server {
         match self {
             Self::Server(server) | Self::Channel(server, _) | Self::Query(server, _) => server,
         }
     }
 
-    pub fn channel(&self) -> Option<&str> {
+    pub fn channel(&self) -> Option<&target::Channel> {
         match self {
             Self::Channel(_, channel) => Some(channel),
             Self::Server(_) | Self::Query(_, _) => None,
         }
     }
 
-    pub fn target(&self) -> Option<String> {
+    pub fn target(&self) -> Option<Target> {
         match self {
-            Self::Channel(_, channel) => Some(channel.clone()),
-            Self::Query(_, nick) => Some(nick.to_string()),
+            Self::Channel(_, channel) => Some(Target::Channel(channel.clone())),
+            Self::Query(_, query) => Some(Target::Query(query.clone())),
             Self::Server(_) => None,
         }
     }
@@ -75,10 +95,9 @@ impl Upstream {
             Self::Channel(_, channel) => message::Target::Channel {
                 channel,
                 source: message::Source::Server(source),
-                prefix: None,
             },
-            Self::Query(_, nick) => message::Target::Query {
-                nick,
+            Self::Query(_, query) => message::Target::Query {
+                query,
                 source: message::Source::Server(source),
             },
         }
@@ -87,6 +106,15 @@ impl Upstream {
 
 impl Internal {
     pub const ALL: &'static [Self] = &[Self::FileTransfers, Self::Logs, Self::Highlights];
+
+    pub fn key(&self) -> String {
+        match self {
+            Internal::FileTransfers => "file-transfers",
+            Internal::Logs => "logs",
+            Internal::Highlights => "highlights",
+        }
+        .to_string()
+    }
 }
 
 #[derive(Debug, Clone, Default, Deserialize, Serialize)]
@@ -102,12 +130,39 @@ impl From<config::Buffer> for Settings {
     }
 }
 
-#[derive(Debug, Clone, Copy, Default, Deserialize)]
+#[derive(Debug, Clone, Default, Deserialize)]
 pub struct TextInput {
     #[serde(default)]
     pub visibility: TextInputVisibility,
     #[serde(default)]
     pub auto_format: AutoFormat,
+    #[serde(default)]
+    pub autocomplete: Autocomplete,
+}
+
+#[derive(Debug, Clone, Copy, Default, Deserialize)]
+#[serde(rename_all = "kebab-case")]
+pub enum SortDirection {
+    #[default]
+    Asc,
+    Desc,
+}
+
+#[derive(Debug, Clone, Deserialize)]
+pub struct Autocomplete {
+    #[serde(default)]
+    pub sort_direction: SortDirection,
+    #[serde(default = "default_completion_suffixes")]
+    pub completion_suffixes: [String; 2],
+}
+
+impl Default for Autocomplete {
+    fn default() -> Self {
+        Self {
+            sort_direction: SortDirection::default(),
+            completion_suffixes: default_completion_suffixes(),
+        }
+    }
 }
 
 #[derive(Debug, Clone, Copy, Default, Deserialize)]
@@ -144,6 +199,23 @@ impl Default for Timestamp {
     }
 }
 
+#[derive(Debug, Clone, Deserialize)]
+pub struct DateSeparators {
+    #[serde(default = "default_date_separators")]
+    pub format: String,
+    #[serde(default = "default_bool_true")]
+    pub show: bool,
+}
+
+impl Default for DateSeparators {
+    fn default() -> Self {
+        Self {
+            format: default_date_separators(),
+            show: true,
+        }
+    }
+}
+
 #[derive(Debug, Clone, Deserialize)]
 pub struct Nickname {
     #[serde(default)]
@@ -154,6 +226,8 @@ pub struct Nickname {
     pub alignment: Alignment,
     #[serde(default = "default_bool_true")]
     pub show_access_levels: bool,
+    #[serde(default)]
+    pub click: NicknameClickAction,
 }
 
 impl Default for Nickname {
@@ -163,6 +237,7 @@ impl Default for Nickname {
             brackets: Default::default(),
             alignment: Default::default(),
             show_access_levels: default_bool_true(),
+            click: Default::default(),
         }
     }
 }
@@ -229,10 +304,43 @@ impl Resize {
     }
 }
 
+#[derive(Debug, Clone, Copy, PartialEq, Eq, Default, Deserialize)]
+#[serde(rename_all = "kebab-case")]
+pub enum SkinTone {
+    #[default]
+    Default,
+    Light,
+    MediumLight,
+    Medium,
+    MediumDark,
+    Dark,
+}
+
+impl From<SkinTone> for emojis::SkinTone {
+    fn from(skin_tone: SkinTone) -> Self {
+        match skin_tone {
+            SkinTone::Default => emojis::SkinTone::Default,
+            SkinTone::Light => emojis::SkinTone::Light,
+            SkinTone::MediumLight => emojis::SkinTone::MediumLight,
+            SkinTone::Medium => emojis::SkinTone::Medium,
+            SkinTone::MediumDark => emojis::SkinTone::MediumDark,
+            SkinTone::Dark => emojis::SkinTone::Dark,
+        }
+    }
+}
+
 fn default_timestamp() -> String {
     "%R".to_string()
 }
 
+fn default_date_separators() -> String {
+    "%A, %B %-d".to_string()
+}
+
 fn default_bool_true() -> bool {
     true
 }
+
+fn default_completion_suffixes() -> [String; 2] {
+    [": ".to_string(), " ".to_string()]
+}
