@@ -1,23 +1,12 @@
 use std::path::PathBuf;
 use std::{str, string};
 
-use tokio_stream::wrappers::ReadDirStream;
-use tokio_stream::StreamExt;
-
 use rand::prelude::*;
 use rand_chacha::ChaCha8Rng;
-
 use serde::Deserialize;
 use thiserror::Error;
-
-pub use self::buffer::Buffer;
-pub use self::channel::Channel;
-pub use self::file_transfer::FileTransfer;
-pub use self::keys::Keyboard;
-pub use self::notification::Notifications;
-pub use self::proxy::Proxy;
-pub use self::server::Server;
-pub use self::sidebar::Sidebar;
+use tokio_stream::wrappers::ReadDirStream;
+use tokio_stream::StreamExt;
 
 use crate::appearance::theme::Colors;
 use crate::appearance::{self, Appearance};
@@ -26,11 +15,26 @@ use crate::environment::config_dir;
 use crate::server::Map as ServerMap;
 use crate::{environment, Theme};
 
+pub use self::actions::Actions;
+pub use self::buffer::Buffer;
+pub use self::file_transfer::FileTransfer;
+pub use self::highlights::Highlights;
+pub use self::keys::Keyboard;
+pub use self::notification::Notifications;
+pub use self::pane::Pane;
+pub use self::preview::Preview;
+pub use self::proxy::Proxy;
+pub use self::server::Server;
+pub use self::sidebar::Sidebar;
+
+pub mod actions;
 pub mod buffer;
-pub mod channel;
 pub mod file_transfer;
+pub mod highlights;
 pub mod keys;
 pub mod notification;
+pub mod pane;
+pub mod preview;
 pub mod proxy;
 pub mod server;
 pub mod sidebar;
@@ -46,11 +50,15 @@ pub struct Config {
     pub font: Font,
     pub scale_factor: ScaleFactor,
     pub buffer: Buffer,
+    pub pane: Pane,
     pub sidebar: Sidebar,
     pub keyboard: Keyboard,
     pub notifications: Notifications<Sound>,
     pub file_transfer: FileTransfer,
     pub tooltips: bool,
+    pub preview: Preview,
+    pub highlights: Highlights,
+    pub actions: Actions,
 }
 
 #[derive(Debug, Clone, Copy, Deserialize)]
@@ -156,6 +164,8 @@ impl Config {
             #[serde(default)]
             pub buffer: Buffer,
             #[serde(default)]
+            pub pane: Pane,
+            #[serde(default)]
             pub sidebar: Sidebar,
             #[serde(default)]
             pub keyboard: Keyboard,
@@ -165,6 +175,12 @@ impl Config {
             pub file_transfer: FileTransfer,
             #[serde(default = "default_tooltip")]
             pub tooltips: bool,
+            #[serde(default)]
+            pub preview: Preview,
+            #[serde(default)]
+            pub highlights: Highlights,
+            #[serde(default)]
+            pub actions: Actions,
         }
 
         let path = Self::path();
@@ -189,6 +205,10 @@ impl Config {
             notifications,
             file_transfer,
             tooltips,
+            preview,
+            pane,
+            highlights,
+            actions,
         } = toml::from_str(content.as_ref()).map_err(|e| Error::Parse(e.to_string()))?;
 
         servers.read_passwords().await?;
@@ -211,6 +231,10 @@ impl Config {
             notifications: loaded_notifications,
             file_transfer,
             tooltips,
+            preview,
+            pane,
+            highlights,
+            actions,
         })
     }
 
@@ -308,12 +332,12 @@ impl Config {
 }
 
 pub fn random_nickname() -> String {
-    let mut rng = ChaCha8Rng::from_entropy();
+    let mut rng = ChaCha8Rng::from_rng(&mut rand::rng());
     random_nickname_with_seed(&mut rng)
 }
 
 pub fn random_nickname_with_seed<R: Rng>(rng: &mut R) -> String {
-    let rand_digit: u16 = rng.gen_range(1000..=9999);
+    let rand_digit: u16 = rng.random_range(1000..=9999);
     let rand_nick = format!("halloy{rand_digit}");
 
     rand_nick
