@@ -1,14 +1,21 @@
 use chrono::{DateTime, Local, Utc};
 use serde::Deserialize;
 
-use super::Channel;
+pub use self::away::Away;
+pub use self::channel::Channel;
+
+pub mod away;
+pub mod channel;
+
 use crate::{
-    buffer::{Nickname, StatusMessagePrefix, TextInput, Timestamp},
+    buffer::{DateSeparators, Nickname, SkinTone, StatusMessagePrefix, TextInput, Timestamp},
     message::source,
 };
 
 #[derive(Debug, Clone, Deserialize, Default)]
 pub struct Buffer {
+    #[serde(default)]
+    pub away: Away,
     #[serde(default)]
     pub timestamp: Timestamp,
     #[serde(default)]
@@ -23,6 +30,56 @@ pub struct Buffer {
     pub internal_messages: InternalMessages,
     #[serde(default)]
     pub status_message_prefix: StatusMessagePrefix,
+    #[serde(default)]
+    pub chathistory: ChatHistory,
+    #[serde(default)]
+    pub date_separators: DateSeparators,
+    #[serde(default)]
+    pub commands: Commands,
+    #[serde(default)]
+    pub emojis: Emojis,
+}
+
+#[derive(Debug, Clone, Deserialize, Default)]
+#[serde(rename_all = "kebab-case")]
+pub enum NicknameClickAction {
+    #[default]
+    OpenQuery,
+    InsertNickname,
+}
+
+#[derive(Debug, Clone, Deserialize)]
+pub struct Emojis {
+    #[serde(default = "default_bool_true")]
+    pub show_picker: bool,
+    #[serde(default)]
+    pub skin_tone: SkinTone,
+    #[serde(default = "default_bool_true")]
+    pub auto_replace: bool,
+}
+
+impl Default for Emojis {
+    fn default() -> Self {
+        Self {
+            show_picker: default_bool_true(),
+            skin_tone: Default::default(),
+            auto_replace: default_bool_true(),
+        }
+    }
+}
+
+#[derive(Debug, Clone, Deserialize)]
+pub struct Commands {
+    #[serde(default = "default_bool_true")]
+    pub show_description: bool,
+}
+
+impl Default for Commands {
+    fn default() -> Self {
+        Self {
+            show_description: default_bool_true(),
+        }
+    }
 }
 
 #[derive(Debug, Clone, Default, Deserialize)]
@@ -41,6 +98,12 @@ pub struct ServerMessages {
     pub monitored_online: ServerMessage,
     #[serde(default)]
     pub monitored_offline: ServerMessage,
+    #[serde(default)]
+    pub standard_reply_fail: ServerMessage,
+    #[serde(default)]
+    pub standard_reply_warn: ServerMessage,
+    #[serde(default)]
+    pub standard_reply_note: ServerMessage,
 }
 
 impl ServerMessages {
@@ -53,6 +116,15 @@ impl ServerMessages {
             source::server::Kind::ChangeHost => Some(&self.change_host),
             source::server::Kind::MonitoredOnline => Some(&self.monitored_online),
             source::server::Kind::MonitoredOffline => Some(&self.monitored_offline),
+            source::server::Kind::StandardReply(source::server::StandardReply::Fail) => {
+                Some(&self.standard_reply_fail)
+            }
+            source::server::Kind::StandardReply(source::server::StandardReply::Warn) => {
+                Some(&self.standard_reply_warn)
+            }
+            source::server::Kind::StandardReply(source::server::StandardReply::Note) => {
+                Some(&self.standard_reply_note)
+            }
         }
     }
 }
@@ -139,6 +211,20 @@ impl Default for InternalMessage {
     }
 }
 
+#[derive(Debug, Clone, Deserialize)]
+pub struct ChatHistory {
+    #[serde(default = "default_bool_true")]
+    pub infinite_scroll: bool,
+}
+
+impl Default for ChatHistory {
+    fn default() -> Self {
+        Self {
+            infinite_scroll: true,
+        }
+    }
+}
+
 #[derive(Debug, Copy, Clone, Default, Deserialize)]
 #[serde(rename_all = "kebab-case")]
 pub enum UsernameFormat {
