@@ -87,15 +87,11 @@ pub struct Server {
         deserialize_with = "deserialize_duration_from_u64"
     )]
     pub who_poll_interval: Duration,
-    /// WHO retry interval for servers without away-notify.
-    #[serde(
-        default = "default_who_retry_interval",
-        deserialize_with = "deserialize_duration_from_u64"
-    )]
-    pub who_retry_interval: Duration,
     /// A list of nicknames to monitor (if MONITOR is supported by the server).
     #[serde(default)]
     pub monitor: Vec<String>,
+    #[serde(default = "default_chathistory")]
+    pub chathistory: bool,
 }
 
 impl Server {
@@ -173,8 +169,8 @@ impl Default for Server {
             on_connect: Default::default(),
             who_poll_enabled: default_who_poll_enabled(),
             who_poll_interval: default_who_poll_interval(),
-            who_retry_interval: default_who_retry_interval(),
             monitor: Default::default(),
+            chathistory: default_chathistory(),
         }
     }
 }
@@ -215,7 +211,9 @@ impl Sasl {
         }
     }
 
-    pub fn param(&self) -> String {
+    pub fn params(&self) -> Vec<String> {
+        const CHUNK_SIZE: usize = 400;
+
         match self {
             Sasl::Plain {
                 username, password, ..
@@ -226,10 +224,35 @@ impl Sasl {
                     .as_ref()
                     .expect("SASL password must exist at this point!");
 
-                base64::engine::general_purpose::STANDARD
-                    .encode(format!("{username}\x00{username}\x00{password}"))
+                // Exclude authorization ID, to use the authentication ID as the authorization ID
+                // https://datatracker.ietf.org/doc/html/rfc4616#section-2
+                let encoding = base64::engine::general_purpose::STANDARD
+                    .encode(format!("\x00{username}\x00{password}"));
+
+                let chunks = encoding
+                    .as_bytes()
+                    .chunks(CHUNK_SIZE)
+                    .collect::<Vec<&[u8]>>();
+
+                let signal_end_of_response = chunks
+                    .iter()
+                    .last()
+                    .is_none_or(|chunk| chunk.len() == CHUNK_SIZE);
+
+                let mut params = chunks
+                    .into_iter()
+                    .map(|chunk| {
+                        String::from_utf8(chunk.into()).expect("chunks should be valid UTF-8")
+                    })
+                    .collect::<Vec<String>>();
+
+                if signal_end_of_response {
+                    params.push("+".into())
+                }
+
+                params
             }
-            Sasl::External { .. } => "+".into(),
+            Sasl::External { .. } => vec!["+".into()],
         }
     }
 
@@ -255,7 +278,7 @@ where
     D: Deserializer<'de>,
 {
     let seconds: u64 = Deserialize::deserialize(deserializer)?;
-    Ok(Duration::from_secs(seconds.clamp(5, 3600)))
+    Ok(Duration::from_secs(seconds.clamp(1, 3600)))
 }
 
 fn default_use_tls() -> bool {
@@ -291,9 +314,9 @@ fn default_who_poll_enabled() -> bool {
 }
 
 fn default_who_poll_interval() -> Duration {
-    Duration::from_secs(180)
+    Duration::from_secs(2)
 }
 
-fn default_who_retry_interval() -> Duration {
-    Duration::from_secs(10)
+fn default_chathistory() -> bool {
+    true
 }
