@@ -1,15 +1,9 @@
 use serde::Deserialize;
 
-use crate::dashboard::{BufferAction, BufferFocusedAction};
-
 #[derive(Debug, Copy, Clone, Deserialize)]
 pub struct Sidebar {
-    #[serde(default, alias = "default_action")]
-    pub buffer_action: BufferAction,
     #[serde(default)]
-    pub buffer_focused_action: Option<BufferFocusedAction>,
-    #[serde(default = "default_sidebar_width")]
-    pub width: u16,
+    pub max_width: Option<u16>,
     #[serde(default)]
     pub unread_indicator: UnreadIndicator,
     #[serde(default)]
@@ -49,9 +43,7 @@ impl Position {
 impl Default for Sidebar {
     fn default() -> Self {
         Sidebar {
-            buffer_action: Default::default(),
-            buffer_focused_action: Default::default(),
-            width: default_sidebar_width(),
+            max_width: None,
             unread_indicator: UnreadIndicator::default(),
             position: Position::default(),
             show_user_menu: default_bool_true(),
@@ -59,10 +51,6 @@ impl Default for Sidebar {
     }
 }
 
-fn default_sidebar_width() -> u16 {
-    120
-}
-
 fn default_bool_true() -> bool {
     true
 }
