@@ -1,9 +1,12 @@
+use std::collections::HashMap;
 use std::io;
 use std::path::PathBuf;
 
 use serde::{Deserialize, Serialize};
 
+use crate::buffer::{self, Buffer};
 use crate::pane::Pane;
+use crate::serde::fail_as_none;
 use crate::{compression, environment};
 
 #[derive(Debug, Clone, Serialize, Deserialize)]
@@ -11,6 +14,29 @@ pub struct Dashboard {
     pub pane: Pane,
     #[serde(default)]
     pub popout_panes: Vec<Pane>,
+    #[serde(default)]
+    pub buffer_settings: BufferSettings,
+    #[serde(default, deserialize_with = "fail_as_none")]
+    pub focus_buffer: Option<Buffer>,
+}
+
+#[derive(Debug, Clone, Serialize, Deserialize, Default)]
+pub struct BufferSettings(HashMap<String, buffer::Settings>);
+
+impl BufferSettings {
+    pub fn get(&self, buffer: &buffer::Buffer) -> Option<&buffer::Settings> {
+        self.0.get(&buffer.key())
+    }
+
+    pub fn entry(
+        &mut self,
+        buffer: &buffer::Buffer,
+        maybe_default: Option<buffer::Settings>,
+    ) -> &mut buffer::Settings {
+        self.0
+            .entry(buffer.key())
+            .or_insert_with(|| maybe_default.unwrap_or_default())
+    }
 }
 
 #[derive(Debug, Clone, Copy, Default, Deserialize, Serialize)]
