@@ -1,3 +1,4 @@
+use std::cmp::Ordering;
 use std::path::PathBuf;
 use std::time::Duration;
 use std::{fmt, io};
@@ -5,12 +6,12 @@ use std::{fmt, io};
 use chrono::{DateTime, Utc};
 use futures::future::BoxFuture;
 use futures::{Future, FutureExt};
-use irc::proto;
 use tokio::fs;
 use tokio::time::Instant;
 
-use crate::user::Nick;
-use crate::{buffer, compression, environment, message, Buffer, Message, Server};
+use crate::message::{self, MessageReferences};
+use crate::target::{self, Target};
+use crate::{buffer, compression, environment, isupport, Buffer, Message, Server};
 
 pub use self::manager::{Manager, Resource};
 pub use self::metadata::{Metadata, ReadMarker};
@@ -29,21 +30,33 @@ const FLUSH_AFTER_LAST_RECEIVED: Duration = Duration::from_secs(5);
 #[derive(Debug, Clone, PartialEq, Eq, Hash)]
 pub enum Kind {
     Server(Server),
-    Channel(Server, String),
-    Query(Server, Nick),
+    Channel(Server, target::Channel),
+    Query(Server, target::Query),
     Logs,
     Highlights,
 }
 
 impl Kind {
-    pub fn from_target(server: Server, target: String) -> Self {
-        if proto::is_channel(&target) {
-            Self::Channel(server, target)
-        } else {
-            Self::Query(server, Nick::from(target))
+    pub fn from_target(server: Server, target: Target) -> Self {
+        match target {
+            Target::Channel(channel) => Self::Channel(server, channel),
+            Target::Query(query) => Self::Query(server, query),
         }
     }
 
+    pub fn from_str(
+        server: Server,
+        chantypes: &[char],
+        statusmsg: &[char],
+        casemapping: isupport::CaseMap,
+        target: &str,
+    ) -> Self {
+        Self::from_target(
+            server,
+            Target::parse(target, chantypes, statusmsg, casemapping),
+        )
+    }
+
     pub fn from_input_buffer(buffer: buffer::Upstream) -> Self {
         match buffer {
             buffer::Upstream::Server(server) => Self::Server(server),
@@ -58,7 +71,7 @@ impl Kind {
             message::Target::Channel { channel, .. } => {
                 Some(Self::Channel(server, channel.clone()))
             }
-            message::Target::Query { nick, .. } => Some(Self::Query(server, nick.clone())),
+            message::Target::Query { query, .. } => Some(Self::Query(server, query.clone())),
             message::Target::Logs => None,
             message::Target::Highlights { .. } => None,
         }
@@ -79,8 +92,8 @@ impl Kind {
     pub fn target(&self) -> Option<&str> {
         match self {
             Kind::Server(_) => None,
-            Kind::Channel(_, channel) => Some(channel),
-            Kind::Query(_, nick) => Some(nick.as_ref()),
+            Kind::Channel(_, channel) => Some(channel.as_str()),
+            Kind::Query(_, nick) => Some(nick.as_str()),
             Kind::Logs => None,
             Kind::Highlights => None,
         }
@@ -157,7 +170,9 @@ pub async fn append(
     let loaded = load(kind.clone()).await?;
 
     let mut all_messages = loaded.messages;
-    all_messages.extend(messages);
+    messages.into_iter().for_each(|message| {
+        insert_message(&mut all_messages, message);
+    });
 
     overwrite(kind, &all_messages, read_marker).await
 }
@@ -184,8 +199,8 @@ async fn path(kind: &Kind) -> Result<PathBuf, Error> {
 
     let name = match kind {
         Kind::Server(server) => format!("{server}"),
-        Kind::Channel(server, channel) => format!("{server}channel{channel}"),
-        Kind::Query(server, nick) => format!("{server}nickname{}", nick),
+        Kind::Channel(server, channel) => format!("{server}channel{}", channel.as_normalized_str()),
+        Kind::Query(server, query) => format!("{server}nickname{}", query.as_normalized_str()),
         Kind::Logs => "logs".to_string(),
         Kind::Highlights => "highlights".to_string(),
     };
@@ -203,6 +218,7 @@ pub enum History {
         last_updated_at: Option<Instant>,
         max_triggers_unread: Option<DateTime<Utc>>,
         read_marker: Option<ReadMarker>,
+        chathistory_references: Option<MessageReferences>,
     },
     Full {
         kind: Kind,
@@ -220,6 +236,7 @@ impl History {
             last_updated_at: None,
             max_triggers_unread: None,
             read_marker: None,
+            chathistory_references: None,
         }
     }
 
@@ -227,11 +244,15 @@ impl History {
         if let Self::Partial {
             max_triggers_unread,
             read_marker,
+            chathistory_references,
             ..
         } = self
         {
             *read_marker = (*read_marker).max(metadata.read_marker);
             *max_triggers_unread = (*max_triggers_unread).max(metadata.last_triggers_unread);
+            *chathistory_references = chathistory_references
+                .clone()
+                .max(metadata.chathistory_references);
         }
     }
 
@@ -279,7 +300,7 @@ impl History {
             } => {
                 *last_updated_at = Some(Instant::now());
 
-                messages.push(message);
+                insert_message(messages, message);
             }
         }
     }
@@ -343,7 +364,9 @@ impl History {
         }
     }
 
-    fn make_partial(&mut self) -> Option<impl Future<Output = Result<Option<ReadMarker>, Error>>> {
+    fn make_partial(
+        &mut self,
+    ) -> Option<impl Future<Output = Result<Option<ReadMarker>, Error>> + use<>> {
         match self {
             History::Partial { .. } => None,
             History::Full {
@@ -357,6 +380,7 @@ impl History {
 
                 let read_marker = ReadMarker::latest(&messages).max(*read_marker);
                 let max_triggers_unread = metadata::latest_triggers_unread(&messages);
+                let chathistory_references = metadata::latest_can_reference(&messages);
 
                 *self = Self::Partial {
                     kind: kind.clone(),
@@ -364,6 +388,7 @@ impl History {
                     last_updated_at: None,
                     read_marker,
                     max_triggers_unread,
+                    chathistory_references,
                 };
 
                 Some(async move {
@@ -402,6 +427,48 @@ impl History {
         }
     }
 
+    pub fn first_can_reference(&self) -> Option<&Message> {
+        match self {
+            History::Partial { messages, .. } | History::Full { messages, .. } => {
+                messages.iter().find(|message| message.can_reference())
+            }
+        }
+    }
+
+    pub fn last_can_reference_before(
+        &self,
+        server_time: DateTime<Utc>,
+    ) -> Option<MessageReferences> {
+        match self {
+            History::Partial {
+                messages,
+                chathistory_references,
+                ..
+            } => messages
+                .iter()
+                .rev()
+                .find(|message| message.can_reference() && message.server_time < server_time)
+                .map_or(
+                    if chathistory_references
+                        .as_ref()
+                        .is_some_and(|chathistory_references| {
+                            chathistory_references.timestamp < server_time
+                        })
+                    {
+                        chathistory_references.clone()
+                    } else {
+                        None
+                    },
+                    |message| Some(message.references()),
+                ),
+            History::Full { messages, .. } => messages
+                .iter()
+                .rev()
+                .find(|message| message.can_reference() && message.server_time < server_time)
+                .map(|message| message.references()),
+        }
+    }
+
     pub fn update_read_marker(&mut self, read_marker: ReadMarker) {
         let stored = match self {
             History::Partial { read_marker, .. } => read_marker,
@@ -418,11 +485,135 @@ impl History {
             }
         }
     }
+
+    pub fn hide_preview(&mut self, message: message::Hash, url: url::Url) {
+        if let Self::Full {
+            messages,
+            last_updated_at,
+            ..
+        } = self
+        {
+            if let Some(message) = messages.iter_mut().find(|m| m.hash == message) {
+                message.hidden_urls.insert(url);
+
+                *last_updated_at = Some(Instant::now());
+            }
+        }
+    }
+}
+
+/// Insert the incoming message into the provided vector, sorted
+/// on server time
+///
+/// Deduplication is only checked +/- 1 second around the server time
+/// of the incoming message. Either message IDs match, or server times
+/// have an exact match + target & content.
+pub fn insert_message(messages: &mut Vec<Message>, message: Message) {
+    let fuzz_seconds =
+        if matches!(message.direction, message::Direction::Received) && message.is_echo {
+            chrono::Duration::seconds(300)
+        } else {
+            chrono::Duration::seconds(1)
+        };
+
+    if messages.is_empty() {
+        messages.push(message);
+
+        return;
+    }
+
+    let start = message.server_time - fuzz_seconds;
+    let end = message.server_time + fuzz_seconds;
+
+    let start_index = match messages.binary_search_by(|stored| stored.server_time.cmp(&start)) {
+        Ok(match_index) => match_index,
+        Err(sorted_insert_index) => sorted_insert_index,
+    };
+    let end_index = match messages.binary_search_by(|stored| stored.server_time.cmp(&end)) {
+        Ok(match_index) => match_index,
+        Err(sorted_insert_index) => sorted_insert_index,
+    };
+
+    let mut current_index = start_index;
+    let mut insert_at = start_index;
+    let mut replace_at = None;
+
+    for stored in &messages[start_index..end_index] {
+        if replace_at.is_none()
+            && ((message.id.is_some() && stored.id == message.id)
+                || ((stored.server_time == message.server_time
+                    || (matches!(stored.direction, message::Direction::Sent)
+                        && matches!(message.direction, message::Direction::Received)
+                        && message.is_echo))
+                    && has_matching_content(stored, &message)))
+        {
+            replace_at = Some(current_index);
+        }
+
+        if message.server_time >= stored.server_time {
+            insert_at = current_index + 1;
+        }
+
+        current_index += 1;
+    }
+
+    if let Some(index) = replace_at {
+        if messages[index].server_time == message.server_time {
+            if has_matching_content(&messages[index], &message) {
+                messages[index].id = message.id;
+                messages[index].received_at = message.received_at;
+            } else {
+                messages[index] = message;
+            }
+        } else {
+            match insert_at.cmp(&index) {
+                Ordering::Less => {
+                    messages.remove(index);
+                    messages.insert(insert_at, message);
+                }
+                Ordering::Equal => messages[index] = message,
+                Ordering::Greater => {
+                    messages.insert(insert_at, message);
+                    messages.remove(index);
+                }
+            }
+        }
+    } else {
+        messages.insert(insert_at, message);
+    }
+}
+
+/// The content of JOIN, PART, and QUIT messages may be dependent on how
+/// the user attributes are resolved.  Match those messages based on Nick
+/// alone (covered by comparing target components) to avoid false negatives.
+fn has_matching_content(message: &Message, other: &Message) -> bool {
+    if message.target == other.target {
+        if let message::Source::Server(Some(source)) = message.target.source() {
+            match source.kind() {
+                message::source::server::Kind::Join
+                | message::source::server::Kind::Part
+                | message::source::server::Kind::Quit => {
+                    return true;
+                }
+                message::source::server::Kind::ReplyTopic
+                | message::source::server::Kind::ChangeHost
+                | message::source::server::Kind::MonitoredOnline
+                | message::source::server::Kind::MonitoredOffline
+                | message::source::server::Kind::StandardReply(_) => (),
+            }
+        }
+
+        message.content == other.content
+    } else {
+        false
+    }
 }
 
 #[derive(Debug)]
 pub struct View<'a> {
     pub total: usize,
+    pub has_more_older_messages: bool,
+    pub has_more_newer_messages: bool,
     pub old_messages: Vec<&'a Message>,
     pub new_messages: Vec<&'a Message>,
     pub max_nick_chars: Option<usize>,
