@@ -5,10 +5,11 @@ use futures::future::BoxFuture;
 use futures::{future, Future, FutureExt};
 use tokio::time::Instant;
 
-use crate::history::{self, History};
+use crate::history::{self, History, MessageReferences};
 use crate::message::{self, Limit};
+use crate::target::{self, Target};
 use crate::user::Nick;
-use crate::{buffer, config, input};
+use crate::{buffer, config, input, isupport};
 use crate::{server, Config, Input, Server, User};
 
 #[derive(Debug, Clone, PartialEq, Eq, Hash)]
@@ -161,13 +162,13 @@ impl Manager {
         self.data.flush_all(now)
     }
 
-    pub fn close(&mut self, kind: history::Kind) -> Option<impl Future<Output = Message>> {
+    pub fn close(&mut self, kind: history::Kind) -> Option<impl Future<Output = Message> + use<>> {
         let history = self.data.map.remove(&kind)?;
 
         Some(history.close().map(|result| Message::Closed(kind, result)))
     }
 
-    pub fn exit(&mut self) -> impl Future<Output = Message> {
+    pub fn exit(&mut self) -> impl Future<Output = Message> + use<> {
         let map = std::mem::take(&mut self.data).map;
 
         async move {
@@ -179,27 +180,38 @@ impl Manager {
         }
     }
 
-    pub fn record_input(
+    pub fn record_input_message(
         &mut self,
         input: Input,
         user: User,
         channel_users: &[User],
-    ) -> Vec<impl Future<Output = Message>> {
+        chantypes: &[char],
+        statusmsg: &[char],
+        casemapping: isupport::CaseMap,
+        config: &Config,
+    ) -> Vec<impl Future<Output = Message> + use<>> {
         let mut tasks = vec![];
 
-        if let Some(messages) = input.messages(user, channel_users) {
+        if let Some(messages) = input.messages(
+            user,
+            channel_users,
+            chantypes,
+            statusmsg,
+            casemapping,
+            config,
+        ) {
             for message in messages {
                 tasks.extend(self.record_message(input.server(), message));
             }
         }
 
-        if let Some(text) = input.raw() {
-            self.data.input.record(&input.buffer, text.to_string());
-        }
-
         tasks
     }
 
+    pub fn record_input_history(&mut self, buffer: &buffer::Upstream, text: String) {
+        self.data.input.record(buffer, text);
+    }
+
     pub fn record_draft(&mut self, draft: input::Draft) {
         self.data.input.store_draft(draft);
     }
@@ -208,7 +220,7 @@ impl Manager {
         &mut self,
         server: &Server,
         message: crate::Message,
-    ) -> Option<impl Future<Output = Message>> {
+    ) -> Option<impl Future<Output = Message> + use<>> {
         history::Kind::from_server_message(server.clone(), &message)
             .and_then(|kind| self.data.add_message(kind, message))
     }
@@ -216,7 +228,7 @@ impl Manager {
     pub fn record_log(
         &mut self,
         record: crate::log::Record,
-    ) -> Option<impl Future<Output = Message>> {
+    ) -> Option<impl Future<Output = Message> + use<>> {
         self.data
             .add_message(history::Kind::Logs, crate::Message::log(record))
     }
@@ -224,24 +236,38 @@ impl Manager {
     pub fn record_highlight(
         &mut self,
         message: crate::Message,
-    ) -> Option<impl Future<Output = Message>> {
+    ) -> Option<impl Future<Output = Message> + use<>> {
         self.data.add_message(history::Kind::Highlights, message)
     }
 
-    pub fn update_read_marker(
+    pub fn update_read_marker<T: Into<history::Kind>>(
         &mut self,
-        kind: impl Into<history::Kind>,
+        kind: T,
         read_marker: history::ReadMarker,
-    ) -> Option<impl Future<Output = Message>> {
+    ) -> Option<impl Future<Output = Message> + use<T>> {
         self.data.update_read_marker(kind, read_marker)
     }
 
-    pub fn channel_joined(
+    pub fn load_metadata(
         &mut self,
         server: Server,
-        channel: String,
-    ) -> Option<impl Future<Output = Message>> {
-        self.data.channel_joined(server, channel)
+        target: Target,
+    ) -> Option<impl Future<Output = Message> + use<>> {
+        self.data.load_metadata(server, target)
+    }
+
+    pub fn first_can_reference(&self, server: Server, target: Target) -> Option<&crate::Message> {
+        self.data.first_can_reference(server, target)
+    }
+
+    pub fn last_can_reference_before(
+        &self,
+        server: Server,
+        target: Target,
+        server_time: DateTime<Utc>,
+    ) -> Option<MessageReferences> {
+        self.data
+            .last_can_reference_before(server, target, server_time)
     }
 
     pub fn get_messages(
@@ -253,13 +279,13 @@ impl Manager {
         self.data.history_view(kind, limit, buffer_config)
     }
 
-    pub fn get_unique_queries(&self, server: &Server) -> Vec<&Nick> {
+    pub fn get_unique_queries(&self, server: &Server) -> Vec<&target::Query> {
         let queries = self
             .data
             .map
             .keys()
             .filter_map(|kind| match kind {
-                history::Kind::Query(s, user) => (s == server).then_some(user),
+                history::Kind::Query(s, query) => (s == server).then_some(query),
                 _ => None,
             })
             .collect::<Vec<_>>();
@@ -289,7 +315,7 @@ impl Manager {
         broadcast: Broadcast,
         config: &Config,
         sent_time: DateTime<Utc>,
-    ) -> Vec<impl Future<Output = Message>> {
+    ) -> Vec<impl Future<Output = Message> + use<>> {
         let channels = self
             .data
             .map
@@ -330,7 +356,7 @@ impl Manager {
                 comment,
                 user_channels,
             } => {
-                let user_query = queries.find(|nick| user.nickname() == *nick);
+                let user_query = queries.find(|query| user.as_str() == query.as_str());
 
                 message::broadcast::quit(
                     user_channels,
@@ -359,7 +385,7 @@ impl Manager {
                     )
                 } else {
                     // Otherwise just the query channel of the user w/ nick change
-                    let user_query = queries.find(|nick| old_nick == *nick);
+                    let user_query = queries.find(|query| old_nick.as_ref() == query.as_str());
                     message::broadcast::nickname(
                         user_channels,
                         user_query,
@@ -380,6 +406,7 @@ impl Manager {
                 new_username,
                 new_hostname,
                 ourself,
+                logged_in,
                 user_channels,
             } => {
                 if ourself {
@@ -391,11 +418,12 @@ impl Manager {
                         &new_username,
                         &new_hostname,
                         ourself,
+                        logged_in,
                         sent_time,
                     )
                 } else {
                     // Otherwise just the query channel of the user w/ host change
-                    let user_query = queries.find(|nick| old_user.nickname() == *nick);
+                    let user_query = queries.find(|query| old_user.as_str() == query.as_str());
                     message::broadcast::change_host(
                         user_channels,
                         user_query,
@@ -403,6 +431,7 @@ impl Manager {
                         &new_username,
                         &new_hostname,
                         ourself,
+                        logged_in,
                         sent_time,
                     )
                 }
@@ -418,6 +447,15 @@ impl Manager {
     pub fn input<'a>(&'a self, buffer: &buffer::Upstream) -> input::Cache<'a> {
         self.data.input.get(buffer)
     }
+
+    pub fn hide_preview(
+        &mut self,
+        kind: impl Into<history::Kind>,
+        message: message::Hash,
+        url: url::Url,
+    ) {
+        self.data.hide_preview(&kind.into(), message, url);
+    }
 }
 
 fn with_limit<'a>(
@@ -432,7 +470,7 @@ fn with_limit<'a>(
             collected[length.saturating_sub(n)..length].to_vec()
         }
         Some(Limit::Since(timestamp)) => messages
-            .skip_while(|message| message.received_at < timestamp)
+            .skip_while(|message| message.server_time < timestamp)
             .collect(),
         None => messages.collect(),
     }
@@ -464,7 +502,11 @@ impl Data {
                     let read_marker = (*partial_read_marker).max(metadata.read_marker);
 
                     let last_updated_at = *last_updated_at;
-                    messages.extend(std::mem::take(new_messages));
+                    std::mem::take(new_messages)
+                        .into_iter()
+                        .for_each(|message| {
+                            history::insert_message(&mut messages, message);
+                        });
                     entry.insert(History::Full {
                         kind,
                         messages,
@@ -522,7 +564,7 @@ impl Data {
                     if let Some(server_message) = buffer_config.server_messages.get(source) {
                         // Check if target is a channel, and if included/excluded.
                         if let message::Target::Channel { channel, .. } = &message.target {
-                            if !server_message.should_send_message(channel.as_ref()) {
+                            if !server_message.should_send_message(channel.as_str()) {
                                 return false;
                             }
                         }
@@ -603,11 +645,11 @@ impl Data {
                 filtered
                     .iter()
                     .filter_map(|message| {
-                        message.target.prefix().map(|prefix| {
+                        message.target.prefixes().map(|prefixes| {
                             buffer_config
                                 .status_message_prefix
                                 .brackets
-                                .format(prefix)
+                                .format(prefixes.iter().collect::<String>())
                                 .chars()
                                 .count()
                                 + 1
@@ -628,8 +670,14 @@ impl Data {
             })
             .unwrap_or_default();
 
+        let first_without_limit = filtered.first().copied();
+        let last_without_limit = filtered.last().copied();
+
         let limited = with_limit(limit, filtered.into_iter());
 
+        let first_with_limit = limited.first();
+        let last_with_limit = limited.last();
+
         let split_at = read_marker.map_or(0, |read_marker| {
             limited
                 .iter()
@@ -650,8 +698,23 @@ impl Data {
 
         let (old, new) = limited.split_at(split_at);
 
+        let has_more_older_messages =
+            first_without_limit
+                .zip(first_with_limit)
+                .is_some_and(|(without_limit, with_limit)| {
+                    without_limit.server_time < with_limit.server_time
+                });
+        let has_more_newer_messages =
+            last_without_limit
+                .zip(last_with_limit)
+                .is_some_and(|(without_limit, with_limit)| {
+                    without_limit.server_time > with_limit.server_time
+                });
+
         Some(history::View {
             total,
+            has_more_older_messages,
+            has_more_newer_messages,
             old_messages: old.to_vec(),
             new_messages: new.to_vec(),
             max_nick_chars,
@@ -663,7 +726,7 @@ impl Data {
         &mut self,
         kind: history::Kind,
         message: crate::Message,
-    ) -> Option<impl Future<Output = Message>> {
+    ) -> Option<impl Future<Output = Message> + use<>> {
         use std::collections::hash_map;
 
         match self.map.entry(kind.clone()) {
@@ -689,11 +752,11 @@ impl Data {
         }
     }
 
-    fn update_read_marker(
+    fn update_read_marker<T: Into<history::Kind>>(
         &mut self,
-        kind: impl Into<history::Kind>,
+        kind: T,
         read_marker: history::ReadMarker,
-    ) -> Option<impl Future<Output = Message>> {
+    ) -> Option<impl Future<Output = Message> + use<T>> {
         use std::collections::hash_map;
 
         let kind = kind.into();
@@ -715,14 +778,14 @@ impl Data {
         }
     }
 
-    fn channel_joined(
+    fn load_metadata(
         &mut self,
         server: server::Server,
-        channel: String,
-    ) -> Option<impl Future<Output = Message>> {
+        target: Target,
+    ) -> Option<impl Future<Output = Message> + use<>> {
         use std::collections::hash_map;
 
-        let kind = history::Kind::Channel(server, channel);
+        let kind = history::Kind::from_target(server, target);
 
         match self.map.entry(kind.clone()) {
             hash_map::Entry::Occupied(_) => None,
@@ -741,10 +804,36 @@ impl Data {
         }
     }
 
+    fn first_can_reference(
+        &self,
+        server: server::Server,
+        target: Target,
+    ) -> Option<&crate::Message> {
+        let kind = history::Kind::from_target(server, target);
+
+        self.map
+            .get(&kind)
+            .and_then(|history| history.first_can_reference())
+    }
+
+    fn last_can_reference_before(
+        &self,
+        server: Server,
+        target: Target,
+        server_time: DateTime<Utc>,
+    ) -> Option<MessageReferences> {
+        let kind = history::Kind::from_target(server, target);
+
+        self.map
+            .get(&kind)
+            .and_then(|history| history.last_can_reference_before(server_time))
+    }
+
     fn untrack(
         &mut self,
         kind: &history::Kind,
-    ) -> Option<impl Future<Output = Result<Option<history::ReadMarker>, history::Error>>> {
+    ) -> Option<impl Future<Output = Result<Option<history::ReadMarker>, history::Error>> + use<>>
+    {
         self.map.get_mut(kind).and_then(History::make_partial)
     }
 
@@ -761,6 +850,12 @@ impl Data {
             })
             .collect()
     }
+
+    fn hide_preview(&mut self, kind: &history::Kind, message: message::Hash, url: url::Url) {
+        if let Some(history) = self.map.get_mut(kind) {
+            history.hide_preview(message, url);
+        }
+    }
 }
 
 fn smart_filter_message(
@@ -804,24 +899,25 @@ pub enum Broadcast {
     Quit {
         user: User,
         comment: Option<String>,
-        user_channels: Vec<String>,
+        user_channels: Vec<target::Channel>,
     },
     Nickname {
         old_nick: Nick,
         new_nick: Nick,
         ourself: bool,
-        user_channels: Vec<String>,
+        user_channels: Vec<target::Channel>,
     },
     Invite {
         inviter: Nick,
-        channel: String,
-        user_channels: Vec<String>,
+        channel: target::Channel,
+        user_channels: Vec<target::Channel>,
     },
     ChangeHost {
         old_user: User,
         new_username: String,
         new_hostname: String,
         ourself: bool,
-        user_channels: Vec<String>,
+        logged_in: bool,
+        user_channels: Vec<target::Channel>,
     },
 }
