@@ -1,14 +1,22 @@
-use irc::proto;
+use std::collections::HashMap;
+use std::fmt;
 use std::str::FromStr;
 
+use chrono::{format::SecondsFormat, DateTime, Utc};
+
+use crate::{target::Target, Message};
+
 // Utilized ISUPPORT parameters should have an associated Kind enum variant
 // returned by Operation::kind() and Parameter::kind()
 #[allow(non_camel_case_types)]
 #[derive(Clone, Debug, Eq, Hash, PartialEq)]
 pub enum Kind {
     AWAYLEN,
+    CASEMAPPING,
     CHANLIMIT,
     CHANNELLEN,
+    CHANTYPES,
+    CHATHISTORY,
     CNOTICE,
     CPRIVMSG,
     ELIST,
@@ -16,6 +24,8 @@ pub enum Kind {
     KICKLEN,
     KNOCK,
     MONITOR,
+    MSGREFTYPES,
+    NAMELEN,
     NICKLEN,
     SAFELIST,
     STATUSMSG,
@@ -61,7 +71,7 @@ impl FromStr for Operation {
                             }
                         }
                         "AWAYLEN" => Ok(Operation::Add(Parameter::AWAYLEN(
-                            parse_optional_positive_integer(value)?,
+                            parse_required_positive_integer(value)?,
                         ))),
                         "BOT" => Ok(Operation::Add(Parameter::BOT(parse_required_letter(
                             value, None,
@@ -88,23 +98,21 @@ impl FromStr for Operation {
                             value.split(',').for_each(|channel_limit| {
                                 if let Some((prefix, limit)) = channel_limit.split_once(':') {
                                     if limit.is_empty() {
-                                        prefix.chars().for_each(|c| {
-                                            if proto::CHANNEL_PREFIXES.contains(&c) {
-                                                channel_limits.push(ChannelLimit {
-                                                    prefix: c,
-                                                    limit: None,
-                                                });
-                                            }
-                                        });
+                                        for c in prefix.chars() {
+                                            // TODO validate after STATUSMSG received
+                                            channel_limits.push(ChannelLimit {
+                                                prefix: c,
+                                                limit: None,
+                                            });
+                                        }
                                     } else if let Ok(limit) = limit.parse::<u16>() {
-                                        prefix.chars().for_each(|c| {
-                                            if proto::CHANNEL_PREFIXES.contains(&c) {
-                                                channel_limits.push(ChannelLimit {
-                                                    prefix: c,
-                                                    limit: Some(limit),
-                                                });
-                                            }
-                                        });
+                                        for c in prefix.chars() {
+                                            // TODO validate after STATUSMSG received
+                                            channel_limits.push(ChannelLimit {
+                                                prefix: c,
+                                                limit: Some(limit),
+                                            });
+                                        }
                                     }
                                 }
                             });
@@ -139,19 +147,17 @@ impl FromStr for Operation {
                             parse_required_positive_integer(value)?,
                         ))),
                         "CHANTYPES" => {
-                            if value.is_empty() {
+                            let chars = value.chars().collect::<Vec<_>>();
+                            if chars.is_empty() {
                                 Ok(Operation::Add(Parameter::CHANTYPES(None)))
-                            } else if value.chars().all(|c| proto::CHANNEL_PREFIXES.contains(&c)) {
-                                Ok(Operation::Add(Parameter::CHANTYPES(Some(
-                                    value.to_string(),
-                                ))))
                             } else {
-                                Err("value must only contain channel types if specified")
+                                // TODO validate after STATUSMSG is received
+                                Ok(Operation::Add(Parameter::CHANTYPES(Some(chars))))
                             }
                         }
-                        "CHATHISTORY" => Ok(Operation::Add(Parameter::CHATHISTORY(
-                            parse_required_positive_integer(value)?,
-                        ))),
+                        "CHATHISTORY" | "draft/CHATHISTORY" => Ok(Operation::Add(
+                            Parameter::CHATHISTORY(parse_required_positive_integer(value)?),
+                        )),
                         "CLIENTTAGDENY" => {
                             let mut client_tag_denials = vec![];
 
@@ -310,7 +316,7 @@ impl FromStr for Operation {
                             value.split(',').for_each(|message_reference_type| {
                                 match message_reference_type {
                                     "msgid" => message_reference_types
-                                        .insert(0, MessageReferenceType::MessageID),
+                                        .insert(0, MessageReferenceType::MessageId),
                                     "timestamp" => message_reference_types
                                         .insert(0, MessageReferenceType::Timestamp),
                                     _ => (),
@@ -321,6 +327,9 @@ impl FromStr for Operation {
                                 message_reference_types,
                             )))
                         }
+                        "NAMELEN" => Ok(Operation::Add(Parameter::NAMELEN(
+                            parse_required_positive_integer(value)?,
+                        ))),
                         "NAMESX" => Ok(Operation::Add(Parameter::NAMESX)),
                         "NETWORK" => Ok(Operation::Add(Parameter::NETWORK(value.to_string()))),
                         "NICKLEN" | "MAXNICKLEN" => Ok(Operation::Add(Parameter::NICKLEN(
@@ -331,13 +340,9 @@ impl FromStr for Operation {
                             let mut prefix_maps = vec![];
 
                             if let Some((modes, prefixes)) = value.split_once(')') {
-                                modes.chars().skip(1).zip(prefixes.chars()).for_each(
-                                    |(mode, prefix)| {
-                                        if proto::CHANNEL_MEMBERSHIP_PREFIXES.contains(&prefix) {
-                                            prefix_maps.push(PrefixMap { mode, prefix })
-                                        }
-                                    },
-                                );
+                                for (mode, prefix) in modes.chars().skip(1).zip(prefixes.chars()) {
+                                    prefix_maps.push(PrefixMap { mode, prefix })
+                                }
 
                                 Ok(Operation::Add(Parameter::PREFIX(prefix_maps)))
                             } else {
@@ -350,14 +355,9 @@ impl FromStr for Operation {
                             parse_optional_positive_integer(value)?,
                         ))),
                         "STATUSMSG" => {
-                            if value
-                                .chars()
-                                .all(|c| proto::CHANNEL_MEMBERSHIP_PREFIXES.contains(&c))
-                            {
-                                Ok(Operation::Add(Parameter::STATUSMSG(value.to_string())))
-                            } else {
-                                Err("unknown channel membership prefix(es)")
-                            }
+                            let chars = value.chars().collect::<Vec<_>>();
+                            // TODO validate that STATUSMSG ⊂ PREFIX after ISUPPORT ends
+                            Ok(Operation::Add(Parameter::STATUSMSG(chars)))
                         }
                         "TARGMAX" => {
                             let mut command_target_limits = vec![];
@@ -411,7 +411,7 @@ impl FromStr for Operation {
                     match token {
                         "ACCEPT" => Err("value required"),
                         "ACCOUNTEXTBAN" => Err("value(s) required"),
-                        "AWAYLEN" => Ok(Operation::Add(Parameter::AWAYLEN(None))),
+                        "AWAYLEN" => Err("value required"),
                         "BOT" => Err("value required"),
                         "CALLERID" => Ok(Operation::Add(Parameter::CALLERID(
                             DEFAULT_CALLER_ID_LETTER,
@@ -452,6 +452,7 @@ impl FromStr for Operation {
                         "MONITOR" => Ok(Operation::Add(Parameter::MONITOR(None))),
                         "MSGREFTYPES" => Ok(Operation::Add(Parameter::MSGREFTYPES(vec![]))),
                         "NAMESX" => Ok(Operation::Add(Parameter::NAMESX)),
+                        "NAMELEN" => Err("value required"),
                         "NETWORK" => Err("value required"),
                         "NICKLEN" | "MAXNICKLEN" => Err("value required"),
                         "OVERRIDE" => Ok(Operation::Add(Parameter::OVERRIDE)),
@@ -483,8 +484,11 @@ impl Operation {
             Operation::Add(parameter) => parameter.kind(),
             Operation::Remove(parameter) => match parameter.as_ref() {
                 "AWAYLEN" => Some(Kind::AWAYLEN),
+                "CASEMAPPING" => Some(Kind::CASEMAPPING),
                 "CHANLIMIT" => Some(Kind::CHANLIMIT),
                 "CHANNELLEN" => Some(Kind::CHANNELLEN),
+                "CHANTYPES" => Some(Kind::CHANTYPES),
+                "CHATHISTORY" => Some(Kind::CHATHISTORY),
                 "CNOTICE" => Some(Kind::CNOTICE),
                 "CPRIVMSG" => Some(Kind::CPRIVMSG),
                 "ELIST" => Some(Kind::ELIST),
@@ -492,6 +496,8 @@ impl Operation {
                 "KICKLEN" => Some(Kind::KICKLEN),
                 "KNOCK" => Some(Kind::KNOCK),
                 "MONITOR" => Some(Kind::MONITOR),
+                "MSGREFTYPES" => Some(Kind::MSGREFTYPES),
+                "NAMELEN" => Some(Kind::NAMELEN),
                 "NICKLEN" => Some(Kind::NICKLEN),
                 "SAFELIST" => Some(Kind::SAFELIST),
                 "STATUSMSG" => Some(Kind::STATUSMSG),
@@ -519,14 +525,14 @@ impl Operation {
 pub enum Parameter {
     ACCEPT(u16),
     ACCOUNTEXTBAN(Vec<String>),
-    AWAYLEN(Option<u16>),
+    AWAYLEN(u16),
     BOT(char),
     CALLERID(char),
     CASEMAPPING(CaseMap),
     CHANLIMIT(Vec<ChannelLimit>),
     CHANMODES(Vec<ChannelMode>),
     CHANNELLEN(u16),
-    CHANTYPES(Option<String>),
+    CHANTYPES(Option<Vec<char>>),
     CHATHISTORY(u16),
     CLIENTTAGDENY(Vec<ClientOnlyTags>),
     CLIENTVER(u16, u16),
@@ -555,6 +561,7 @@ pub enum Parameter {
     MODES(Option<u16>),
     MONITOR(Option<u16>),
     MSGREFTYPES(Vec<MessageReferenceType>),
+    NAMELEN(u16),
     NAMESX,
     NETWORK(String),
     NICKLEN(u16),
@@ -563,7 +570,7 @@ pub enum Parameter {
     SAFELIST,
     SECURELIST,
     SILENCE(Option<u16>),
-    STATUSMSG(String),
+    STATUSMSG(Vec<char>),
     TARGMAX(Vec<CommandTargetLimit>),
     TOPICLEN(u16),
     UHNAMES,
@@ -579,8 +586,11 @@ impl Parameter {
     pub fn kind(&self) -> Option<Kind> {
         match self {
             Parameter::AWAYLEN(_) => Some(Kind::AWAYLEN),
+            Parameter::CASEMAPPING(_) => Some(Kind::CASEMAPPING),
             Parameter::CHANLIMIT(_) => Some(Kind::CHANLIMIT),
             Parameter::CHANNELLEN(_) => Some(Kind::CHANNELLEN),
+            Parameter::CHANTYPES(_) => Some(Kind::CHANTYPES),
+            Parameter::CHATHISTORY(_) => Some(Kind::CHATHISTORY),
             Parameter::CNOTICE => Some(Kind::CNOTICE),
             Parameter::CPRIVMSG => Some(Kind::CPRIVMSG),
             Parameter::ELIST(_) => Some(Kind::ELIST),
@@ -588,6 +598,8 @@ impl Parameter {
             Parameter::KICKLEN(_) => Some(Kind::KICKLEN),
             Parameter::KNOCK => Some(Kind::KNOCK),
             Parameter::MONITOR(_) => Some(Kind::MONITOR),
+            Parameter::MSGREFTYPES(_) => Some(Kind::MSGREFTYPES),
+            Parameter::NAMELEN(_) => Some(Kind::NAMELEN),
             Parameter::NICKLEN(_) => Some(Kind::NICKLEN),
             Parameter::SAFELIST => Some(Kind::SAFELIST),
             Parameter::STATUSMSG(_) => Some(Kind::STATUSMSG),
@@ -602,14 +614,95 @@ impl Parameter {
 }
 
 #[allow(non_camel_case_types)]
-#[derive(Clone, Debug)]
+#[derive(Clone, Copy, Debug, Default)]
 pub enum CaseMap {
     ASCII,
     RFC1459,
     RFC1459_STRICT,
+    #[default]
     RFC7613,
 }
 
+impl CaseMap {
+    pub fn normalize(&self, from_str: &str) -> String {
+        match self {
+            CaseMap::ASCII => from_str.to_ascii_lowercase(),
+            CaseMap::RFC1459 => from_str
+                .chars()
+                .map(|c| match c {
+                    'A' => 'a',
+                    'B' => 'b',
+                    'C' => 'c',
+                    'D' => 'd',
+                    'E' => 'e',
+                    'F' => 'f',
+                    'G' => 'g',
+                    'H' => 'h',
+                    'I' => 'i',
+                    'J' => 'j',
+                    'K' => 'k',
+                    'L' => 'l',
+                    'M' => 'm',
+                    'N' => 'n',
+                    'O' => 'o',
+                    'P' => 'p',
+                    'Q' => 'q',
+                    'R' => 'r',
+                    'S' => 's',
+                    'T' => 't',
+                    'U' => 'u',
+                    'V' => 'v',
+                    'W' => 'w',
+                    'X' => 'x',
+                    'Y' => 'y',
+                    'Z' => 'z',
+                    '[' => '{',
+                    ']' => '}',
+                    '\\' => '|',
+                    '~' => '^',
+                    _ => c,
+                })
+                .collect(),
+            CaseMap::RFC1459_STRICT => from_str
+                .chars()
+                .map(|c| match c {
+                    'A' => 'a',
+                    'B' => 'b',
+                    'C' => 'c',
+                    'D' => 'd',
+                    'E' => 'e',
+                    'F' => 'f',
+                    'G' => 'g',
+                    'H' => 'h',
+                    'I' => 'i',
+                    'J' => 'j',
+                    'K' => 'k',
+                    'L' => 'l',
+                    'M' => 'm',
+                    'N' => 'n',
+                    'O' => 'o',
+                    'P' => 'p',
+                    'Q' => 'q',
+                    'R' => 'r',
+                    'S' => 's',
+                    'T' => 't',
+                    'U' => 'u',
+                    'V' => 'v',
+                    'W' => 'w',
+                    'X' => 'x',
+                    'Y' => 'y',
+                    'Z' => 'z',
+                    '[' => '{',
+                    ']' => '}',
+                    '\\' => '|',
+                    _ => c,
+                })
+                .collect(),
+            CaseMap::RFC7613 => from_str.to_lowercase(),
+        }
+    }
+}
+
 #[derive(Clone, Debug)]
 pub struct ChannelLimit {
     pub prefix: char,
@@ -622,6 +715,32 @@ pub struct ChannelMode {
     pub modes: String,
 }
 
+#[derive(Clone, Debug, PartialEq)]
+pub enum ChatHistorySubcommand {
+    Latest(Target, MessageReference, u16),
+    Before(Target, MessageReference, u16),
+    Between(Target, MessageReference, MessageReference, u16),
+    Targets(MessageReference, MessageReference, u16),
+}
+
+impl ChatHistorySubcommand {
+    pub fn target(&self) -> Option<&str> {
+        match self {
+            ChatHistorySubcommand::Latest(target, _, _)
+            | ChatHistorySubcommand::Before(target, _, _)
+            | ChatHistorySubcommand::Between(target, _, _, _) => Some(target.as_str()),
+            ChatHistorySubcommand::Targets(_, _, _) => None,
+        }
+    }
+}
+
+#[derive(Clone, Copy, Debug, PartialEq)]
+pub enum ChatHistoryState {
+    Exhausted,
+    PendingRequest,
+    Ready,
+}
+
 #[derive(Clone, Debug)]
 pub enum ClientOnlyTags {
     Allowed(String),
@@ -635,10 +754,41 @@ pub struct CommandTargetLimit {
     pub limit: Option<u16>,
 }
 
+#[derive(Clone, Debug, PartialEq)]
+pub enum MessageReference {
+    Timestamp(DateTime<Utc>),
+    MessageId(String),
+    None,
+}
+
+impl fmt::Display for MessageReference {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        match self {
+            MessageReference::Timestamp(server_time) => write!(
+                f,
+                "timestamp={}",
+                server_time.to_rfc3339_opts(SecondsFormat::Millis, true)
+            ),
+            MessageReference::MessageId(id) => write!(f, "msgid={id}"),
+            MessageReference::None => write!(f, "*"),
+        }
+    }
+}
+
+impl PartialEq<Message> for MessageReference {
+    fn eq(&self, other: &Message) -> bool {
+        match self {
+            MessageReference::Timestamp(server_time) => other.server_time == *server_time,
+            MessageReference::MessageId(id) => other.id.as_deref() == Some(id.as_str()),
+            MessageReference::None => false,
+        }
+    }
+}
+
 #[derive(Clone, Debug)]
 pub enum MessageReferenceType {
     Timestamp,
-    MessageID,
+    MessageId,
 }
 
 #[derive(Clone, Debug)]
@@ -661,6 +811,54 @@ const DEFAULT_DEAF_LETTER: char = 'D';
 
 const DEFAULT_INVITE_EXCEPTION_LETTER: char = 'I';
 
+const FUZZ_SECONDS: chrono::Duration = chrono::Duration::seconds(5);
+
+pub fn fuzz_start_message_reference(message_reference: MessageReference) -> MessageReference {
+    match message_reference {
+        MessageReference::Timestamp(start_server_time) => {
+            MessageReference::Timestamp(start_server_time - FUZZ_SECONDS)
+        }
+        _ => message_reference,
+    }
+}
+
+pub fn fuzz_end_message_reference(message_reference: MessageReference) -> MessageReference {
+    match message_reference {
+        MessageReference::Timestamp(end_server_time) => {
+            MessageReference::Timestamp(end_server_time + FUZZ_SECONDS)
+        }
+        _ => message_reference,
+    }
+}
+
+pub fn fuzz_message_reference_range(
+    first_message_reference: MessageReference,
+    second_message_reference: MessageReference,
+) -> (MessageReference, MessageReference) {
+    match (
+        first_message_reference.clone(),
+        second_message_reference.clone(),
+    ) {
+        (
+            MessageReference::Timestamp(start_server_time),
+            MessageReference::Timestamp(end_server_time),
+        ) => {
+            if start_server_time < end_server_time {
+                (
+                    fuzz_start_message_reference(first_message_reference),
+                    fuzz_end_message_reference(second_message_reference),
+                )
+            } else {
+                (
+                    fuzz_end_message_reference(first_message_reference),
+                    fuzz_start_message_reference(second_message_reference),
+                )
+            }
+        }
+        _ => (first_message_reference, second_message_reference),
+    }
+}
+
 #[derive(Clone, Copy, Debug, Eq, PartialEq)]
 pub struct WhoToken {
     digits: [char; 3],
@@ -690,9 +888,30 @@ impl FromStr for WhoToken {
     }
 }
 
-pub const WHO_POLL_TOKEN: WhoToken = WhoToken {
-    digits: ['9', '\0', '\0'],
-};
+pub enum WhoXPollParameters {
+    Default,
+    WithAccountName,
+}
+
+impl WhoXPollParameters {
+    pub fn fields(&self) -> &'static str {
+        match self {
+            WhoXPollParameters::Default => "tcnf",
+            WhoXPollParameters::WithAccountName => "tcnfa",
+        }
+    }
+
+    pub fn token(&self) -> WhoToken {
+        match self {
+            WhoXPollParameters::Default => WhoToken {
+                digits: ['9', '\0', '\0'],
+            },
+            WhoXPollParameters::WithAccountName => WhoToken {
+                digits: ['9', '9', '\0'],
+            },
+        }
+    }
+}
 
 fn parse_optional_letters(value: &str) -> Result<Option<String>, &'static str> {
     if value.is_empty() {
@@ -745,3 +964,18 @@ fn parse_required_positive_integer(value: &str) -> Result<u16, &'static str> {
         Err("value required to be a positive integer")
     }
 }
+
+// Returns the limit directly if found, since we currently treat "no target limit specified"
+// the same as "specifying no limit to the number of targets".
+pub fn find_target_limit(isupport: &HashMap<Kind, Parameter>, command: &str) -> Option<u16> {
+    if let Some(Parameter::TARGMAX(target_limits)) = isupport.get(&Kind::TARGMAX) {
+        target_limits
+            .iter()
+            .find_map(|target_limit| {
+                (target_limit.command == command).then_some(target_limit.limit)
+            })
+            .flatten()
+    } else {
+        None
+    }
+}
