@@ -1,11 +1,15 @@
 //! Generate messages that can be broadcast into every buffer
+use std::collections::HashSet;
+
 use chrono::{DateTime, Utc};
 
-use super::{parse_fragments, plain, source, Content, Direction, Message, Source, Target};
+use super::{
+    parse_fragments_with_user, parse_fragments_with_users, plain, source, Content, Direction, Message, Source, Target
+};
 use crate::config::buffer::UsernameFormat;
 use crate::time::Posix;
 use crate::user::Nick;
-use crate::{message, Config, User};
+use crate::{Config, User, message, target};
 
 enum Cause {
     Server(Option<source::Server>),
@@ -13,25 +17,26 @@ enum Cause {
 }
 
 fn expand(
-    channels: impl IntoIterator<Item = String>,
-    queries: impl IntoIterator<Item = Nick>,
+    channels: impl IntoIterator<Item = target::Channel>,
+    queries: impl IntoIterator<Item = target::Query>,
     include_server: bool,
     cause: Cause,
     content: Content,
     sent_time: DateTime<Utc>,
 ) -> Vec<Message> {
     let message = |target, content| -> Message {
-        let received_at = Posix::now();
-        let hash = message::Hash::new(&received_at, &content);
+        let hash = message::Hash::new(&sent_time, &content);
 
         Message {
-            received_at,
+            received_at: Posix::now(),
             server_time: sent_time,
             direction: Direction::Received,
             target,
             content,
             id: None,
             hash,
+            hidden_urls: HashSet::default(),
+            is_echo: false,
         }
     };
 
@@ -45,17 +50,16 @@ fn expand(
         .map(|channel| {
             message(
                 Target::Channel {
-                    channel,
+                    channel: channel.clone(),
                     source: source.clone(),
-                    prefix: None,
                 },
                 content.clone(),
             )
         })
-        .chain(queries.into_iter().map(|nick| {
+        .chain(queries.into_iter().map(|query| {
             message(
                 Target::Query {
-                    nick,
+                    query: query.clone(),
                     source: source.clone(),
                 },
                 content.clone(),
@@ -109,8 +113,8 @@ pub fn connection_failed(error: String, sent_time: DateTime<Utc>) -> Vec<Message
 }
 
 pub fn disconnected(
-    channels: impl IntoIterator<Item = String>,
-    queries: impl IntoIterator<Item = Nick>,
+    channels: impl IntoIterator<Item = target::Channel>,
+    queries: impl IntoIterator<Item = target::Query>,
     error: Option<String>,
     sent_time: DateTime<Utc>,
 ) -> Vec<Message> {
@@ -127,8 +131,8 @@ pub fn disconnected(
 }
 
 pub fn reconnected(
-    channels: impl IntoIterator<Item = String>,
-    queries: impl IntoIterator<Item = Nick>,
+    channels: impl IntoIterator<Item = target::Channel>,
+    queries: impl IntoIterator<Item = target::Query>,
     sent_time: DateTime<Utc>,
 ) -> Vec<Message> {
     let content = plain("connection to server restored".into());
@@ -143,8 +147,8 @@ pub fn reconnected(
 }
 
 pub fn quit(
-    channels: impl IntoIterator<Item = String>,
-    queries: impl IntoIterator<Item = Nick>,
+    channels: impl IntoIterator<Item = target::Channel>,
+    queries: impl IntoIterator<Item = target::Query>,
     user: &User,
     comment: &Option<String>,
     config: &Config,
@@ -155,12 +159,12 @@ pub fn quit(
         .map(|comment| format!(" ({comment})"))
         .unwrap_or_default();
 
-    let content = parse_fragments(
+    let content = parse_fragments_with_user(
         format!(
             "⟵ {} has quit{comment}",
             user.formatted(config.buffer.server_messages.quit.username_format)
         ),
-        &[],
+        user,
     );
 
     expand(
@@ -177,17 +181,20 @@ pub fn quit(
 }
 
 pub fn nickname(
-    channels: impl IntoIterator<Item = String>,
-    queries: impl IntoIterator<Item = Nick>,
+    channels: impl IntoIterator<Item = target::Channel>,
+    queries: impl IntoIterator<Item = target::Query>,
     old_nick: &Nick,
     new_nick: &Nick,
     ourself: bool,
     sent_time: DateTime<Utc>,
 ) -> Vec<Message> {
+    let old_user = User::from(old_nick.clone());
+    let new_user = User::from(new_nick.clone());
+
     let content = if ourself {
-        plain(format!("You're now known as {new_nick}"))
+        parse_fragments_with_user(format!("You're now known as {new_nick}"), &new_user)
     } else {
-        plain(format!("{old_nick} is now known as {new_nick}"))
+        parse_fragments_with_users(format!("{old_nick} is now known as {new_nick}"), &[old_user, new_user])
     };
 
     expand(
@@ -202,22 +209,27 @@ pub fn nickname(
 
 pub fn invite(
     inviter: Nick,
-    channel: String,
-    channels: impl IntoIterator<Item = String>,
+    channel: target::Channel,
+    channels: impl IntoIterator<Item = target::Channel>,
     sent_time: DateTime<Utc>,
 ) -> Vec<Message> {
-    let content = plain(format!("{inviter} invited you to join {channel}"));
+    let inviter = User::from(inviter);
+    let content = parse_fragments_with_user(
+        format!("{} invited you to join {channel}", inviter.nickname()),
+        &inviter,
+    );
 
     expand(channels, [], false, Cause::Server(None), content, sent_time)
 }
 
 pub fn change_host(
-    channels: impl IntoIterator<Item = String>,
-    queries: impl IntoIterator<Item = Nick>,
+    channels: impl IntoIterator<Item = target::Channel>,
+    queries: impl IntoIterator<Item = target::Query>,
     old_user: &User,
     new_username: &str,
     new_hostname: &str,
     ourself: bool,
+    logged_in: bool,
     sent_time: DateTime<Utc>,
 ) -> Vec<Message> {
     let content = if ourself {
@@ -225,21 +237,38 @@ pub fn change_host(
             "You've changed host to {new_username}@{new_hostname}",
         ))
     } else {
-        plain(format!(
-            "{} changed host to {new_username}@{new_hostname}",
-            old_user.formatted(UsernameFormat::Full)
-        ))
+        parse_fragments_with_user(
+            format!(
+                "{} changed host to {new_username}@{new_hostname}",
+                old_user.formatted(UsernameFormat::Full)
+            ),
+            old_user,
+        )
     };
 
-    expand(
-        channels,
-        queries,
-        false,
-        Cause::Server(Some(source::Server::new(
-            source::server::Kind::ChangeHost,
-            Some(old_user.nickname().to_owned()),
-        ))),
-        content,
-        sent_time,
-    )
+    if ourself && !logged_in {
+        expand(
+            [],
+            [],
+            true,
+            Cause::Server(Some(source::Server::new(
+                source::server::Kind::ChangeHost,
+                Some(old_user.nickname().to_owned()),
+            ))),
+            content,
+            sent_time,
+        )
+    } else {
+        expand(
+            channels,
+            queries,
+            false,
+            Cause::Server(Some(source::Server::new(
+                source::server::Kind::ChangeHost,
+                Some(old_user.nickname().to_owned()),
+            ))),
+            content,
+            sent_time,
+        )
+    }
 }
