@@ -4,8 +4,9 @@ use iced_core::color;
 use itertools::PeekingNext;
 use serde::{Deserialize, Serialize};
 
+use crate::appearance::theme;
+
 pub use self::encode::encode;
-use crate::theme;
 
 pub mod encode;
 
@@ -151,7 +152,7 @@ pub fn parse(text: &str) -> Option<Vec<Fragment>> {
     }
 }
 
-#[derive(Debug, Clone, Copy, PartialEq, Eq, Serialize, Deserialize)]
+#[derive(Debug, Clone, Copy, PartialEq, Eq, Hash, Serialize, Deserialize)]
 pub struct Formatting {
     pub bold: bool,
     pub italics: bool,
@@ -232,7 +233,7 @@ impl TryFrom<char> for Modifier {
 }
 
 /// https://modern.ircdocs.horse/formatting.html#colors
-#[derive(Debug, Clone, Copy, PartialEq, Eq, Serialize, Deserialize)]
+#[derive(Debug, Clone, Copy, PartialEq, Eq, Hash, Serialize, Deserialize)]
 #[repr(u8)]
 pub enum Color {
     White = 0,
