@@ -8,7 +8,7 @@ pub use self::server::Server;
 pub enum Source {
     User(User),
     Server(Option<Server>),
-    Action,
+    Action(Option<User>),
     Internal(Internal),
 }
 
@@ -68,6 +68,14 @@ pub mod server {
         ChangeHost,
         MonitoredOnline,
         MonitoredOffline,
+        StandardReply(StandardReply),
+    }
+
+    #[derive(Debug, Clone, Copy, PartialEq, Eq, Hash, Serialize, Deserialize)]
+    pub enum StandardReply {
+        Fail,
+        Warn,
+        Note,
     }
 
     #[derive(Debug, Clone, PartialEq, Eq, Hash, Serialize, Deserialize)]
