@@ -36,12 +36,20 @@ pub enum Command {
     ToggleNicklist,
     ToggleTopic,
     ToggleSidebar,
+    ToggleFullscreen,
     CommandBar,
     ReloadConfiguration,
     FileTransfers,
     Logs,
     ThemeEditor,
     Highlight,
+    QuitApplication,
+    ScrollUpPage,
+    ScrollDownPage,
+    ScrollToTop,
+    ScrollToBottom,
+    CycleNextUnreadBuffer,
+    CyclePreviousUnreadBuffer,
 }
 
 macro_rules! default {
@@ -113,25 +121,33 @@ impl Hash for KeyBind {
 }
 
 impl KeyBind {
-    default!(move_up, ArrowUp, ALT);
-    default!(move_down, ArrowDown, ALT);
-    default!(move_left, ArrowLeft, ALT);
-    default!(move_right, ArrowRight, ALT);
+    default!(move_up, ArrowUp, COMMAND | ALT);
+    default!(move_down, ArrowDown, COMMAND | ALT);
+    default!(move_left, ArrowLeft, COMMAND | ALT);
+    default!(move_right, ArrowRight, COMMAND | ALT);
     default!(close_buffer, "w", COMMAND);
-    default!(maximize_buffer, ArrowUp, COMMAND);
-    default!(restore_buffer, ArrowDown, COMMAND);
+    default!(maximize_buffer, ArrowUp, COMMAND | SHIFT);
+    default!(restore_buffer, ArrowDown, COMMAND | SHIFT);
     default!(cycle_next_buffer, Tab, CTRL);
     default!(cycle_previous_buffer, Tab, CTRL | SHIFT);
     default!(leave_buffer, "w", COMMAND | SHIFT);
     default!(toggle_nick_list, "m", COMMAND | ALT);
     default!(toggle_sidebar, "b", COMMAND | ALT);
     default!(toggle_topic, "t", COMMAND | ALT);
+    default!(toggle_fullscreen, "f", CTRL | SHIFT);
     default!(command_bar, "k", COMMAND);
     default!(reload_configuration, "r", COMMAND);
     default!(file_transfers, "j", COMMAND);
     default!(logs, "l", COMMAND);
     default!(theme_editor, "t", COMMAND);
     default!(highlight, "i", COMMAND);
+    default!(scroll_up_page, PageUp);
+    default!(scroll_down_page, PageDown);
+    // Don't use HOME / END since text input is always focused
+    default!(scroll_to_top, ArrowUp, COMMAND);
+    default!(scroll_to_bottom, ArrowDown, COMMAND);
+    default!(cycle_next_unread_buffer, "`", CTRL);
+    default!(cycle_previous_unread_buffer, "`", CTRL | SHIFT);
 
     pub fn is_pressed(
         &self,
@@ -319,7 +335,7 @@ impl fmt::Display for KeyCode {
             key::Key::Unidentified => String::new(),
         };
 
-        write!(f, "{}", key)
+        write!(f, "{key}")
     }
 }
 
