@@ -1,7 +1,7 @@
 use std::str::FromStr;
 
+use fancy_regex::Regex;
 use log::warn;
-use regex::Regex;
 
 use crate::{appearance::theme, config, Server};
 
@@ -35,6 +35,10 @@ pub fn theme(colors: &theme::Colors) -> String {
     format!("halloy:///theme?e={}", colors.encode_base64())
 }
 
+pub fn theme_submit(colors: &theme::Colors) -> String {
+    format!("https://themes.halloy.chat/submit?e={}", colors.encode_base64())
+}
+
 impl Url {
     pub fn find_in(mut args: impl Iterator<Item = String>) -> Option<Self> {
         args.find_map(|arg| arg.parse().ok())
@@ -90,7 +94,7 @@ fn parse(url: url::Url) -> Result<Url, Error> {
 fn generate_server_name(host: &str) -> &str {
     let pattern = Regex::new(r"irc\.([^.]+)").unwrap();
 
-    if let Some(captures) = pattern.captures(host) {
+    if let Ok(Some(captures)) = pattern.captures(host) {
         if let Some(matched) = captures.get(1) {
             return matched.as_str();
         }
@@ -117,7 +121,7 @@ fn parse_server_config(url: &url::Url) -> Option<config::Server> {
             if channel.starts_with('#') {
                 Some(channel.to_string())
             } else {
-                Some(format!("#{}", channel))
+                Some(format!("#{channel}"))
             }
         };
 
