@@ -2,11 +2,13 @@ use std::collections::HashSet;
 use std::fmt;
 use std::hash::Hash;
 
+use thiserror::Error;
+
 use irc::proto;
 use itertools::sorted;
 use serde::{Deserialize, Serialize};
 
-use crate::{appearance::theme::Colors, buffer, config::buffer::UsernameFormat, mode};
+use crate::{config::buffer::UsernameFormat, mode};
 
 #[derive(Debug, Clone, Serialize, Deserialize)]
 #[serde(into = "String")]
@@ -49,8 +51,16 @@ impl PartialOrd for User {
     }
 }
 
+#[derive(Error, Debug)]
+pub enum TryFromUserError {
+    #[error("nickname can't be empty")]
+    NicknameEmpty,
+    #[error("nickname must start with alphabetic or [ \\ ] ^ _ ` {{ | }} *")]
+    NicknameInvalidCharacter,
+}
+
 impl TryFrom<String> for User {
-    type Error = &'static str;
+    type Error = TryFromUserError;
 
     fn try_from(value: String) -> Result<Self, Self::Error> {
         Self::try_from(value.as_str())
@@ -58,17 +68,17 @@ impl TryFrom<String> for User {
 }
 
 impl<'a> TryFrom<&'a str> for User {
-    type Error = &'static str;
+    type Error = TryFromUserError;
 
     fn try_from(value: &'a str) -> Result<Self, Self::Error> {
         if value.is_empty() {
-            return Err("nickname can't be empty");
+            return Err(Self::Error::NicknameEmpty);
         }
 
         let Some(index) =
             value.find(|c: char| c.is_alphabetic() || "[\\]^_`{|}*".find(c).is_some())
         else {
-            return Err("nickname must start with alphabetic or [ \\ ] ^ _ ` { | } *");
+            return Err(Self::Error::NicknameInvalidCharacter);
         };
 
         let (access_levels, rest) = (&value[..index], &value[index..]);
@@ -145,15 +155,8 @@ impl From<Nick> for User {
 }
 
 impl User {
-    pub fn nick_color(&self, colors: &Colors, kind: buffer::Color) -> NickColor {
-        let color = colors.buffer.nickname;
-        match kind {
-            buffer::Color::Solid => NickColor { seed: None, color },
-            buffer::Color::Unique => NickColor {
-                seed: Some(self.nickname().as_ref().to_string()),
-                color,
-            },
-        }
+    pub fn seed(&self) -> &str {
+        self.as_str()
     }
 
     pub fn display(&self, with_access_levels: bool) -> String {
@@ -313,37 +316,37 @@ impl<'a> From<&'a str> for NickRef<'a> {
     }
 }
 
-impl<'a> NickRef<'a> {
+impl NickRef<'_> {
     pub fn to_owned(self) -> Nick {
         Nick(self.0.to_string())
     }
 }
 
-impl<'a> fmt::Display for NickRef<'a> {
+impl fmt::Display for NickRef<'_> {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         self.0.fmt(f)
     }
 }
 
-impl<'a> AsRef<str> for NickRef<'a> {
+impl AsRef<str> for NickRef<'_> {
     fn as_ref(&self) -> &str {
         self.0
     }
 }
 
-impl<'a> PartialOrd for NickRef<'a> {
+impl PartialOrd for NickRef<'_> {
     fn partial_cmp(&self, other: &Self) -> Option<std::cmp::Ordering> {
         Some(self.0.to_lowercase().cmp(&other.0.to_lowercase()))
     }
 }
 
-impl<'a> Ord for NickRef<'a> {
+impl Ord for NickRef<'_> {
     fn cmp(&self, other: &Self) -> std::cmp::Ordering {
         self.0.to_lowercase().cmp(&other.0.to_lowercase())
     }
 }
 
-impl<'a> PartialEq<Nick> for NickRef<'a> {
+impl PartialEq<Nick> for NickRef<'_> {
     fn eq(&self, other: &Nick) -> bool {
         self.0.eq(other.0.as_str())
     }
@@ -370,7 +373,7 @@ impl std::fmt::Display for AccessLevel {
             AccessLevel::Member => "",
         };
 
-        write!(f, "{}", access_level)
+        write!(f, "{access_level}")
     }
 }
 
