@@ -98,7 +98,10 @@ pub enum Command {
     /// <accountname>
     ACCOUNT(String),
     /* IRC extensions */
+    /// <type> [<params>]
     BATCH(String, Vec<String>),
+    /// <subcommand> [<params>]
+    CHATHISTORY(String, Vec<String>),
     /// <new_username> <new_hostname>
     CHGHOST(String, String),
     /// <nickname> <channel> :<message>
@@ -111,11 +114,21 @@ pub enum Command {
     MARKREAD(String, Option<String>),
     /// <subcommand> [<targets>]
     MONITOR(String, Option<String>),
+    /// <realname>
+    SETNAME(String),
     /// <msgtarget>
     TAGMSG(String),
     /// <nickname>
     USERIP(String),
 
+    /* Standard Replies */
+    /// <command> <code> [<context>] <description>
+    FAIL(String, String, Option<Vec<String>>, String),
+    /// <command> <code> [<context>] <description>
+    WARN(String, String, Option<Vec<String>>, String),
+    /// <command> <code> [<context>] <description>
+    NOTE(String, String, Option<Vec<String>>, String),
+
     Numeric(Numeric, Vec<String>),
     Unknown(String, Vec<String>),
     Raw(String),
@@ -203,14 +216,49 @@ impl Command {
             "WALLOPS" if len > 0 => WALLOPS(req!()),
             "ACCOUNT" if len > 0 => ACCOUNT(req!()),
             "BATCH" if len > 0 => BATCH(req!(), params.collect()),
+            "CHATHISTORY" if len > 0 => CHATHISTORY(req!(), params.collect()),
             "CHGHOST" if len > 1 => CHGHOST(req!(), req!()),
             "CNOTICE" if len > 2 => CNOTICE(req!(), req!(), req!()),
             "CPRIVMSG" if len > 2 => CPRIVMSG(req!(), req!(), req!()),
             "KNOCK" if len > 0 => KNOCK(req!(), opt!()),
             "MARKREAD" if len > 0 => MARKREAD(req!(), opt!()),
             "MONITOR" if len > 0 => MONITOR(req!(), opt!()),
+            "SETNAME" if len > 0 => SETNAME(req!()),
             "TAGMSG" if len > 0 => TAGMSG(req!()),
             "USERIP" if len > 0 => USERIP(req!()),
+            "FAIL" if len > 2 => {
+                let a = req!();
+                let b = req!();
+                let mut c: Vec<String> = params.collect();
+                let d = c.pop().unwrap();
+                if !c.is_empty() {
+                    FAIL(a, b, Some(c), d)
+                } else {
+                    FAIL(a, b, None, d)
+                }
+            }
+            "WARN" if len > 2 => {
+                let a = req!();
+                let b = req!();
+                let mut c: Vec<String> = params.collect();
+                let d = c.pop().unwrap();
+                if !c.is_empty() {
+                    WARN(a, b, Some(c), d)
+                } else {
+                    WARN(a, b, None, d)
+                }
+            }
+            "NOTE" if len > 2 => {
+                let a = req!();
+                let b = req!();
+                let mut c: Vec<String> = params.collect();
+                let d = c.pop().unwrap();
+                if !c.is_empty() {
+                    NOTE(a, b, Some(c), d)
+                } else {
+                    NOTE(a, b, None, d)
+                }
+            }
             _ => Self::Unknown(tag, params.collect()),
         }
     }
@@ -250,7 +298,7 @@ impl Command {
             Command::PRIVMSG(a, b) => vec![a, b],
             Command::NOTICE(a, b) => vec![a, b],
             Command::WHO(a, b, c) => std::iter::once(a)
-                .chain(b.map(|b| c.map_or_else(|| format!("%{}", b), |c| format!("%{},{}", b, c))))
+                .chain(b.map(|b| c.map_or_else(|| format!("%{b}"), |c| format!("%{b},{c}"))))
                 .collect(),
             Command::WHOIS(a, b) => a.into_iter().chain(Some(b)).collect(),
             Command::WHOWAS(a, b) => std::iter::once(a).chain(b).collect(),
@@ -264,14 +312,31 @@ impl Command {
             Command::WALLOPS(a) => vec![a],
             Command::ACCOUNT(a) => vec![a],
             Command::BATCH(a, rest) => std::iter::once(a).chain(rest).collect(),
+            Command::CHATHISTORY(a, b) => std::iter::once(a).chain(b).collect(),
             Command::CHGHOST(a, b) => vec![a, b],
             Command::CNOTICE(a, b, c) => vec![a, b, c],
             Command::CPRIVMSG(a, b, c) => vec![a, b, c],
             Command::KNOCK(a, b) => std::iter::once(a).chain(b).collect(),
             Command::MARKREAD(a, b) => std::iter::once(a).chain(b).collect(),
             Command::MONITOR(a, b) => std::iter::once(a).chain(b).collect(),
+            Command::SETNAME(a) => vec![a],
             Command::TAGMSG(a) => vec![a],
             Command::USERIP(a) => vec![a],
+            Command::FAIL(a, b, c, d) => std::iter::once(a)
+                .chain(Some(b))
+                .chain(c.into_iter().flatten())
+                .chain(Some(d))
+                .collect(),
+            Command::WARN(a, b, c, d) => std::iter::once(a)
+                .chain(Some(b))
+                .chain(c.into_iter().flatten())
+                .chain(Some(d))
+                .collect(),
+            Command::NOTE(a, b, c, d) => std::iter::once(a)
+                .chain(Some(b))
+                .chain(c.into_iter().flatten())
+                .chain(Some(d))
+                .collect(),
             Command::Numeric(_, params) => params,
             Command::Unknown(_, params) => params,
             Command::Raw(_) => vec![],
@@ -324,17 +389,22 @@ impl Command {
             WALLOPS(_) => "WALLOPS".to_string(),
             ACCOUNT(_) => "ACCOUNT".to_string(),
             BATCH(_, _) => "BATCH".to_string(),
+            CHATHISTORY(_, _) => "CHATHISTORY".to_string(),
             CHGHOST(_, _) => "CHGHOST".to_string(),
             CNOTICE(_, _, _) => "CNOTICE".to_string(),
             CPRIVMSG(_, _, _) => "CPRIVMSG".to_string(),
             KNOCK(_, _) => "KNOCK".to_string(),
             MARKREAD(_, _) => "MARKREAD".to_string(),
             MONITOR(_, _) => "MONITOR".to_string(),
+            SETNAME(_) => "SETNAME".to_string(),
             TAGMSG(_) => "TAGMSG".to_string(),
             USERIP(_) => "USERIP".to_string(),
+            FAIL(_, _, _, _) => "FAIL".to_string(),
+            WARN(_, _, _, _) => "WARN".to_string(),
+            NOTE(_, _, _, _) => "NOTE".to_string(),
             Numeric(numeric, _) => format!("{:03}", *numeric as u16),
             Unknown(tag, _) => tag.clone(),
-            Raw(_) => "".to_string(),
+            Raw(_) => String::new(),
         }
     }
 }
