@@ -6,10 +6,10 @@ HALLOY_VERSION=$(cat VERSION)
 scripts/build-windows.sh
 
 # install latest wix
-dotnet tool install --global wix
+dotnet tool install --global wix --version 5.0.2
 
 # add required wix extension
-wix extension add WixToolset.UI.wixext
+wix extension add WixToolset.UI.wixext/5.0.2
 
 # build the installer
 wix build -pdbtype none -arch x64 -d PackageVersion=$HALLOY_VERSION $WXS_FILE -o target/release/halloy-installer.msi -ext WixToolset.UI.wixext
\ No newline at end of file
