@@ -1,6 +1,8 @@
 pub use data::buffer::{Internal, Settings, Upstream};
+use data::dashboard::BufferAction;
+use data::target::{self, Target};
 use data::user::Nick;
-use data::{buffer, file_transfer, history, message, Config};
+use data::{buffer, file_transfer, history, message, preview, Config};
 use iced::Task;
 
 pub use self::channel::Channel;
@@ -24,7 +26,7 @@ mod scroll_view;
 pub mod server;
 pub mod user_context;
 
-#[derive(Clone)]
+#[derive(Clone, Debug)]
 pub enum Buffer {
     Empty,
     Channel(Channel),
@@ -47,9 +49,12 @@ pub enum Message {
 
 pub enum Event {
     UserContext(user_context::Event),
-    OpenChannel(String),
-    GoToMessage(data::Server, String, message::Hash),
+    OpenBuffers(Vec<(Target, BufferAction)>),
+    GoToMessage(data::Server, target::Channel, message::Hash),
     History(Task<history::manager::Message>),
+    RequestOlderChatHistory,
+    PreviewChanged,
+    HidePreview(history::Kind, message::Hash, url::Url),
 }
 
 impl Buffer {
@@ -105,8 +110,13 @@ impl Buffer {
 
                 let event = event.map(|event| match event {
                     channel::Event::UserContext(event) => Event::UserContext(event),
-                    channel::Event::OpenChannel(channel) => Event::OpenChannel(channel),
+                    channel::Event::OpenBuffers(targets) => Event::OpenBuffers(targets),
                     channel::Event::History(task) => Event::History(task),
+                    channel::Event::RequestOlderChatHistory => Event::RequestOlderChatHistory,
+                    channel::Event::PreviewChanged => Event::PreviewChanged,
+                    channel::Event::HidePreview(kind, hash, url) => {
+                        Event::HidePreview(kind, hash, url)
+                    }
                 });
 
                 (command.map(Message::Channel), event)
@@ -116,7 +126,7 @@ impl Buffer {
 
                 let event = event.map(|event| match event {
                     server::Event::UserContext(event) => Event::UserContext(event),
-                    server::Event::OpenChannel(channel) => Event::OpenChannel(channel),
+                    server::Event::OpenBuffers(targets) => Event::OpenBuffers(targets),
                     server::Event::History(task) => Event::History(task),
                 });
 
@@ -127,8 +137,13 @@ impl Buffer {
 
                 let event = event.map(|event| match event {
                     query::Event::UserContext(event) => Event::UserContext(event),
-                    query::Event::OpenChannel(channel) => Event::OpenChannel(channel),
+                    query::Event::OpenBuffers(targets) => Event::OpenBuffers(targets),
                     query::Event::History(task) => Event::History(task),
+                    query::Event::RequestOlderChatHistory => Event::RequestOlderChatHistory,
+                    query::Event::PreviewChanged => Event::PreviewChanged,
+                    query::Event::HidePreview(kind, hash, url) => {
+                        Event::HidePreview(kind, hash, url)
+                    }
                 });
 
                 (command.map(Message::Query), event)
@@ -139,22 +154,26 @@ impl Buffer {
                 (command.map(Message::FileTransfers), None)
             }
             (Buffer::Logs(state), Message::Logs(message)) => {
-                let (command, event) = state.update(message);
+                let (command, event) = state.update(message, history, clients, config);
 
                 let event = event.map(|event| match event {
                     logs::Event::UserContext(event) => Event::UserContext(event),
-                    logs::Event::OpenChannel(channel) => Event::OpenChannel(channel),
+                    logs::Event::OpenBuffer(target, buffer_action) => {
+                        Event::OpenBuffers(vec![(target, buffer_action)])
+                    }
                     logs::Event::History(task) => Event::History(task),
                 });
 
                 (command.map(Message::Logs), event)
             }
             (Buffer::Highlights(state), Message::Highlights(message)) => {
-                let (command, event) = state.update(message);
+                let (command, event) = state.update(message, history, clients, config);
 
                 let event = event.map(|event| match event {
                     highlights::Event::UserContext(event) => Event::UserContext(event),
-                    highlights::Event::OpenChannel(channel) => Event::OpenChannel(channel),
+                    highlights::Event::OpenBuffer(target, buffer_action) => {
+                        Event::OpenBuffers(vec![(target, buffer_action)])
+                    }
                     highlights::Event::GoToMessage(server, channel, message) => {
                         Event::GoToMessage(server, channel, message)
                     }
@@ -172,7 +191,8 @@ impl Buffer {
         clients: &'a data::client::Map,
         file_transfers: &'a file_transfer::Manager,
         history: &'a history::Manager,
-        settings: &'a buffer::Settings,
+        previews: &'a preview::Collection,
+        settings: Option<&'a buffer::Settings>,
         config: &'a Config,
         theme: &'a Theme,
         is_focused: bool,
@@ -181,13 +201,7 @@ impl Buffer {
         match self {
             Buffer::Empty => empty::view(config, sidebar),
             Buffer::Channel(state) => channel::view(
-                state,
-                clients,
-                history,
-                &settings.channel,
-                config,
-                theme,
-                is_focused,
+                state, clients, history, previews, settings, config, theme, is_focused,
             )
             .map(Message::Channel),
             Buffer::Server(state) => {
@@ -195,7 +209,8 @@ impl Buffer {
                     .map(Message::Server)
             }
             Buffer::Query(state) => {
-                query::view(state, clients, history, config, theme, is_focused).map(Message::Query)
+                query::view(state, clients, history, previews, config, theme, is_focused)
+                    .map(Message::Query)
             }
             Buffer::FileTransfers(state) => {
                 file_transfers::view(state, file_transfers).map(Message::FileTransfers)
@@ -219,9 +234,13 @@ impl Buffer {
 
     // TODO: Placeholder in case we need
     #[allow(unused)]
-    pub fn get_channel(&self, server: &data::Server, channel: &str) -> Option<&Channel> {
+    pub fn get_channel(
+        &self,
+        server: &data::Server,
+        channel: &target::Channel,
+    ) -> Option<&Channel> {
         if let Buffer::Channel(state) = self {
-            (&state.server == server && state.channel.as_str() == channel).then_some(state)
+            (&state.server == server && state.target == *channel).then_some(state)
         } else {
             None
         }
@@ -269,6 +288,58 @@ impl Buffer {
         }
     }
 
+    pub fn scroll_up_page(&mut self) -> Task<Message> {
+        match self {
+            Buffer::Empty | Buffer::FileTransfers(_) => Task::none(),
+            Buffer::Channel(channel) => channel
+                .scroll_view
+                .scroll_up_page()
+                .map(|message| Message::Channel(channel::Message::ScrollView(message))),
+            Buffer::Server(server) => server
+                .scroll_view
+                .scroll_up_page()
+                .map(|message| Message::Server(server::Message::ScrollView(message))),
+            Buffer::Query(query) => query
+                .scroll_view
+                .scroll_up_page()
+                .map(|message| Message::Query(query::Message::ScrollView(message))),
+            Buffer::Logs(log) => log
+                .scroll_view
+                .scroll_up_page()
+                .map(|message| Message::Logs(logs::Message::ScrollView(message))),
+            Buffer::Highlights(highlights) => highlights
+                .scroll_view
+                .scroll_up_page()
+                .map(|message| Message::Highlights(highlights::Message::ScrollView(message))),
+        }
+    }
+
+    pub fn scroll_down_page(&mut self) -> Task<Message> {
+        match self {
+            Buffer::Empty | Buffer::FileTransfers(_) => Task::none(),
+            Buffer::Channel(channel) => channel
+                .scroll_view
+                .scroll_down_page()
+                .map(|message| Message::Channel(channel::Message::ScrollView(message))),
+            Buffer::Server(server) => server
+                .scroll_view
+                .scroll_down_page()
+                .map(|message| Message::Server(server::Message::ScrollView(message))),
+            Buffer::Query(query) => query
+                .scroll_view
+                .scroll_down_page()
+                .map(|message| Message::Query(query::Message::ScrollView(message))),
+            Buffer::Logs(log) => log
+                .scroll_view
+                .scroll_down_page()
+                .map(|message| Message::Logs(logs::Message::ScrollView(message))),
+            Buffer::Highlights(highlights) => highlights
+                .scroll_view
+                .scroll_down_page()
+                .map(|message| Message::Highlights(highlights::Message::ScrollView(message))),
+        }
+    }
+
     pub fn scroll_to_start(&mut self) -> Task<Message> {
         match self {
             Buffer::Empty | Buffer::FileTransfers(_) => Task::none(),
@@ -333,7 +404,7 @@ impl Buffer {
                 .scroll_view
                 .scroll_to_message(
                     message,
-                    scroll_view::Kind::Channel(&state.server, &state.channel),
+                    scroll_view::Kind::Channel(&state.server, &state.target),
                     history,
                     config,
                 )
@@ -351,7 +422,7 @@ impl Buffer {
                 .scroll_view
                 .scroll_to_message(
                     message,
-                    scroll_view::Kind::Query(&state.server, &state.nick),
+                    scroll_view::Kind::Query(&state.server, &state.target),
                     history,
                     config,
                 )
@@ -377,7 +448,7 @@ impl Buffer {
             Buffer::Channel(state) => state
                 .scroll_view
                 .scroll_to_backlog(
-                    scroll_view::Kind::Channel(&state.server, &state.channel),
+                    scroll_view::Kind::Channel(&state.server, &state.target),
                     history,
                     config,
                 )
@@ -389,7 +460,7 @@ impl Buffer {
             Buffer::Query(state) => state
                 .scroll_view
                 .scroll_to_backlog(
-                    scroll_view::Kind::Query(&state.server, &state.nick),
+                    scroll_view::Kind::Query(&state.server, &state.target),
                     history,
                     config,
                 )
@@ -415,6 +486,17 @@ impl Buffer {
             Buffer::Highlights(highlights) => Some(highlights.scroll_view.is_scrolled_to_bottom()),
         }
     }
+
+    pub fn close_picker(&mut self) -> bool {
+        match self {
+            Buffer::Empty | Buffer::FileTransfers(_) | Buffer::Logs(_) | Buffer::Highlights(_) => {
+                false
+            }
+            Buffer::Server(state) => state.input_view.close_picker(),
+            Buffer::Channel(state) => state.input_view.close_picker(),
+            Buffer::Query(state) => state.input_view.close_picker(),
+        }
+    }
 }
 
 impl From<data::Buffer> for Buffer {
@@ -425,7 +507,7 @@ impl From<data::Buffer> for Buffer {
                 buffer::Upstream::Channel(server, channel) => {
                     Self::Channel(Channel::new(server, channel))
                 }
-                buffer::Upstream::Query(server, user) => Self::Query(Query::new(server, user)),
+                buffer::Upstream::Query(server, query) => Self::Query(Query::new(server, query)),
             },
             data::Buffer::Internal(internal) => match internal {
                 buffer::Internal::FileTransfers => Self::FileTransfers(FileTransfers::new()),
