@@ -1,9 +1,11 @@
+use data::dashboard::BufferAction;
 use data::server::Server;
+use data::target::{self, Target};
 use data::user::Nick;
-use data::{buffer, User};
-use data::{channel, history, message, Config};
+use data::{buffer, history, message, preview, Config, User};
+use iced::advanced::text;
 use iced::widget::{column, container, row};
-use iced::{alignment, padding, Length, Task};
+use iced::{padding, Length, Task};
 
 use super::{input_view, scroll_view, user_context};
 use crate::widget::{message_content, message_marker, selectable_text, Element};
@@ -21,36 +23,45 @@ pub enum Message {
 
 pub enum Event {
     UserContext(user_context::Event),
-    OpenChannel(String),
+    OpenBuffers(Vec<(Target, BufferAction)>),
     History(Task<history::manager::Message>),
+    RequestOlderChatHistory,
+    PreviewChanged,
+    HidePreview(history::Kind, message::Hash, url::Url),
 }
 
 pub fn view<'a>(
     state: &'a Channel,
     clients: &'a data::client::Map,
     history: &'a history::Manager,
-    settings: &'a channel::Settings,
+    previews: &'a preview::Collection,
+    settings: Option<&'a buffer::Settings>,
     config: &'a Config,
     theme: &'a Theme,
     is_focused: bool,
 ) -> Element<'a, Message> {
     let server = &state.server;
-    let channel = &state.channel;
+    let casemapping = clients.get_casemapping(server);
+    let channel = &state.target;
     let buffer = &state.buffer;
     let input = history.input(buffer);
     let our_nick = clients.nickname(&state.server);
 
     let our_user = our_nick
         .map(|our_nick| User::from(Nick::from(our_nick.as_ref())))
-        .and_then(|user| clients.resolve_user_attributes(&state.server, &state.channel, &user));
+        .and_then(|user| clients.resolve_user_attributes(&state.server, channel, &user));
 
-    let users = clients.get_channel_users(&state.server, &state.channel);
+    let users = clients.get_channel_users(&state.server, channel);
+
+    let chathistory_state = clients.get_chathistory_state(server, &channel.to_target());
 
     let messages = container(
         scroll_view::view(
             &state.scroll_view,
-            scroll_view::Kind::Channel(&state.server, &state.channel),
+            scroll_view::Kind::Channel(&state.server, channel),
             history,
+            Some(previews),
+            chathistory_state,
             config,
             move |message, max_nick_width, max_prefix_width| {
                 let timestamp =
@@ -61,26 +72,27 @@ pub fn view<'a>(
                             selectable_text(timestamp).style(theme::selectable_text::timestamp)
                         });
 
-                let prefix = message.target.prefix().map_or(
+                let prefixes = message.target.prefixes().map_or(
                     max_nick_width.and_then(|_| {
                         max_prefix_width.map(|width| {
                             selectable_text("")
                                 .width(width)
-                                .horizontal_alignment(alignment::Horizontal::Right)
+                                .align_x(text::Alignment::Right)
                         })
                     }),
-                    |prefix| {
+                    |prefixes| {
                         let text = selectable_text(format!(
                             "{} ",
-                            config.buffer.status_message_prefix.brackets.format(prefix)
+                            config
+                                .buffer
+                                .status_message_prefix
+                                .brackets
+                                .format(String::from_iter(prefixes))
                         ))
                         .style(theme::selectable_text::tertiary);
 
                         if let Some(width) = max_prefix_width {
-                            Some(
-                                text.width(width)
-                                    .horizontal_alignment(alignment::Horizontal::Right),
-                            )
+                            Some(text.width(width).align_x(text::Alignment::Right))
                         } else {
                             Some(text)
                         }
@@ -92,7 +104,8 @@ pub fn view<'a>(
 
                 match message.target.source() {
                     message::Source::User(user) => {
-                        let current_user = users.iter().find(|current_user| *current_user == user);
+                        let current_user: Option<&User> =
+                            users.iter().find(|current_user| *current_user == user);
 
                         let mut text = selectable_text(
                             config
@@ -101,32 +114,28 @@ pub fn view<'a>(
                                 .brackets
                                 .format(user.display(with_access_levels)),
                         )
-                        .style(|theme| {
-                            theme::selectable_text::nickname(
-                                theme,
-                                user.nick_color(theme.colors(), config.buffer.nickname.color),
-                                user.is_away(),
-                            )
-                        });
+                        .style(|theme| theme::selectable_text::nickname(theme, config, user));
 
                         if let Some(width) = max_nick_width {
-                            text = text
-                                .width(width)
-                                .horizontal_alignment(alignment::Horizontal::Right);
+                            text = text.width(width).align_x(text::Alignment::Right);
                         }
 
                         let nick = user_context::view(
                             text,
                             server,
+                            casemapping,
                             Some(channel),
                             user,
                             current_user,
                             our_user,
+                            config,
+                            &config.buffer.nickname.click,
                         )
                         .map(scroll_view::Message::UserContext);
 
                         let message_content = message_content::with_context(
                             &message.content,
+                            casemapping,
                             theme,
                             scroll_view::Message::Link,
                             theme::selectable_text::default,
@@ -136,7 +145,15 @@ pub fn view<'a>(
                             },
                             move |link, entry, length| match link {
                                 message::Link::User(user) => entry
-                                    .view(server, Some(channel), user, current_user, length)
+                                    .view(
+                                        server,
+                                        casemapping,
+                                        Some(channel),
+                                        user,
+                                        current_user,
+                                        length,
+                                        config,
+                                    )
                                     .map(scroll_view::Message::UserContext),
                                 _ => row![].into(),
                             },
@@ -145,19 +162,11 @@ pub fn view<'a>(
 
                         let timestamp_nickname_row = row![]
                             .push_maybe(timestamp)
-                            .push_maybe(prefix)
+                            .push_maybe(prefixes)
                             .push(nick)
                             .push(space);
 
-                        let text_container =
-                            container(message_content).style(move |theme| match our_nick {
-                                Some(nick)
-                                    if message::references_user(user.nickname(), nick, message) =>
-                                {
-                                    theme::container::highlight(theme)
-                                }
-                                _ => Default::default(),
-                            });
+                        let text_container = container(message_content);
 
                         match &config.buffer.nickname.alignment {
                             data::buffer::Alignment::Left | data::buffer::Alignment::Right => Some(
@@ -174,52 +183,76 @@ pub fn view<'a>(
                             ),
                         }
                     }
-                    message::Source::Server(server) => {
+                    message::Source::Server(server_message) => {
                         let message_style = move |message_theme: &Theme| {
-                            theme::selectable_text::server(message_theme, server.as_ref())
+                            theme::selectable_text::server(message_theme, server_message.as_ref())
                         };
 
                         let marker = message_marker(max_nick_width, message_style);
 
-                        let message = message_content(
+                        let message_content = message_content::with_context(
                             &message.content,
+                            casemapping,
                             theme,
                             scroll_view::Message::Link,
                             message_style,
+                            move |link| match link {
+                                message::Link::User(_) => user_context::Entry::list(true, our_user),
+                                _ => vec![],
+                            },
+                            move |link, entry, length| match link {
+                                message::Link::User(user) => entry
+                                    .view(
+                                        server,
+                                        casemapping,
+                                        Some(channel),
+                                        user,
+                                        users.iter().find(|current_user| *current_user == user),
+                                        length,
+                                        config,
+                                    )
+                                    .map(scroll_view::Message::UserContext),
+                                _ => row![].into(),
+                            },
                             config,
                         );
 
+                        let text_container = container(message_content);
+
                         Some(
                             container(
                                 row![]
                                     .push_maybe(timestamp)
-                                    .push_maybe(prefix)
+                                    .push_maybe(prefixes)
                                     .push(marker)
                                     .push(space)
-                                    .push(message),
+                                    .push(text_container),
                             )
                             .into(),
                         )
                     }
-                    message::Source::Action => {
+                    message::Source::Action(_) => {
                         let marker = message_marker(max_nick_width, theme::selectable_text::action);
 
-                        let message = message_content(
+                        let message_content = message_content(
                             &message.content,
+                            casemapping,
                             theme,
                             scroll_view::Message::Link,
                             theme::selectable_text::action,
                             config,
                         );
 
+                        let text_container = container(message_content);
+
                         Some(
                             container(
                                 row![]
                                     .push_maybe(timestamp)
-                                    .push_maybe(prefix)
+                                    .push_maybe(prefixes)
                                     .push(marker)
                                     .push(space)
-                                    .push(message),
+                                    .push(text_container),
                             )
                             .into(),
                         )
@@ -233,6 +266,7 @@ pub fn view<'a>(
 
                         let message = message_content(
                             &message.content,
+                            casemapping,
                             theme,
                             scroll_view::Message::Link,
                             message_style,
@@ -243,7 +277,7 @@ pub fn view<'a>(
                             container(
                                 row![]
                                     .push_maybe(timestamp)
-                                    .push_maybe(prefix)
+                                    .push_maybe(prefixes)
                                     .push(marker)
                                     .push(space)
                                     .push(message),
@@ -260,8 +294,8 @@ pub fn view<'a>(
     .width(Length::FillPortion(2))
     .height(Length::Fill);
 
-    let nick_list =
-        nick_list::view(server, channel, users, our_user, config).map(Message::UserContext);
+    let nick_list = nick_list::view(server, casemapping, channel, users, our_user, config)
+        .map(Message::UserContext);
 
     // If topic toggles from None to Some then it messes with messages' scroll state,
     // so produce a zero-height placeholder when topic is None.
@@ -274,7 +308,7 @@ pub fn view<'a>(
     };
 
     let channels = clients.get_channels(&state.server);
-    let is_connected_to_channel = channels.iter().any(|c| c == &state.channel);
+    let is_connected_to_channel = channels.iter().any(|c| c == &state.target);
 
     let text_input = show_text_input.then(move || {
         input_view::view(
@@ -282,16 +316,17 @@ pub fn view<'a>(
             input,
             is_focused,
             !is_connected_to_channel,
+            config,
         )
         .map(Message::InputView)
     });
 
     let content = column![topic, messages].spacing(4);
 
-    let content = match (
-        settings.nicklist.enabled,
-        config.buffer.channel.nicklist.position,
-    ) {
+    let nicklist_enabled = settings
+        .map_or(config.buffer.channel.nicklist.enabled, |settings| settings.channel.nicklist.enabled);
+
+    let content = match (nicklist_enabled, config.buffer.channel.nicklist.position) {
         (true, data::channel::Position::Left) => row![nick_list, content],
         (true, data::channel::Position::Right) => row![content, nick_list],
         (false, _) => { row![content] }.height(Length::Fill),
@@ -315,18 +350,17 @@ pub fn view<'a>(
 pub struct Channel {
     pub buffer: buffer::Upstream,
     pub server: Server,
-    pub channel: String,
-
+    pub target: target::Channel,
     pub scroll_view: scroll_view::State,
     pub input_view: input_view::State,
 }
 
 impl Channel {
-    pub fn new(server: Server, channel: String) -> Self {
+    pub fn new(server: Server, target: target::Channel) -> Self {
         Self {
-            buffer: buffer::Upstream::Channel(server.clone(), channel.clone()),
+            buffer: buffer::Upstream::Channel(server.clone(), target.clone()),
             server,
-            channel,
+            target,
             scroll_view: scroll_view::State::new(),
             input_view: input_view::State::new(),
         }
@@ -341,12 +375,28 @@ impl Channel {
     ) -> (Task<Message>, Option<Event>) {
         match message {
             Message::ScrollView(message) => {
-                let (command, event) = self.scroll_view.update(message);
+                let (command, event) = self.scroll_view.update(
+                    message,
+                    config.buffer.chathistory.infinite_scroll,
+                    scroll_view::Kind::Channel(&self.server, &self.target),
+                    history,
+                    clients,
+                    config,
+                );
 
                 let event = event.and_then(|event| match event {
                     scroll_view::Event::UserContext(event) => Some(Event::UserContext(event)),
-                    scroll_view::Event::OpenChannel(channel) => Some(Event::OpenChannel(channel)),
+                    scroll_view::Event::OpenBuffer(target, buffer_action) => {
+                        Some(Event::OpenBuffers(vec![(target, buffer_action)]))
+                    }
                     scroll_view::Event::GoToMessage(..) => None,
+                    scroll_view::Event::RequestOlderChatHistory => {
+                        Some(Event::RequestOlderChatHistory)
+                    }
+                    scroll_view::Event::PreviewChanged => Some(Event::PreviewChanged),
+                    scroll_view::Event::HidePreview(kind, hash, url) => {
+                        Some(Event::HidePreview(kind, hash, url))
+                    }
                 });
 
                 (command.map(Message::ScrollView), event)
@@ -366,6 +416,9 @@ impl Channel {
 
                         (command, Some(Event::History(history_task)))
                     }
+                    Some(input_view::Event::OpenBuffers { targets }) => {
+                        (command, Some(Event::OpenBuffers(targets)))
+                    }
                     None => (command, None),
                 }
             }
@@ -377,7 +430,10 @@ impl Channel {
                 Task::none(),
                 topic::update(message).map(|event| match event {
                     topic::Event::UserContext(event) => Event::UserContext(event),
-                    topic::Event::OpenChannel(channel) => Event::OpenChannel(channel),
+                    topic::Event::OpenChannel(channel) => Event::OpenBuffers(vec![(
+                        Target::Channel(channel),
+                        config.actions.buffer.click_channel_name,
+                    )]),
                 }),
             ),
         }
@@ -397,20 +453,26 @@ fn topic<'a>(
     clients: &'a data::client::Map,
     users: &'a [User],
     our_user: Option<&'a User>,
-    settings: &'a channel::Settings,
+    settings: Option<&'a buffer::Settings>,
     config: &'a Config,
     theme: &'a Theme,
 ) -> Option<Element<'a, Message>> {
-    if !settings.topic.enabled {
+    let topic_enabled = settings
+        .map_or(config.buffer.channel.topic.enabled, |settings| settings.channel.topic.enabled);
+
+    if !topic_enabled {
         return None;
     }
 
-    let topic = clients.get_channel_topic(&state.server, &state.channel)?;
+    let casemapping = clients.get_casemapping(&state.server);
+
+    let topic = clients.get_channel_topic(&state.server, &state.target)?;
 
     Some(
         topic::view(
             &state.server,
-            &state.channel,
+            casemapping,
+            &state.target,
             topic.content.as_ref()?,
             topic.who.as_deref(),
             topic.time.as_ref(),
@@ -425,9 +487,10 @@ fn topic<'a>(
 }
 
 mod nick_list {
-    use data::{config, Config, Server, User};
+    use data::{config, isupport, target, Config, Server, User};
+    use iced::advanced::text;
     use iced::widget::{column, scrollable, Scrollable};
-    use iced::{alignment, Length};
+    use iced::Length;
     use user_context::Message;
 
     use crate::buffer::user_context;
@@ -436,7 +499,8 @@ mod nick_list {
 
     pub fn view<'a>(
         server: &'a Server,
-        channel: &'a str,
+        casemapping: isupport::CaseMap,
+        channel: &'a target::Channel,
         users: &'a [User],
         our_user: Option<&'a User>,
         config: &'a Config,
@@ -462,20 +526,24 @@ mod nick_list {
 
         let content = column(users.iter().map(|user| {
             let content = selectable_text(user.display(nicklist_config.show_access_levels))
-                .style(|theme| {
-                    theme::selectable_text::nickname(
-                        theme,
-                        user.nick_color(theme.colors(), nicklist_config.color),
-                        user.is_away(),
-                    )
-                })
-                .horizontal_alignment(match nicklist_config.alignment {
-                    config::channel::Alignment::Left => alignment::Horizontal::Left,
-                    config::channel::Alignment::Right => alignment::Horizontal::Right,
+                .style(|theme| theme::selectable_text::nicklist_nickname(theme, config, user))
+                .align_x(match nicklist_config.alignment {
+                    config::buffer::channel::Alignment::Left => text::Alignment::Left,
+                    config::buffer::channel::Alignment::Right => text::Alignment::Right,
                 })
                 .width(Length::Fixed(width));
 
-            user_context::view(content, server, Some(channel), user, Some(user), our_user)
+            user_context::view(
+                content,
+                server,
+                casemapping,
+                Some(channel),
+                user,
+                Some(user),
+                our_user,
+                config,
+                &config.buffer.channel.nicklist.click,
+            )
         }));
 
         Scrollable::new(content)
