@@ -1,17 +1,16 @@
 use chrono::{DateTime, Utc};
-use data::user::Nick;
-use data::{message, Config, Server, User};
-use iced::widget::{column, container, horizontal_rule, row, scrollable, Scrollable};
+use data::{Config, Server, User, isupport, message, target};
 use iced::Length;
+use iced::widget::{Scrollable, column, container, horizontal_rule, row, scrollable};
 
 use super::user_context;
-use crate::widget::{double_pass, message_content, selectable_text, Element};
-use crate::{theme, Theme};
+use crate::widget::{Element, double_pass, message_content, selectable_text};
+use crate::{Theme, theme};
 
 #[derive(Debug, Clone)]
 pub enum Event {
     UserContext(user_context::Event),
-    OpenChannel(String),
+    OpenChannel(target::Channel),
 }
 
 #[derive(Debug, Clone)]
@@ -29,7 +28,7 @@ pub fn update(message: Message) -> Option<Event> {
             None
         }
         Message::Link(message::Link::User(user)) => Some(Event::UserContext(
-            user_context::Event::SingleClick(user.nickname().to_owned()),
+            user_context::Event::InsertNickname(user.nickname().to_owned()),
         )),
         Message::Link(message::Link::GoToMessage(..)) => None,
     }
@@ -37,7 +36,8 @@ pub fn update(message: Message) -> Option<Event> {
 
 pub fn view<'a>(
     server: &'a Server,
-    channel: &'a String,
+    casemapping: isupport::CaseMap,
+    channel: &'a target::Channel,
     content: &'a message::Content,
     who: Option<&'a str>,
     time: Option<&'a DateTime<Utc>>,
@@ -47,43 +47,48 @@ pub fn view<'a>(
     config: &'a Config,
     theme: &'a Theme,
 ) -> Element<'a, Message> {
-    let set_by = who.and_then(|who| {
-        let nick = Nick::from(who.split('!').next()?);
+    let set_by = who
+        .and_then(|who| User::try_from(who).ok())
+        .and_then(|user| {
+            let channel_user = users.iter().find(|u| **u == user);
 
-        let user = if let Some(user) = users.iter().find(|user| user.nickname() == nick) {
-            user_context::view(
-                selectable_text(who).style(|theme| {
-                    theme::selectable_text::nickname(
-                        theme,
-                        user.nick_color(theme.colors(), config.buffer.nickname.color),
-                        false,
-                    )
-                }),
-                server,
-                Some(channel),
-                user,
-                Some(user),
-                our_user,
+            // If user is in channel, we return user_context component.
+            // Otherwise selectable_text component.
+            let content = if let Some(user) = channel_user {
+                user_context::view(
+                    selectable_text(user.nickname().to_string())
+                        .style(|theme| theme::selectable_text::topic_nickname(theme, config, user)),
+                    server,
+                    casemapping,
+                    Some(channel),
+                    user,
+                    Some(user),
+                    our_user,
+                    config,
+                    &config.buffer.nickname.click,
+                )
+            } else {
+                selectable_text(user.display(false))
+                    .style(move |theme| {
+                        theme::selectable_text::topic_nickname(theme, config, &user)
+                    })
+                    .into()
+            };
+
+            Some(
+                Element::new(row![
+                    selectable_text("set by ").style(theme::selectable_text::topic),
+                    content,
+                    selectable_text(format!(" at {}", time?.to_rfc2822()))
+                        .style(theme::selectable_text::topic),
+                ])
+                .map(Message::UserContext),
             )
-        } else {
-            selectable_text(who)
-                .style(theme::selectable_text::tertiary)
-                .into()
-        };
-
-        Some(
-            Element::new(row![
-                selectable_text("set by ").style(theme::selectable_text::topic),
-                user,
-                selectable_text(format!(" at {}", time?.to_rfc2822()))
-                    .style(theme::selectable_text::topic),
-            ])
-            .map(Message::UserContext),
-        )
-    });
+        });
 
     let content = column![message_content(
         content,
+        casemapping,
         theme,
         Message::Link,
         theme::selectable_text::topic,
