@@ -1,3 +1,5 @@
+use data::dashboard::BufferAction;
+use data::target::Target;
 use data::{buffer, history, message, Config};
 use iced::widget::{column, container, row, vertical_space};
 use iced::{Length, Task};
@@ -14,7 +16,7 @@ pub enum Message {
 
 pub enum Event {
     UserContext(user_context::Event),
-    OpenChannel(String),
+    OpenBuffers(Vec<(Target, BufferAction)>),
     History(Task<history::manager::Message>),
 }
 
@@ -27,6 +29,7 @@ pub fn view<'a>(
     is_focused: bool,
 ) -> Element<'a, Message> {
     let status = clients.status(&state.server);
+    let casemapping = clients.get_casemapping(&state.server);
     let buffer = &state.buffer;
     let input = history.input(buffer);
 
@@ -35,6 +38,8 @@ pub fn view<'a>(
             &state.scroll_view,
             scroll_view::Kind::Server(&state.server),
             history,
+            None,
+            None,
             config,
             move |message, _, _| {
                 let timestamp =
@@ -49,6 +54,7 @@ pub fn view<'a>(
                     message::Source::Server(server) => {
                         let message = message_content(
                             &message.content,
+                            casemapping,
                             theme,
                             scroll_view::Message::Link,
                             move |theme| theme::selectable_text::server(theme, server.as_ref()),
@@ -60,6 +66,7 @@ pub fn view<'a>(
                     message::Source::Internal(message::source::Internal::Status(status)) => {
                         let message = message_content(
                             &message.content,
+                            casemapping,
                             theme,
                             scroll_view::Message::Link,
                             move |theme| theme::selectable_text::status(theme, *status),
@@ -84,8 +91,14 @@ pub fn view<'a>(
     let text_input = show_text_input.then(|| {
         column![
             vertical_space().height(4),
-            input_view::view(&state.input_view, input, is_focused, !status.connected())
-                .map(Message::InputView)
+            input_view::view(
+                &state.input_view,
+                input,
+                is_focused,
+                !status.connected(),
+                config
+            )
+            .map(Message::InputView)
         ]
         .width(Length::Fill)
     });
@@ -128,12 +141,24 @@ impl Server {
     ) -> (Task<Message>, Option<Event>) {
         match message {
             Message::ScrollView(message) => {
-                let (command, event) = self.scroll_view.update(message);
+                let (command, event) = self.scroll_view.update(
+                    message,
+                    false,
+                    scroll_view::Kind::Server(&self.server),
+                    history,
+                    clients,
+                    config,
+                );
 
                 let event = event.and_then(|event| match event {
                     scroll_view::Event::UserContext(event) => Some(Event::UserContext(event)),
-                    scroll_view::Event::OpenChannel(channel) => Some(Event::OpenChannel(channel)),
+                    scroll_view::Event::OpenBuffer(target, buffer_action) => {
+                        Some(Event::OpenBuffers(vec![(target, buffer_action)]))
+                    }
                     scroll_view::Event::GoToMessage(_, _, _) => None,
+                    scroll_view::Event::RequestOlderChatHistory => None,
+                    scroll_view::Event::PreviewChanged => None,
+                    scroll_view::Event::HidePreview(..) => None,
                 });
 
                 (command.map(Message::ScrollView), event)
@@ -152,6 +177,9 @@ impl Server {
                         ]),
                         Some(Event::History(history_task)),
                     ),
+                    Some(input_view::Event::OpenBuffers { targets }) => {
+                        (command, Some(Event::OpenBuffers(targets)))
+                    }
                     None => (command, None),
                 }
             }
