@@ -1,10 +1,11 @@
+use data::dashboard::BufferAction;
 use data::user::Nick;
-use data::{Server, User};
-use iced::widget::{button, container, horizontal_rule, row, text, Space};
+use data::{config, isupport, target, Config, Server, User};
+use iced::widget::{button, column, container, horizontal_rule, row, text, Space};
 use iced::{padding, Length, Padding};
 
 use crate::widget::{context_menu, double_pass, Element};
-use crate::{icon, theme};
+use crate::{theme, widget};
 
 #[derive(Debug, Clone, Copy)]
 pub enum Entry {
@@ -47,18 +48,26 @@ impl Entry {
     pub fn view<'a>(
         self,
         server: &Server,
-        channel: Option<&str>,
+        casemapping: isupport::CaseMap,
+        channel: Option<&target::Channel>,
         user: &User,
         current_user: Option<&User>,
         length: Length,
+        config: &Config,
     ) -> Element<'a, Message> {
         let nickname = user.nickname().to_owned();
 
         match self {
             Entry::Whois => menu_button("Whois", Message::Whois(server.clone(), nickname), length),
-            Entry::Query => {
-                menu_button("Message", Message::Query(server.clone(), nickname), length)
-            }
+            Entry::Query => menu_button(
+                "Message",
+                Message::Query(
+                    server.clone(),
+                    target::Query::from_user(user, casemapping),
+                    config.actions.buffer.message_user,
+                ),
+                length,
+            ),
             Entry::ToggleAccessLevelOp => {
                 if let Some(channel) = channel {
                     if user.has_access_level(data::user::AccessLevel::Oper) {
@@ -66,7 +75,7 @@ impl Entry {
                             "Take Op (-o)",
                             Message::ToggleAccessLevel(
                                 server.clone(),
-                                channel.to_string(),
+                                channel.clone(),
                                 nickname,
                                 "-o".to_owned(),
                             ),
@@ -77,7 +86,7 @@ impl Entry {
                             "Give Op (+o)",
                             Message::ToggleAccessLevel(
                                 server.clone(),
-                                channel.to_string(),
+                                channel.clone(),
                                 nickname,
                                 "+o".to_owned(),
                             ),
@@ -95,7 +104,7 @@ impl Entry {
                             "Take Voice (-v)",
                             Message::ToggleAccessLevel(
                                 server.clone(),
-                                channel.to_string(),
+                                channel.clone(),
                                 nickname,
                                 "-v".to_owned(),
                             ),
@@ -106,7 +115,7 @@ impl Entry {
                             "Give Voice (+v)",
                             Message::ToggleAccessLevel(
                                 server.clone(),
-                                channel.to_string(),
+                                channel.clone(),
                                 nickname,
                                 "+v".to_owned(),
                             ),
@@ -122,7 +131,7 @@ impl Entry {
                 Message::SendFile(server.clone(), nickname),
                 length,
             ),
-            Entry::UserInfo => user_info(current_user, length),
+            Entry::UserInfo => user_info(current_user, nickname, length, config),
             Entry::HorizontalRule => match length {
                 Length::Fill => container(horizontal_rule(1)).padding([0, 6]).into(),
                 _ => Space::new(length, 1).into(),
@@ -134,53 +143,76 @@ impl Entry {
 #[derive(Clone, Debug)]
 pub enum Message {
     Whois(Server, Nick),
-    Query(Server, Nick),
-    ToggleAccessLevel(Server, String, Nick, String),
+    Query(Server, target::Query, BufferAction),
+    ToggleAccessLevel(Server, target::Channel, Nick, String),
     SendFile(Server, Nick),
-    SingleClick(Nick),
+    InsertNickname(Nick),
 }
 
 #[derive(Debug, Clone)]
 pub enum Event {
     SendWhois(Server, Nick),
-    OpenQuery(Server, Nick),
-    ToggleAccessLevel(Server, String, Nick, String),
+    OpenQuery(Server, target::Query, BufferAction),
+    ToggleAccessLevel(Server, target::Channel, Nick, String),
     SendFile(Server, Nick),
-    SingleClick(Nick),
+    InsertNickname(Nick),
 }
 
 pub fn update(message: Message) -> Option<Event> {
     match message {
         Message::Whois(server, nick) => Some(Event::SendWhois(server, nick)),
-        Message::Query(server, nick) => Some(Event::OpenQuery(server, nick)),
+        Message::Query(server, nick, buffer_action) => {
+            Some(Event::OpenQuery(server, nick, buffer_action))
+        }
         Message::ToggleAccessLevel(server, target, nick, mode) => {
             Some(Event::ToggleAccessLevel(server, target, nick, mode))
         }
         Message::SendFile(server, nick) => Some(Event::SendFile(server, nick)),
-        Message::SingleClick(nick) => Some(Event::SingleClick(nick)),
+        Message::InsertNickname(nick) => Some(Event::InsertNickname(nick)),
     }
 }
 
 pub fn view<'a>(
     content: impl Into<Element<'a, Message>>,
     server: &'a Server,
-    channel: Option<&'a str>,
+    casemapping: isupport::CaseMap,
+    channel: Option<&'a target::Channel>,
     user: &'a User,
     current_user: Option<&'a User>,
     our_user: Option<&'a User>,
+    config: &'a Config,
+    click: &'a config::buffer::NicknameClickAction,
 ) -> Element<'a, Message> {
     let entries = Entry::list(channel.is_some(), our_user);
 
-    let content = button(content)
-        .padding(0)
-        .style(theme::button::bare)
-        .on_press(Message::SingleClick(user.nickname().to_owned()));
+    let message = match click {
+        data::config::buffer::NicknameClickAction::OpenQuery => Message::Query(
+            server.clone(),
+            target::Query::from_user(user, casemapping),
+            config.actions.buffer.click_username,
+        ),
+        data::config::buffer::NicknameClickAction::InsertNickname => {
+            Message::InsertNickname(user.nickname().to_owned())
+        }
+    };
+
+    let base = widget::button::transparent_button(content, message);
 
     context_menu(
-        Default::default(),
-        content,
+        context_menu::MouseButton::default(),
+        base,
         entries,
-        move |entry, length| entry.view(server, channel, user, current_user, length),
+        move |entry, length| {
+            entry.view(
+                server,
+                casemapping,
+                channel,
+                user,
+                current_user,
+                length,
+                config,
+            )
+        },
     )
     .into()
 }
@@ -197,44 +229,37 @@ fn right_justified_padding() -> Padding {
     padding::all(5).right(5.0 + double_pass::horizontal_expansion())
 }
 
-fn user_info<'a>(current_user: Option<&User>, length: Length) -> Element<'a, Message> {
-    if let Some(current_user) = current_user {
-        if current_user.is_away() {
-            row![]
-                .push(text("Away").style(theme::text::secondary).width(length))
-                .push(
-                    icon::dot()
-                        .size(6)
-                        .style(theme::text::tertiary)
-                        .shaping(text::Shaping::Advanced),
-                )
-                .padding(right_justified_padding())
-                .align_y(iced::Alignment::Center)
-                .into()
-        } else {
-            row![]
-                .push(text("Online").style(theme::text::secondary).width(length))
-                .push(
-                    icon::dot()
-                        .size(6)
-                        .style(theme::text::success)
-                        .shaping(text::Shaping::Advanced),
-                )
-                .padding(right_justified_padding())
-                .align_y(iced::Alignment::Center)
-                .into()
+fn user_info<'a>(
+    current_user: Option<&User>,
+    nickname: Nick,
+    length: Length,
+    config: &Config,
+) -> Element<'a, Message> {
+    let state = match current_user {
+        Some(user) => {
+            if user.is_away() {
+                Some(text("Away").style(theme::text::secondary).width(length))
+            } else {
+                None
+            }
         }
-    } else {
-        row![]
-            .push(text("Offline").style(theme::text::secondary).width(length))
-            .push(
-                icon::dot()
-                    .size(6)
-                    .style(theme::text::error)
-                    .shaping(text::Shaping::Advanced),
-            )
-            .padding(right_justified_padding())
-            .align_y(iced::Alignment::Center)
-            .into()
-    }
+        None => Some(text("Offline").style(theme::text::secondary).width(length)),
+    };
+
+    // Dimmed if away or offline.
+    let is_user_away = current_user.is_some_and(|u| u.is_away());
+    let away_appearance = config.buffer.away.appearance(is_user_away);
+    let seed = match config.buffer.nickname.color {
+        data::buffer::Color::Solid => None,
+        data::buffer::Color::Unique => Some(nickname.to_string()),
+    };
+
+    column![container(
+        text(nickname.to_string())
+            .style(move |theme| theme::text::nickname(theme, seed.clone(), away_appearance))
+            .width(length)
+    )
+    .padding(right_justified_padding()),]
+    .push_maybe(state.map(|s| container(s).padding(right_justified_padding())))
+    .into()
 }
