@@ -23,10 +23,10 @@ use std::{env, mem};
 use appearance::{theme, Theme};
 use chrono::Utc;
 use data::config::{self, Config};
-use data::history::manager::Broadcast;
+use data::history::{self, manager::Broadcast};
+use data::target::{self, Target};
 use data::version::Version;
-use data::{environment, server, version, Url, User};
-use data::{history, Server};
+use data::{environment, server, version, Notification, Server, Url, User};
 use iced::widget::{column, container};
 use iced::{padding, Length, Subscription, Task};
 use screen::{dashboard, help, migration, welcome};
@@ -35,6 +35,7 @@ use tokio_stream::wrappers::ReceiverStream;
 
 use self::event::{events, Event};
 use self::modal::Modal;
+use self::notification::Notifications;
 use self::widget::Element;
 use self::window::Window;
 
@@ -44,8 +45,7 @@ pub fn main() -> Result<(), Box<dyn std::error::Error>> {
 
     let version = args
         .next()
-        .map(|s| s == "--version" || s == "-V")
-        .unwrap_or_default();
+        .is_some_and(|s| s == "--version" || s == "-V");
 
     if version {
         println!("halloy {}", environment::formatted_version());
@@ -65,14 +65,18 @@ pub fn main() -> Result<(), Box<dyn std::error::Error>> {
 
     // spin up a single-threaded tokio runtime to run the config loading task to completion
     // we don't want to wrap our whole program with a runtime since iced starts its own.
-    let config_load = {
+    let (config_load, window_load) = {
         let rt = runtime::Builder::new_current_thread()
             .enable_all()
             .build()?;
 
-        rt.block_on(Config::load())
-    };
+        rt.block_on(async {
+            let config = Config::load().await;
+            let window = data::Window::load().await;
 
+            (config, window)
+        })
+    };
     // DANGER ZONE - font must be set using config
     // before we do any iced related stuff w/ it
     font::set(config_load.as_ref().ok());
@@ -84,18 +88,12 @@ pub fn main() -> Result<(), Box<dyn std::error::Error>> {
         }
     }
 
-    // TODO: Renable persistant window position and size:
-    // Winit currently has a bug with resize and move events.
-    // Until it have been fixed, the persistant position and size has been disabled.
-    //
-    // let window_load = Window::load().unwrap_or_default();
-
     iced::daemon("Halloy", Halloy::update, Halloy::view)
         .theme(Halloy::theme)
         .scale_factor(Halloy::scale_factor)
         .subscription(Halloy::subscription)
         .settings(settings(&config_load))
-        .run_with(move || Halloy::new(config_load, destination, log_stream))
+        .run_with(move || Halloy::new(config_load, window_load, destination, log_stream))
         .inspect_err(|err| log::error!("{}", err))?;
 
     Ok(())
@@ -106,8 +104,7 @@ fn settings(config_load: &Result<Config, config::Error>) -> iced::Settings {
         .as_ref()
         .ok()
         .and_then(|config| config.font.size)
-        .map(f32::from)
-        .unwrap_or(theme::TEXT_SIZE);
+        .map_or(theme::TEXT_SIZE, f32::from);
 
     iced::Settings {
         default_font: font::MONO.clone().into(),
@@ -118,6 +115,10 @@ fn settings(config_load: &Result<Config, config::Error>) -> iced::Settings {
     }
 }
 
+fn handle_irc_error(e: anyhow::Error) {
+    log::error!("{:#}", e);
+}
+
 struct Halloy {
     version: Version,
     screen: Screen,
@@ -128,6 +129,7 @@ struct Halloy {
     modal: Option<Modal>,
     main_window: Window,
     pending_logs: Vec<data::log::Record>,
+    notifications: Notifications,
 }
 
 impl Halloy {
@@ -142,7 +144,7 @@ impl Halloy {
             Err(error) => {
                 log::warn!("failed to load dashboard: {error}");
 
-                screen::Dashboard::empty(config)
+                screen::Dashboard::empty(config, &main_window)
             }
         };
 
@@ -185,12 +187,13 @@ impl Halloy {
                 version: Version::new(),
                 screen,
                 theme: appearance::theme(&config.appearance.selected).into(),
-                clients: Default::default(),
+                clients: data::client::Map::default(),
                 servers: config.servers.clone(),
                 config,
                 modal: None,
                 main_window,
                 pending_logs: vec![],
+                notifications: Notifications::new(),
             },
             command,
         )
@@ -228,12 +231,16 @@ pub enum Message {
 impl Halloy {
     fn new(
         config_load: Result<Config, config::Error>,
+        window_load: Result<data::Window, window::Error>,
         url_received: Option<data::Url>,
         log_stream: ReceiverStream<Vec<logger::Record>>,
     ) -> (Halloy, Task<Message>) {
+        let data::Window { size, position } = window_load.unwrap_or_default();
+        let position = position.map(window::Position::Specific).unwrap_or_default();
+
         let (main_window, open_main_window) = window::open(window::Settings {
-            size: window::default_size(),
-            position: window::Position::Default,
+            size,
+            position,
             min_size: Some(window::MIN_SIZE),
             exit_on_close_request: false,
             ..window::settings()
@@ -345,6 +352,10 @@ impl Halloy {
                         self.clients.quit(&server, None);
                         Task::none()
                     }
+                    Some(dashboard::Event::IrcError(e)) => {
+                        handle_irc_error(e);
+                        Task::none()
+                    }
                     Some(dashboard::Event::Exit) => {
                         let pending_exit = self.clients.exit();
 
@@ -425,7 +436,11 @@ impl Halloy {
                             .broadcast(&server, &self.config, sent_time, Broadcast::Connecting)
                             .map(Message::Dashboard)
                     } else {
-                        notification::disconnected(&self.config.notifications, &server);
+                        self.notifications.notify(
+                            &self.config.notifications,
+                            &Notification::Disconnected,
+                            &server,
+                        );
 
                         dashboard
                             .broadcast(
@@ -449,19 +464,31 @@ impl Halloy {
                         return Task::none();
                     };
 
-                    if is_initial {
-                        notification::connected(&self.config.notifications, &server);
+                    let broadcast = if is_initial {
+                        self.notifications.notify(
+                            &self.config.notifications,
+                            &Notification::Connected,
+                            &server,
+                        );
 
                         dashboard
                             .broadcast(&server, &self.config, sent_time, Broadcast::Connected)
                             .map(Message::Dashboard)
                     } else {
-                        notification::reconnected(&self.config.notifications, &server);
+                        self.notifications.notify(
+                            &self.config.notifications,
+                            &Notification::Reconnected,
+                            &server,
+                        );
 
                         dashboard
                             .broadcast(&server, &self.config, sent_time, Broadcast::Reconnected)
                             .map(Message::Dashboard)
-                    }
+                    };
+
+                    let refocus_pane = dashboard.refocus_pane().map(Message::Dashboard);
+
+                    Task::batch(vec![broadcast, refocus_pane])
                 }
                 stream::Update::ConnectionFailed {
                     server,
@@ -489,20 +516,37 @@ impl Halloy {
                     let commands = messages
                         .into_iter()
                         .flat_map(|message| {
+                            let events = match self.clients.receive(
+                                &server,
+                                message,
+                                &self.config.actions,
+                            ) {
+                                Ok(events) => events,
+                                Err(e) => {
+                                    handle_irc_error(e);
+                                    return vec![];
+                                }
+                            };
+
                             let mut commands = vec![];
 
-                            for event in self.clients.receive(&server, message) {
+                            for event in events {
                                 // Resolve a user using client state which stores attributes
-                                let resolve_user_attributes = |user: &User, channel: &str| {
-                                    self.clients
-                                        .resolve_user_attributes(&server, channel, user)
-                                        .cloned()
-                                };
+                                let resolve_user_attributes =
+                                    |user: &User, channel: &target::Channel| {
+                                        self.clients
+                                            .resolve_user_attributes(&server, channel, user)
+                                            .cloned()
+                                    };
 
-                                let channel_users = |channel: &str| -> &[User] {
+                                let channel_users = |channel: &target::Channel| -> &[User] {
                                     self.clients.get_channel_users(&server, channel)
                                 };
 
+                                let chantypes = self.clients.get_chantypes(&server);
+                                let statusmsg = self.clients.get_statusmsg(&server);
+                                let casemapping = self.clients.get_casemapping(&server);
+
                                 match event {
                                     data::client::Event::Single(encoded, our_nick) => {
                                         if let Some(message) = data::Message::received(
@@ -511,6 +555,9 @@ impl Halloy {
                                             &self.config,
                                             resolve_user_attributes,
                                             channel_users,
+                                            chantypes,
+                                            statusmsg,
+                                            casemapping,
                                         ) {
                                             commands.push(
                                                 dashboard
@@ -519,6 +566,46 @@ impl Halloy {
                                             );
                                         }
                                     }
+                                    data::client::Event::PrivOrNotice(
+                                        encoded,
+                                        our_nick,
+                                        highlight_notification_enabled,
+                                    ) => {
+                                        if let Some(message) = data::Message::received(
+                                            encoded,
+                                            our_nick,
+                                            &self.config,
+                                            resolve_user_attributes,
+                                            channel_users,
+                                            chantypes,
+                                            statusmsg,
+                                            casemapping,
+                                        ) {
+                                            if let Some((message, channel, user)) =
+                                                message.into_highlight(server.clone())
+                                            {
+                                                commands.push(
+                                                    dashboard
+                                                        .record_highlight(message)
+                                                        .map(Message::Dashboard),
+                                                );
+
+                                                if highlight_notification_enabled {
+                                                    self.notifications.notify(
+                                                        &self.config.notifications,
+                                                        &Notification::Highlight { user, channel },
+                                                        &server,
+                                                    );
+                                                }
+                                            }
+
+                                            commands.push(
+                                                dashboard
+                                                    .record_message(&server, message)
+                                                    .map(Message::Dashboard),
+                                            );
+                                        }
+                                    }
                                     data::client::Event::WithTarget(encoded, our_nick, target) => {
                                         if let Some(message) = data::Message::received(
                                             encoded,
@@ -526,6 +613,9 @@ impl Halloy {
                                             &self.config,
                                             resolve_user_attributes,
                                             channel_users,
+                                            chantypes,
+                                            statusmsg,
+                                            casemapping,
                                         ) {
                                             commands.push(
                                                 dashboard
@@ -610,6 +700,7 @@ impl Halloy {
                                             new_username,
                                             new_hostname,
                                             ourself,
+                                            logged_in,
                                             channels,
                                             sent_time,
                                         } => {
@@ -624,6 +715,7 @@ impl Halloy {
                                                             new_username,
                                                             new_hostname,
                                                             ourself,
+                                                            logged_in,
                                                             user_channels: channels,
                                                         },
                                                     )
@@ -631,97 +723,12 @@ impl Halloy {
                                             );
                                         }
                                     },
-                                    data::client::Event::Notification(
-                                        encoded,
-                                        our_nick,
-                                        notification,
-                                    ) => {
-                                        if let Some(message) = data::Message::received(
-                                            encoded,
-                                            our_nick,
-                                            &self.config,
-                                            resolve_user_attributes,
-                                            channel_users,
-                                        ) {
-                                            commands.push(
-                                                dashboard
-                                                    .record_message(&server, message.clone())
-                                                    .map(Message::Dashboard),
-                                            );
-
-                                            if matches!(
-                                                notification,
-                                                data::client::Notification::Highlight { .. }
-                                            ) {
-                                                commands.extend(
-                                                    message.into_highlight(server.clone()).map(
-                                                        |message| {
-                                                            dashboard
-                                                                .record_highlight(message)
-                                                                .map(Message::Dashboard)
-                                                        },
-                                                    ),
-                                                );
-                                            }
-                                        }
-
-                                        match notification {
-                                            data::client::Notification::DirectMessage(user) => {
-                                                // only send notification if query has unread
-                                                // or if window is not focused
-                                                if dashboard.history().has_unread(
-                                                    &history::Kind::Query(
-                                                        server.clone(),
-                                                        user.nickname().to_owned(),
-                                                    ),
-                                                ) || !self.main_window.focused
-                                                {
-                                                    notification::direct_message(
-                                                        &self.config.notifications,
-                                                        user.nickname(),
-                                                    );
-                                                }
-                                            }
-                                            data::client::Notification::Highlight {
-                                                enabled,
-                                                user,
-                                                channel,
-                                            } => {
-                                                if enabled {
-                                                    notification::highlight(
-                                                        &self.config.notifications,
-                                                        user.nickname(),
-                                                        channel,
-                                                    );
-                                                }
-                                            }
-                                            data::client::Notification::MonitoredOnline(
-                                                targets,
-                                            ) => {
-                                                targets.into_iter().for_each(|target| {
-                                                    notification::monitored_online(
-                                                        &self.config.notifications,
-                                                        target.nickname().to_owned(),
-                                                        server.clone(),
-                                                    );
-                                                });
-                                            }
-                                            data::client::Notification::MonitoredOffline(
-                                                targets,
-                                            ) => {
-                                                targets.into_iter().for_each(|target| {
-                                                    notification::monitored_offline(
-                                                        &self.config.notifications,
-                                                        target,
-                                                        server.clone(),
-                                                    );
-                                                });
-                                            }
-                                        }
-                                    }
                                     data::client::Event::FileTransferRequest(request) => {
                                         if let Some(command) = dashboard.receive_file_transfer(
                                             &server,
+                                            chantypes,
+                                            statusmsg,
+                                            casemapping,
                                             request,
                                             &self.config,
                                         ) {
@@ -741,13 +748,109 @@ impl Halloy {
                                                 .map(Message::Dashboard),
                                         );
                                     }
-                                    data::client::Event::JoinedChannel(channel) => {
-                                        commands.push(
-                                            dashboard
-                                                .channel_joined(server.clone(), channel)
-                                                .map(Message::Dashboard),
+                                    data::client::Event::JoinedChannel(channel, server_time) => {
+                                        let command = dashboard
+                                            .load_metadata(
+                                                &self.clients,
+                                                server.clone(),
+                                                Target::Channel(channel),
+                                                server_time,
+                                            )
+                                            .map(Message::Dashboard);
+
+                                        commands.push(command);
+                                    }
+                                    data::client::Event::LoggedIn(server_time) => {
+                                        if self.clients.get_server_supports_chathistory(&server) {
+                                            if let Some(command) = dashboard
+                                                .load_chathistory_targets_timestamp(
+                                                    &self.clients,
+                                                    &server,
+                                                    server_time,
+                                                )
+                                                .map(|command| command.map(Message::Dashboard))
+                                            {
+                                                commands.push(command);
+                                            }
+                                        }
+                                    }
+                                    data::client::Event::ChatHistoryTargetReceived(
+                                        target,
+                                        server_time,
+                                    ) => {
+                                        let command = dashboard
+                                            .load_metadata(
+                                                &self.clients,
+                                                server.clone(),
+                                                target,
+                                                server_time,
+                                            )
+                                            .map(Message::Dashboard);
+
+                                        commands.push(command);
+                                    }
+                                    data::client::Event::ChatHistoryTargetsReceived(
+                                        server_time,
+                                    ) => {
+                                        if let Some(command) = dashboard
+                                            .overwrite_chathistory_targets_timestamp(
+                                                &self.clients,
+                                                &server,
+                                                server_time,
+                                            )
+                                            .map(|command| command.map(Message::Dashboard))
+                                        {
+                                            commands.push(command);
+                                        }
+                                    }
+                                    data::client::Event::DirectMessage(user) => {
+                                        if let Ok(query) = target::Query::parse(
+                                            user.as_str(),
+                                            chantypes,
+                                            statusmsg,
+                                            casemapping,
+                                        ) {
+                                            if dashboard.history().has_unread(
+                                                &history::Kind::Query(server.clone(), query),
+                                            ) || !self.main_window.focused
+                                            {
+                                                self.notifications.notify(
+                                                    &self.config.notifications,
+                                                    &Notification::DirectMessage(user),
+                                                    &server,
+                                                );
+                                            }
+                                        }
+                                    }
+                                    data::client::Event::MonitoredOnline(users) => {
+                                        self.notifications.notify(
+                                            &self.config.notifications,
+                                            &Notification::MonitoredOnline(users),
+                                            &server,
                                         );
                                     }
+                                    data::client::Event::MonitoredOffline(users) => {
+                                        self.notifications.notify(
+                                            &self.config.notifications,
+                                            &Notification::MonitoredOffline(users),
+                                            &server,
+                                        );
+                                    }
+                                    data::client::Event::OpenBuffers(targets) => {
+                                        for (target, buffer_action) in targets {
+                                            commands.push(
+                                                dashboard
+                                                    .open_target(
+                                                        server.clone(),
+                                                        target,
+                                                        &mut self.clients,
+                                                        buffer_action,
+                                                        &self.config,
+                                                    )
+                                                    .map(Message::Dashboard),
+                                            );
+                                        }
+                                    }
                                 }
                             }
 
@@ -804,11 +907,10 @@ impl Halloy {
                         .handle_event(
                             window,
                             event,
-                            &self.clients,
+                            &mut self.clients,
                             &self.version,
                             &self.config,
                             &mut self.theme,
-                            &self.main_window,
                         )
                         .map(Message::Dashboard);
                 }
@@ -816,7 +918,9 @@ impl Halloy {
                 Task::none()
             }
             Message::Tick(now) => {
-                self.clients.tick(now);
+                if let Err(e) = self.clients.tick(now) {
+                    handle_irc_error(e);
+                }
 
                 if let Screen::Dashboard(dashboard) = &mut self.screen {
                     dashboard.tick(now).map(Message::Dashboard)
@@ -844,7 +948,26 @@ impl Halloy {
 
                                 // If server already exists, we only want to join the new channels
                                 if let Some(entry) = existing_entry {
-                                    self.clients.join(&entry.server, &config.channels);
+                                    let chantypes = self.clients.get_chantypes(&server);
+                                    let statusmsg = self.clients.get_statusmsg(&server);
+                                    let casemapping = self.clients.get_casemapping(&server);
+
+                                    self.clients.join(
+                                        &entry.server,
+                                        &config
+                                            .channels
+                                            .iter()
+                                            .filter_map(|channel| {
+                                                target::Channel::parse(
+                                                    channel,
+                                                    chantypes,
+                                                    statusmsg,
+                                                    casemapping,
+                                                )
+                                                .ok()
+                                            })
+                                            .collect::<Vec<_>>(),
+                                    );
                                 } else {
                                     self.servers.insert(server, config);
                                 }
@@ -868,13 +991,13 @@ impl Halloy {
                 if id == self.main_window.id {
                     match event {
                         window::Event::Moved(position) => {
-                            self.main_window.position = Some(position)
+                            self.main_window.position = Some(position);
                         }
                         window::Event::Resized(size) => self.main_window.size = size,
                         window::Event::Focused => self.main_window.focused = true,
                         window::Event::Unfocused => self.main_window.focused = false,
                         window::Event::Opened { position, size } => {
-                            self.main_window.opened(position, size)
+                            self.main_window.opened(position, size);
                         }
                         window::Event::CloseRequested => {
                             if let Screen::Dashboard(dashboard) = &mut self.screen {
@@ -885,10 +1008,22 @@ impl Halloy {
                         }
                     }
 
-                    Task::perform(
+                    let mut tasks = vec![Task::perform(
                         data::Window::from(self.main_window).save(),
                         Message::WindowSettingsSaved,
-                    )
+                    )];
+
+                    if let Some(Screen::Dashboard(dashboard)) =
+                        matches!(event, window::Event::Focused).then_some(&mut self.screen)
+                    {
+                        tasks.push(
+                            dashboard
+                                .focus_window(self.main_window.id)
+                                .map(Message::Dashboard),
+                        );
+                    }
+
+                    Task::batch(tasks)
                 } else if let Screen::Dashboard(dashboard) = &mut self.screen {
                     dashboard
                         .handle_window_event(id, event, &mut self.theme)
@@ -899,7 +1034,7 @@ impl Halloy {
             }
             Message::WindowSettingsSaved(result) => {
                 if let Err(err) = result {
-                    log::error!("window settings failed to save: {:?}", err)
+                    log::error!("window settings failed to save: {:?}", err);
                 }
 
                 Task::none()
@@ -945,13 +1080,7 @@ impl Halloy {
         let content = if id == self.main_window.id {
             let screen = match &self.screen {
                 Screen::Dashboard(dashboard) => dashboard
-                    .view(
-                        &self.clients,
-                        &self.version,
-                        &self.config,
-                        &self.theme,
-                        &self.main_window,
-                    )
+                    .view(&self.clients, &self.version, &self.config, &self.theme)
                     .map(Message::Dashboard),
                 Screen::Help(help) => help.view().map(Message::Help),
                 Screen::Welcome(welcome) => welcome.view().map(Message::Welcome),
@@ -975,13 +1104,7 @@ impl Halloy {
             }
         } else if let Screen::Dashboard(dashboard) = &self.screen {
             dashboard
-                .view_window(
-                    id,
-                    &self.clients,
-                    &self.config,
-                    &self.theme,
-                    &self.main_window,
-                )
+                .view_window(id, &self.clients, &self.config, &self.theme)
                 .map(Message::Dashboard)
         } else {
             column![].into()
@@ -1016,15 +1139,19 @@ impl Halloy {
         )
         .map(Message::Stream);
 
-        Subscription::batch(vec![
+        let mut subscriptions = vec![
             url::listen().map(Message::RouteReceived),
             events().map(|(window, event)| Message::Event(window, event)),
             window::events().map(|(window, event)| Message::Window(window, event)),
-            // Enable once dark_light has a proper way to detect appereance changes without spiking CPU.
-            // See: https://github.com/frewsxcv/rust-dark-light/issues/47
-            // appearance::subscription().map(Message::AppearanceChange),
             tick,
             streams,
-        ])
+        ];
+
+        // We only want to listen for appearance changes if user has dynamic themes.
+        if self.config.appearance.selected.is_dynamic() {
+            subscriptions.push(appearance::subscription().map(Message::AppearanceChange));
+        }
+
+        Subscription::batch(subscriptions)
     }
 }
