@@ -1,9 +1,8 @@
-use data::{file_transfer, history, Config};
+use data::{file_transfer, history, preview, Config};
 use iced::widget::{button, center, container, pane_grid, row, text};
 
 use crate::buffer::{self, Buffer};
 use crate::widget::tooltip;
-use crate::window::{self, Window};
 use crate::{icon, theme, widget, Theme};
 
 use super::sidebar;
@@ -24,60 +23,53 @@ pub enum Message {
     ScrollToBottom,
 }
 
-#[derive(Clone)]
+#[derive(Clone, Debug)]
 pub struct Pane {
     pub buffer: Buffer,
     title_bar: TitleBar,
-    pub settings: buffer::Settings,
 }
 
 #[derive(Debug, Clone, Default)]
 pub struct TitleBar {}
 
 impl Pane {
-    pub fn new(buffer: Buffer, config: &Config) -> Self {
-        Self::with_settings(buffer, buffer::Settings::from(config.buffer.clone()))
-    }
-
-    pub fn with_settings(buffer: Buffer, settings: buffer::Settings) -> Self {
+    pub fn new(buffer: Buffer) -> Self {
         Self {
             buffer,
             title_bar: TitleBar::default(),
-            settings,
         }
     }
 
     pub fn view<'a>(
         &'a self,
         id: pane_grid::Pane,
-        window: window::Id,
         panes: usize,
         is_focused: bool,
         maximized: bool,
         clients: &'a data::client::Map,
         file_transfers: &'a file_transfer::Manager,
         history: &'a history::Manager,
+        previews: &'a preview::Collection,
         sidebar: &'a sidebar::Sidebar,
         config: &'a Config,
         theme: &'a Theme,
-        main_window: &'a Window,
+        settings: Option<&'a buffer::Settings>,
+        is_popout: bool,
     ) -> widget::Content<'a, Message> {
-        let is_popout = window != main_window.id;
-
         let title_bar_text = match &self.buffer {
-            Buffer::Empty => "".to_string(),
+            Buffer::Empty => String::new(),
             Buffer::Channel(state) => {
-                let channel = &state.channel;
+                let channel = state.target.as_str();
                 let server = &state.server;
                 let users = clients
-                    .get_channel_users(&state.server, &state.channel)
+                    .get_channel_users(&state.server, &state.target)
                     .len();
 
                 format!("{channel} @ {server} - {users} users")
             }
             Buffer::Server(state) => state.server.to_string(),
             Buffer::Query(state) => {
-                let nick = &state.nick;
+                let nick = state.target.as_str();
                 let server = &state.server;
 
                 format!("{nick} @ {server}")
@@ -95,9 +87,10 @@ impl Pane {
             is_focused,
             maximized,
             clients,
-            &self.settings,
+            settings,
             config.tooltips,
             is_popout,
+            config,
         );
 
         let content = self
@@ -106,7 +99,8 @@ impl Pane {
                 clients,
                 file_transfers,
                 history,
-                &self.settings,
+                previews,
+                settings,
                 config,
                 theme,
                 is_focused,
@@ -123,13 +117,13 @@ impl Pane {
         match &self.buffer {
             Buffer::Empty => None,
             Buffer::Channel(state) => Some(history::Resource {
-                kind: history::Kind::Channel(state.server.clone(), state.channel.clone()),
+                kind: history::Kind::Channel(state.server.clone(), state.target.clone()),
             }),
             Buffer::Server(state) => Some(history::Resource {
                 kind: history::Kind::Server(state.server.clone()),
             }),
             Buffer::Query(state) => Some(history::Resource {
-                kind: history::Kind::Query(state.server.clone(), state.nick.clone()),
+                kind: history::Kind::Query(state.server.clone(), state.target.clone()),
             }),
             Buffer::FileTransfers(_) => None,
             Buffer::Logs(_) => Some(history::Resource::logs()),
@@ -137,8 +131,16 @@ impl Pane {
         }
     }
 
-    pub fn update_settings(&mut self, f: impl FnOnce(&mut buffer::Settings)) {
-        f(&mut self.settings);
+    pub fn visible_urls(&self) -> Vec<&url::Url> {
+        match &self.buffer {
+            Buffer::Channel(channel) => channel.scroll_view.visible_urls().collect(),
+            Buffer::Query(query) => query.scroll_view.visible_urls().collect(),
+            Buffer::Empty
+            | Buffer::Server(_)
+            | Buffer::FileTransfers(_)
+            | Buffer::Logs(_)
+            | Buffer::Highlights(_) => vec![],
+        }
     }
 }
 
@@ -152,9 +154,10 @@ impl TitleBar {
         _is_focused: bool,
         maximized: bool,
         clients: &'a data::client::Map,
-        settings: &'a buffer::Settings,
+        settings: Option<&'a buffer::Settings>,
         show_tooltips: bool,
         is_popout: bool,
+        config: &'a Config,
     ) -> widget::TitleBar<'a, Message> {
         // Pane controls.
         let mut controls = row![].spacing(2);
@@ -179,15 +182,18 @@ impl TitleBar {
             }
 
             // Show topic button only if there is a topic to show
-            if let Some(topic) = clients.get_channel_topic(&state.server, &state.channel) {
+            if let Some(topic) = clients.get_channel_topic(&state.server, &state.target) {
                 if topic.content.is_some() {
+                    let topic_enabled = settings
+                        .map_or(config.buffer.channel.topic.enabled, |settings| settings.channel.topic.enabled);
+
                     let topic_button = button(center(icon::topic()))
                         .padding(5)
                         .width(22)
                         .height(22)
                         .on_press(Message::ToggleShowTopic)
-                        .style(|theme, status| {
-                            theme::button::secondary(theme, status, settings.channel.topic.enabled)
+                        .style(move |theme, status| {
+                            theme::button::secondary(theme, status, topic_enabled)
                         });
 
                     let topic_button_with_tooltip = tooltip(
@@ -200,13 +206,16 @@ impl TitleBar {
                 }
             }
 
+            let nicklist_enabled = settings
+                .map_or(config.buffer.channel.nicklist.enabled, |settings| settings.channel.nicklist.enabled);
+
             let nicklist_button = button(center(icon::people()))
                 .padding(5)
                 .width(22)
                 .height(22)
                 .on_press(Message::ToggleShowUserList)
-                .style(|theme, status| {
-                    theme::button::secondary(theme, status, settings.channel.nicklist.enabled)
+                .style(move |theme, status| {
+                    theme::button::secondary(theme, status, nicklist_enabled)
                 });
 
             let nicklist_button_with_tooltip = tooltip(
@@ -233,13 +242,7 @@ impl TitleBar {
 
             let maximize_button_with_tooltip = tooltip(
                 maximize_button,
-                show_tooltips.then_some({
-                    if maximized {
-                        "Restore"
-                    } else {
-                        "Maximize"
-                    }
-                }),
+                show_tooltips.then_some(if maximized { "Restore" } else { "Maximize" }),
                 tooltip::Position::Bottom,
             );
 
@@ -319,20 +322,17 @@ impl From<Pane> for data::Pane {
         let buffer = match pane.buffer {
             Buffer::Empty => return data::Pane::Empty,
             Buffer::Channel(state) => {
-                data::Buffer::Upstream(buffer::Upstream::Channel(state.server, state.channel))
+                data::Buffer::Upstream(buffer::Upstream::Channel(state.server, state.target))
             }
             Buffer::Server(state) => data::Buffer::Upstream(buffer::Upstream::Server(state.server)),
             Buffer::Query(state) => {
-                data::Buffer::Upstream(buffer::Upstream::Query(state.server, state.nick))
+                data::Buffer::Upstream(buffer::Upstream::Query(state.server, state.target))
             }
             Buffer::FileTransfers(_) => data::Buffer::Internal(buffer::Internal::FileTransfers),
             Buffer::Logs(_) => data::Buffer::Internal(buffer::Internal::Logs),
             Buffer::Highlights(_) => data::Buffer::Internal(buffer::Internal::Highlights),
         };
 
-        data::Pane::Buffer {
-            buffer,
-            settings: pane.settings,
-        }
+        data::Pane::Buffer { buffer }
     }
 }
