@@ -10,20 +10,20 @@ use std::time::Duration;
 
 use tokio::time;
 
-use super::Panes;
-use crate::widget::{context_menu, Element, Text};
+use super::{Focus, Panes};
+use crate::widget::{context_menu, double_pass, Element, Text};
 use crate::{icon, theme, window};
 
 const CONFIG_RELOAD_DELAY: Duration = Duration::from_secs(1);
 
 #[derive(Debug, Clone)]
 pub enum Message {
-    Open(buffer::Upstream),
+    New(buffer::Upstream),
     Popout(buffer::Upstream),
     Focus(window::Id, pane_grid::Pane),
-    Replace(window::Id, buffer::Upstream, pane_grid::Pane),
+    Replace(buffer::Upstream),
     Close(window::Id, pane_grid::Pane),
-    Swap(window::Id, pane_grid::Pane, window::Id, pane_grid::Pane),
+    Swap(window::Id, pane_grid::Pane),
     Leave(buffer::Upstream),
     ToggleInternalBuffer(buffer::Internal),
     ToggleCommandBar,
@@ -33,17 +33,16 @@ pub enum Message {
     OpenReleaseWebsite,
     OpenDocumentation,
     ReloadComplete,
-    Noop,
 }
 
 #[derive(Debug, Clone)]
 pub enum Event {
-    Open(buffer::Upstream),
+    New(buffer::Upstream),
     Popout(buffer::Upstream),
     Focus(window::Id, pane_grid::Pane),
-    Replace(window::Id, buffer::Upstream, pane_grid::Pane),
+    Replace(buffer::Upstream),
     Close(window::Id, pane_grid::Pane),
-    Swap(window::Id, pane_grid::Pane, window::Id, pane_grid::Pane),
+    Swap(window::Id, pane_grid::Pane),
     Leave(buffer::Upstream),
     ToggleInternalBuffer(buffer::Internal),
     ToggleCommandBar,
@@ -74,22 +73,17 @@ impl Sidebar {
     }
 
     pub fn toggle_visibility(&mut self) {
-        self.hidden = !self.hidden
+        self.hidden = !self.hidden;
     }
 
     pub fn update(&mut self, message: Message) -> (Task<Message>, Option<Event>) {
         match message {
-            Message::Open(source) => (Task::none(), Some(Event::Open(source))),
+            Message::New(source) => (Task::none(), Some(Event::New(source))),
             Message::Popout(source) => (Task::none(), Some(Event::Popout(source))),
             Message::Focus(window, pane) => (Task::none(), Some(Event::Focus(window, pane))),
-            Message::Replace(window, source, pane) => {
-                (Task::none(), Some(Event::Replace(window, source, pane)))
-            }
+            Message::Replace(source) => (Task::none(), Some(Event::Replace(source))),
             Message::Close(window, pane) => (Task::none(), Some(Event::Close(window, pane))),
-            Message::Swap(from_window, from_pane, to_window, to_pane) => (
-                Task::none(),
-                Some(Event::Swap(from_window, from_pane, to_window, to_pane)),
-            ),
+            Message::Swap(window, pane) => (Task::none(), Some(Event::Swap(window, pane))),
             Message::Leave(buffer) => (Task::none(), Some(Event::Leave(buffer))),
             Message::ToggleInternalBuffer(buffer) => {
                 (Task::none(), Some(Event::ToggleInternalBuffer(buffer)))
@@ -101,7 +95,7 @@ impl Sidebar {
                 (Task::perform(Config::load(), Message::ConfigReloaded), None)
             }
             Message::ConfigReloaded(config) => (
-                Task::perform(time::sleep(CONFIG_RELOAD_DELAY), |_| {
+                Task::perform(time::sleep(CONFIG_RELOAD_DELAY), |()| {
                     Message::ReloadComplete
                 }),
                 Some(Event::ConfigReloaded(config)),
@@ -111,7 +105,6 @@ impl Sidebar {
                 self.reloading_config = false;
                 (Task::none(), None)
             }
-            Message::Noop => (Task::none(), None),
             Message::OpenDocumentation => (Task::none(), Some(Event::OpenDocumentation)),
         }
     }
@@ -122,10 +115,7 @@ impl Sidebar {
         file_transfers: &'a file_transfer::Manager,
         version: &'a Version,
     ) -> Element<'a, Message> {
-        let base = button(icon::menu())
-            .padding(5)
-            .width(Length::Shrink)
-            .on_press(Message::Noop);
+        let base = button(icon::menu()).padding(5).width(Length::Shrink);
 
         let menu = Menu::list();
 
@@ -240,157 +230,160 @@ impl Sidebar {
         clients: &data::client::Map,
         history: &'a history::Manager,
         panes: &'a Panes,
-        focus: Option<(window::Id, pane_grid::Pane)>,
-        config: data::config::Sidebar,
-        keyboard: &'a data::config::Keyboard,
+        focus: Focus,
+        config: &'a Config,
         file_transfers: &'a file_transfer::Manager,
         version: &'a Version,
-        main_window: window::Id,
     ) -> Option<Element<'a, Message>> {
         if self.hidden {
             return None;
         }
 
-        let user_menu_button = config
-            .show_user_menu
-            .then(|| self.user_menu_button(keyboard, file_transfers, version));
+        let content = |width| {
+            let user_menu_button = config
+                .sidebar
+                .show_user_menu
+                .then(|| self.user_menu_button(&config.keyboard, file_transfers, version));
 
-        let mut buffers = vec![];
+            let mut buffers = vec![];
 
-        for (i, (server, state)) in clients.iter().enumerate() {
-            match state {
-                data::client::State::Disconnected => {
-                    buffers.push(upstream_buffer_button(
-                        main_window,
+            for (i, (server, state)) in clients.iter().enumerate() {
+                let button = |buffer: buffer::Upstream, connected: bool, has_unread: bool| {
+                    upstream_buffer_button(
                         panes,
                         focus,
-                        buffer::Upstream::Server(server.clone()),
-                        false,
-                        config.buffer_action,
-                        config.buffer_focused_action,
-                        config.position,
-                        config.unread_indicator,
-                        history.has_unread(&history::Kind::Server(server.clone())),
-                    ));
-                }
-                data::client::State::Ready(connection) => {
-                    buffers.push(upstream_buffer_button(
-                        main_window,
-                        panes,
-                        focus,
-                        buffer::Upstream::Server(server.clone()),
-                        true,
-                        config.buffer_action,
-                        config.buffer_focused_action,
-                        config.position,
-                        config.unread_indicator,
-                        history.has_unread(&history::Kind::Server(server.clone())),
-                    ));
+                        buffer,
+                        connected,
+                        config.actions.sidebar.buffer,
+                        config.actions.sidebar.focused_buffer,
+                        config.sidebar.position,
+                        config.sidebar.unread_indicator,
+                        has_unread,
+                        width,
+                    )
+                };
 
-                    for channel in connection.channels() {
-                        buffers.push(upstream_buffer_button(
-                            main_window,
-                            panes,
-                            focus,
-                            buffer::Upstream::Channel(server.clone(), channel.clone()),
-                            true,
-                            config.buffer_action,
-                            config.buffer_focused_action,
-                            config.position,
-                            config.unread_indicator,
-                            history.has_unread(&history::Kind::Channel(
-                                server.clone(),
-                                channel.clone(),
-                            )),
+                match state {
+                    data::client::State::Disconnected => {
+                        // Disconnected server.
+                        buffers.push(button(
+                            buffer::Upstream::Server(server.clone()),
+                            false,
+                            history.has_unread(&history::Kind::Server(server.clone())),
                         ));
                     }
-
-                    let queries = history.get_unique_queries(server);
-                    for user in queries {
-                        buffers.push(upstream_buffer_button(
-                            main_window,
-                            panes,
-                            focus,
-                            buffer::Upstream::Query(server.clone(), user.clone()),
+                    data::client::State::Ready(connection) => {
+                        // Connected server.
+                        buffers.push(button(
+                            buffer::Upstream::Server(server.clone()),
                             true,
-                            config.buffer_action,
-                            config.buffer_focused_action,
-                            config.position,
-                            config.unread_indicator,
-                            history.has_unread(&history::Kind::Query(server.clone(), user.clone())),
+                            history.has_unread(&history::Kind::Server(server.clone())),
                         ));
-                    }
 
-                    // Separator between servers.
-                    if config.position.is_horizontal() {
-                        if i + 1 < clients.len() {
-                            buffers.push(
-                                container(vertical_rule(1))
-                                    .padding(padding::top(6))
-                                    .height(20)
-                                    .width(12)
-                                    .align_x(Alignment::Center)
-                                    .into(),
-                            )
+                        // Channels from the connected server.
+                        for channel in connection.channels() {
+                            buffers.push(button(
+                                buffer::Upstream::Channel(server.clone(), channel.clone()),
+                                true,
+                                history.has_unread(&history::Kind::Channel(
+                                    server.clone(),
+                                    channel.clone(),
+                                )),
+                            ));
+                        }
+
+                        // Queries from the connected server.
+                        let queries = history.get_unique_queries(server);
+                        for query in queries {
+                            let query = clients.resolve_query(server, query).unwrap_or(query);
+
+                            buffers.push(button(
+                                buffer::Upstream::Query(server.clone(), query.clone()),
+                                true,
+                                history.has_unread(&history::Kind::Query(
+                                    server.clone(),
+                                    query.clone(),
+                                )),
+                            ));
+                        }
+
+                        // Separator between servers.
+                        if config.sidebar.position.is_horizontal() {
+                            if i + 1 < clients.len() {
+                                buffers.push(
+                                    container(vertical_rule(1))
+                                        .padding(padding::top(6))
+                                        .height(20)
+                                        .width(12)
+                                        .align_x(Alignment::Center)
+                                        .into(),
+                                );
+                            }
+                        } else {
+                            buffers.push(vertical_space().height(12).into());
                         }
-                    } else {
-                        buffers.push(vertical_space().height(12).into());
                     }
                 }
             }
-        }
 
-        match config.position {
-            sidebar::Position::Left | sidebar::Position::Right => {
-                let content = column![Scrollable::new(Column::with_children(buffers).spacing(1))
-                    .direction(scrollable::Direction::Vertical(
-                        iced::widget::scrollable::Scrollbar::default()
-                            .width(0)
-                            .scroller_width(0),
-                    )),];
+            match config.sidebar.position {
+                sidebar::Position::Left | sidebar::Position::Right => {
+                    // Add buffers to a column.
+                    let buffers =
+                        column![Scrollable::new(Column::with_children(buffers).spacing(1))
+                            .direction(scrollable::Direction::Vertical(
+                                scrollable::Scrollbar::default().width(2).scroller_width(2)
+                            ))];
 
-                let body =
-                    column![container(content).height(Length::Fill)].push_maybe(user_menu_button);
-                let padding = match config.position {
-                    sidebar::Position::Left => padding::top(8).bottom(6).left(6),
-                    sidebar::Position::Right => padding::top(8).bottom(6).right(6),
-                    _ => iced::Padding::default(),
-                };
+                    // Wrap buffers in a column with user_menu_button
+                    let content = column![container(buffers).height(Length::Fill)]
+                        .push_maybe(user_menu_button);
 
-                Some(
-                    container(body)
-                        .height(Length::Fill)
-                        .center_x(Length::Shrink)
-                        .padding(padding)
-                        .max_width(config.width)
-                        .into(),
-                )
+                    container(content)
+                }
+                sidebar::Position::Top | sidebar::Position::Bottom => {
+                    // Add buffers to a row.
+                    let buffers = row![Scrollable::new(Row::with_children(buffers).spacing(2))
+                        .direction(scrollable::Direction::Horizontal(
+                            scrollable::Scrollbar::default().width(2).scroller_width(2)
+                        ))];
+
+                    // Wrap buffers in a row with user_menu_button
+                    let content = row![container(buffers).width(Length::Fill)]
+                        .push_maybe(user_menu_button)
+                        .align_y(Alignment::Center);
+
+                    container(content)
+                }
             }
-            sidebar::Position::Top | sidebar::Position::Bottom => {
-                let content = row![Scrollable::new(Row::with_children(buffers).spacing(2))
-                    .direction(scrollable::Direction::Horizontal(
-                        iced::widget::scrollable::Scrollbar::default()
-                            .width(0)
-                            .scroller_width(0),
-                    )),];
+        };
 
-                let body: Row<Message, theme::Theme> = row![container(content).width(Length::Fill)]
-                    .push_maybe(user_menu_button)
-                    .align_y(Alignment::Center);
-                let padding = match config.position {
-                    sidebar::Position::Top => padding::top(8).left(8).right(8),
-                    sidebar::Position::Bottom => padding::bottom(8).left(8).right(8),
-                    _ => iced::Padding::default(),
-                };
+        let padding = match config.sidebar.position {
+            sidebar::Position::Left => padding::top(8).bottom(6).left(6),
+            sidebar::Position::Right => padding::top(8).bottom(6).right(6),
+            sidebar::Position::Top => padding::top(8).left(6).right(6),
+            sidebar::Position::Bottom => padding::bottom(8).left(6).right(6),
+        };
 
-                Some(
-                    container(body)
-                        .center_x(Length::Shrink)
-                        .padding(padding)
-                        .into(),
+        let content = if config.sidebar.position.is_horizontal() {
+            container(content(Length::Shrink).width(Length::Fill).padding(padding)).into()
+        } else {
+            let first_pass = content(Length::Shrink);
+            let second_pass = content(Length::Fill);
+
+            container(double_pass(first_pass, second_pass))
+                .max_width(
+                    config
+                        .sidebar
+                        .max_width
+                        .map_or(f32::INFINITY, f32::from),
                 )
-            }
-        }
+                .width(Length::Shrink)
+                .padding(padding)
+                .into()
+        };
+
+        Some(content)
     }
 }
 
@@ -427,9 +420,9 @@ impl Menu {
 enum Entry {
     NewPane,
     Popout,
-    Replace(window::Id, pane_grid::Pane),
+    Replace,
     Close(window::Id, pane_grid::Pane),
-    Swap(window::Id, pane_grid::Pane, window::Id, pane_grid::Pane),
+    Swap(window::Id, pane_grid::Pane),
     Leave,
 }
 
@@ -437,27 +430,14 @@ impl Entry {
     fn list(
         num_panes: usize,
         open: Option<(window::Id, pane_grid::Pane)>,
-        focus: Option<(window::Id, pane_grid::Pane)>,
+        focus: Focus,
     ) -> Vec<Self> {
-        match (open, focus) {
-            (None, None) => vec![Entry::NewPane, Entry::Popout, Entry::Leave],
-            (None, Some(close)) => {
-                vec![
-                    Entry::NewPane,
-                    Entry::Popout,
-                    Entry::Replace(close.0, close.1),
-                    Entry::Leave,
-                ]
-            }
-            (Some(open), None) => (num_panes > 1)
-                .then_some(Entry::Close(open.0, open.1))
+        match open {
+            None => vec![Entry::NewPane, Entry::Popout, Entry::Replace, Entry::Leave],
+            Some((window, pane)) => (num_panes > 1)
+                .then_some(Entry::Close(window, pane))
                 .into_iter()
-                .chain(Some(Entry::Leave))
-                .collect(),
-            (Some(open), Some(focus)) => (num_panes > 1)
-                .then_some(Entry::Close(open.0, open.1))
-                .into_iter()
-                .chain((open != focus).then_some(Entry::Swap(open.0, open.1, focus.0, focus.1)))
+                .chain((Focus { window, pane } != focus).then_some(Entry::Swap(window, pane)))
                 .chain(Some(Entry::Leave))
                 .collect(),
         }
@@ -465,9 +445,8 @@ impl Entry {
 }
 
 fn upstream_buffer_button(
-    main_window: window::Id,
     panes: &Panes,
-    focus: Option<(window::Id, pane_grid::Pane)>,
+    focus: Focus,
     buffer: buffer::Upstream,
     connected: bool,
     buffer_action: BufferAction,
@@ -475,18 +454,19 @@ fn upstream_buffer_button(
     position: sidebar::Position,
     unread_indicator: sidebar::UnreadIndicator,
     has_unread: bool,
+    width: Length,
 ) -> Element<Message> {
-    let open = panes
-        .iter(main_window)
-        .find_map(|(window_id, pane, state)| {
-            (state.buffer.upstream() == Some(&buffer)).then_some((window_id, pane))
-        });
-    let is_focused = panes
-        .iter(main_window)
-        .find_map(|(window_id, pane, state)| {
-            (Some((window_id, pane)) == focus && state.buffer.upstream() == Some(&buffer))
-                .then_some((window_id, pane))
-        });
+    let open = panes.iter().find_map(|(window_id, pane, state)| {
+        (state.buffer.upstream() == Some(&buffer)).then_some((window_id, pane))
+    });
+    let is_focused = panes.iter().find_map(|(window_id, pane, state)| {
+        (Focus {
+            window: window_id,
+            pane,
+        } == focus
+            && state.buffer.upstream() == Some(&buffer))
+        .then_some((window_id, pane))
+    });
 
     let show_unread_indicator =
         has_unread && matches!(unread_indicator, sidebar::UnreadIndicator::Dot);
@@ -540,13 +520,13 @@ fn upstream_buffer_button(
             )
             .push(unread_dot_indicator_spacing)
             .push(
-                text(channel.clone())
+                text(channel.to_string())
                     .style(buffer_title_style)
                     .shaping(text::Shaping::Advanced),
             )
             .push(horizontal_space().width(3))
             .align_y(iced::Alignment::Center),
-        buffer::Upstream::Query(_, nick) => row![]
+        buffer::Upstream::Query(_, query) => row![]
             .push(horizontal_space().width(3))
             .push_maybe(
                 show_unread_indicator
@@ -554,7 +534,7 @@ fn upstream_buffer_button(
             )
             .push(unread_dot_indicator_spacing)
             .push(
-                text(nick.to_string())
+                text(query.to_string())
                     .style(buffer_title_style)
                     .shaping(text::Shaping::Advanced),
             )
@@ -562,15 +542,8 @@ fn upstream_buffer_button(
             .align_y(iced::Alignment::Center),
     };
 
-    let width = if position.is_horizontal() {
-        Length::Shrink
-    } else {
-        Length::Fill
-    };
-
-    let base = button(row)
+    let base = button(row.width(width))
         .padding(5)
-        .width(width)
         .style(move |theme, status| {
             theme::button::sidebar_buffer(theme, status, is_focused.is_some(), open.is_some())
         })
@@ -590,13 +563,8 @@ fn upstream_buffer_button(
                         Some(Message::Focus(window, pane))
                     } else {
                         match buffer_action {
-                            BufferAction::NewPane => Some(Message::Open(buffer.clone())),
-                            BufferAction::ReplacePane => match focus {
-                                Some((window, pane)) => {
-                                    Some(Message::Replace(window, buffer.clone(), pane))
-                                }
-                                None => Some(Message::Open(buffer.clone())),
-                            },
+                            BufferAction::NewPane => Some(Message::New(buffer.clone())),
+                            BufferAction::ReplacePane => Some(Message::Replace(buffer.clone())),
                             BufferAction::NewWindow => Some(Message::Popout(buffer.clone())),
                         }
                     }
@@ -609,19 +577,15 @@ fn upstream_buffer_button(
     if entries.is_empty() || !connected {
         base.into()
     } else {
-        context_menu(Default::default(), base, entries, move |entry, length| {
+        context_menu(context_menu::MouseButton::default(), base, entries, move |entry, length| {
             let (content, message) = match entry {
-                Entry::NewPane => ("Open in new pane", Message::Open(buffer.clone())),
+                Entry::NewPane => ("Open in new pane", Message::New(buffer.clone())),
                 Entry::Popout => ("Open in new window", Message::Popout(buffer.clone())),
-                Entry::Replace(window, pane) => (
-                    "Replace current pane",
-                    Message::Replace(window, buffer.clone(), pane),
-                ),
+                Entry::Replace => ("Replace current pane", Message::Replace(buffer.clone())),
                 Entry::Close(window, pane) => ("Close pane", Message::Close(window, pane)),
-                Entry::Swap(from_window, from_pane, to_window, to_pane) => (
-                    "Swap with current pane",
-                    Message::Swap(from_window, from_pane, to_window, to_pane),
-                ),
+                Entry::Swap(window, pane) => {
+                    ("Swap with current pane", Message::Swap(window, pane))
+                }
                 Entry::Leave => (
                     match &buffer {
                         buffer::Upstream::Server(_) => "Leave server",
