@@ -1,19 +1,19 @@
 use std::path::PathBuf;
 use std::time::Duration;
 
-use data::{url, Config};
+use data::{Config, url};
 use futures::TryFutureExt;
 use iced::alignment::Vertical;
 use iced::widget::text::LineHeight;
 use iced::widget::{button, center, column, container, row, text_input};
-use iced::{alignment, clipboard, Length::*};
 use iced::{Color, Length};
+use iced::{Length::*, alignment, clipboard};
 use iced::{Task, Vector};
 use strum::IntoEnumIterator;
 use tokio::time;
 
 use crate::theme::{self, Colors, Theme};
-use crate::widget::{color_picker, combo_box, tooltip, Element};
+use crate::widget::{Element, color_picker, combo_box, tooltip};
 use crate::window::{self, Window};
 use crate::{icon, widget};
 
@@ -34,6 +34,7 @@ pub enum Message {
     Revert,
     Clear,
     Copy,
+    Share,
     SavePath(Option<PathBuf>),
     Saved(Result<(), String>),
     ClearSaveResult,
@@ -100,7 +101,7 @@ impl ThemeEditor {
                 self.hex_input = None;
                 self.combo_box = combo_box::State::new(components().collect());
 
-                self.component = component
+                self.component = component;
             }
             Message::HexInput(input) => {
                 if let Some(color) = theme::hex_to_color(&input) {
@@ -115,17 +116,9 @@ impl ThemeEditor {
             }
             Message::Save => {
                 let task = async move {
-                    // TODO: rfd `set_directory` can't be used in combination with `set_file_name` on macOS.
-                    // https://github.com/PolyMeilex/rfd/issues/214
-
-                    let mut dialog =
-                        rfd::AsyncFileDialog::new().set_directory(Config::themes_dir());
-
-                    if !cfg!(target_os = "macos") {
-                        dialog = dialog.set_file_name("custom-theme.toml");
-                    }
-
-                    dialog
+                    rfd::AsyncFileDialog::new()
+                        .set_directory(Config::themes_dir())
+                        .set_file_name("custom-theme.toml")
                         .save_file()
                         .await
                         .map(|handle| handle.path().to_path_buf())
@@ -171,11 +164,17 @@ impl ThemeEditor {
                 return (
                     Task::batch(vec![
                         clipboard::write(url),
-                        Task::perform(time::sleep(Duration::from_secs(2)), |_| Message::ClearCopy),
+                        Task::perform(time::sleep(Duration::from_secs(2)), |()| Message::ClearCopy),
                     ]),
                     None,
                 );
             }
+            Message::Share => {
+                let url = url::theme_submit(theme.colors());
+                let _ = open::that_detached(url);
+
+                return (Task::none(), None);
+            }
             Message::SavePath(None) => {}
             Message::SavePath(Some(path)) => {
                 log::debug!("Saving theme to {path:?}");
@@ -192,18 +191,18 @@ impl ThemeEditor {
                 self.save_result = Some(false);
 
                 return (
-                    Task::perform(time::sleep(Duration::from_secs(2)), |_| {
+                    Task::perform(time::sleep(Duration::from_secs(2)), |()| {
                         Message::ClearSaveResult
                     }),
                     None,
                 );
             }
-            Message::Saved(Ok(_)) => {
+            Message::Saved(Ok(())) => {
                 log::debug!("Theme saved");
                 self.save_result = Some(true);
 
                 return (
-                    Task::perform(time::sleep(Duration::from_secs(2)), |_| {
+                    Task::perform(time::sleep(Duration::from_secs(2)), |()| {
                         Message::ClearSaveResult
                     }),
                     Some(Event::ReloadThemes),
@@ -268,6 +267,8 @@ impl ThemeEditor {
             icon(icon::copy(), "Copy Theme to URL", Message::Copy)
         };
 
+        let share = icon(icon::share(), "Share Theme with community", Message::Share);
+
         let color_picker = color_picker(color, Message::Color);
 
         let content = column![
@@ -276,6 +277,7 @@ impl ThemeEditor {
                 container(hex_input).width(80),
                 undo,
                 copy,
+                share
             ]
             .align_y(Vertical::Center)
             .spacing(4),
@@ -425,7 +427,7 @@ impl General {
             General::Border => colors.border = color.unwrap_or(Color::TRANSPARENT),
             General::HorizontalRule => colors.horizontal_rule = color.unwrap_or(Color::TRANSPARENT),
             General::UnreadIndicator => {
-                colors.unread_indicator = color.unwrap_or(Color::TRANSPARENT)
+                colors.unread_indicator = color.unwrap_or(Color::TRANSPARENT);
             }
         }
     }
@@ -510,10 +512,10 @@ impl Buffer {
             Buffer::Action => colors.action = color.unwrap_or(Color::TRANSPARENT),
             Buffer::Background => colors.background = color.unwrap_or(Color::TRANSPARENT),
             Buffer::BackgroundTextInput => {
-                colors.background_text_input = color.unwrap_or(Color::TRANSPARENT)
+                colors.background_text_input = color.unwrap_or(Color::TRANSPARENT);
             }
             Buffer::BackgroundTitleBar => {
-                colors.background_title_bar = color.unwrap_or(Color::TRANSPARENT)
+                colors.background_title_bar = color.unwrap_or(Color::TRANSPARENT);
             }
             Buffer::Border => colors.border = color.unwrap_or(Color::TRANSPARENT),
             Buffer::BorderSelected => colors.border_selected = color.unwrap_or(Color::TRANSPARENT),
@@ -522,7 +524,7 @@ impl Buffer {
             Buffer::Nickname => colors.nickname = color.unwrap_or(Color::TRANSPARENT),
             Buffer::Selection => colors.selection = color.unwrap_or(Color::TRANSPARENT),
             Buffer::ServerMessages(server_messages) => {
-                server_messages.update(&mut colors.server_messages, color)
+                server_messages.update(&mut colors.server_messages, color);
             }
             Buffer::Timestamp => colors.timestamp = color.unwrap_or(Color::TRANSPARENT),
             Buffer::Topic => colors.topic = color.unwrap_or(Color::TRANSPARENT),
@@ -542,6 +544,9 @@ pub enum ServerMessages {
     ChangeHost,
     MonitoredOnline,
     MonitoredOffline,
+    StandardReplyFail,
+    StandardReplyWarn,
+    StandardReplyNote,
     Default,
 }
 
@@ -555,6 +560,9 @@ impl ServerMessages {
             ServerMessages::ChangeHost => colors.change_host,
             ServerMessages::MonitoredOnline => colors.monitored_online,
             ServerMessages::MonitoredOffline => colors.monitored_offline,
+            ServerMessages::StandardReplyFail => colors.standard_reply_fail,
+            ServerMessages::StandardReplyWarn => colors.standard_reply_warn,
+            ServerMessages::StandardReplyNote => colors.standard_reply_note,
             ServerMessages::Default => Some(colors.default),
         }
     }
@@ -568,6 +576,9 @@ impl ServerMessages {
             ServerMessages::ChangeHost => colors.change_host = color,
             ServerMessages::MonitoredOnline => colors.monitored_online = color,
             ServerMessages::MonitoredOffline => colors.monitored_offline = color,
+            ServerMessages::StandardReplyFail => colors.standard_reply_fail = color,
+            ServerMessages::StandardReplyWarn => colors.standard_reply_warn = color,
+            ServerMessages::StandardReplyNote => colors.standard_reply_note = color,
             ServerMessages::Default => colors.default = color.unwrap_or(Color::TRANSPARENT),
         }
     }
@@ -621,13 +632,13 @@ impl Button {
         match self {
             Button::Background => colors.background = color.unwrap_or(Color::TRANSPARENT),
             Button::BackgroundHover => {
-                colors.background_hover = color.unwrap_or(Color::TRANSPARENT)
+                colors.background_hover = color.unwrap_or(Color::TRANSPARENT);
             }
             Button::BackgroundSelected => {
-                colors.background_selected = color.unwrap_or(Color::TRANSPARENT)
+                colors.background_selected = color.unwrap_or(Color::TRANSPARENT);
             }
             Button::BackgroundSelectedHover => {
-                colors.background_selected_hover = color.unwrap_or(Color::TRANSPARENT)
+                colors.background_selected_hover = color.unwrap_or(Color::TRANSPARENT);
             }
         }
     }
