@@ -1,7 +1,7 @@
 use iced::advanced::renderer::{Quad, Renderer as _};
 use iced::widget::{column, container, row};
 use iced::{
-    advanced, border, event, mouse, touch, Color,
+    advanced, border, mouse, touch, Color,
     Length::{self, *},
     Point, Rectangle,
 };
@@ -60,7 +60,7 @@ fn bordered<'a, Message: 'a>(element: impl Into<Element<'a, Message>>) -> Contai
             border: border::rounded(2)
                 .width(1)
                 .color(theme.colors().general.border),
-            shadow: Default::default(),
+            shadow: iced::Shadow::default(),
         })
 }
 
@@ -79,11 +79,11 @@ fn preview<'a, Message: 'a>(color: Color) -> Element<'a, Message> {
                 renderer.fill_quad(
                     Quad {
                         bounds: layout.bounds(),
-                        border: Default::default(),
-                        shadow: Default::default(),
+                        border: iced::Border::default(),
+                        shadow: iced::Shadow::default(),
                     },
                     color,
-                )
+                );
             },
         )
         .into()
@@ -302,11 +302,11 @@ fn picker<'a, Message: 'a>(
     let color = data::appearance::theme::to_hsva(color);
 
     decorate(Space::new(width, height))
-        .on_event(
+        .update(
             move |state: &mut Option<Rectangle>,
                   _inner: &mut Element<'a, Message>,
                   _tree: &mut advanced::widget::Tree,
-                  event: iced::Event,
+                  event: &iced::Event,
                   layout: advanced::Layout<'_>,
                   cursor: advanced::mouse::Cursor,
                   _renderer: &Renderer,
@@ -371,8 +371,6 @@ fn picker<'a, Message: 'a>(
                     }
                     _ => {}
                 }
-
-                event::Status::Ignored
             },
         )
         .draw(
@@ -420,8 +418,8 @@ fn picker<'a, Message: 'a>(
                             renderer.fill_quad(
                                 Quad {
                                     bounds: Rectangle::new(top_left, Size::new(width, height)),
-                                    border: Default::default(),
-                                    shadow: Default::default(),
+                                    border: iced::Border::default(),
+                                    shadow: iced::Shadow::default(),
                                 },
                                 color,
                             );
@@ -443,8 +441,8 @@ fn picker<'a, Message: 'a>(
                                 width: 1.0,
                                 height: cell_height,
                             },
-                            border: Default::default(),
-                            shadow: Default::default(),
+                            border: iced::Border::default(),
+                            shadow: iced::Shadow::default(),
                         },
                         data::appearance::theme::from_hsva(color),
                     );
@@ -457,7 +455,7 @@ fn picker<'a, Message: 'a>(
                             border: border::rounded(handle.width / 2.0)
                                 .color(Color::BLACK)
                                 .width(1.0),
-                            shadow: Default::default(),
+                            shadow: iced::Shadow::default(),
                         },
                         Color::WHITE,
                     );
