@@ -5,7 +5,7 @@ use iced::advanced::{
 use iced::overlay::menu;
 use iced::widget::text::LineHeight;
 use iced::widget::{text_input, TextInput};
-use iced::{event, keyboard, Event, Length, Padding, Rectangle, Vector};
+use iced::{keyboard, window, Event, Length, Padding, Rectangle, Vector};
 
 use std::cell::RefCell;
 use std::fmt::Display;
@@ -276,7 +276,7 @@ where
     fn update_text_input(&self, tree: widget::Tree) {
         let mut inner = self.0.borrow_mut();
 
-        inner.update_text_input(tree)
+        inner.update_text_input(tree);
     }
 
     fn with_inner<O>(&self, f: impl FnOnce(&Inner<T>) -> O) -> O {
@@ -403,17 +403,17 @@ where
         })
     }
 
-    fn on_event(
+    fn update(
         &mut self,
         tree: &mut widget::Tree,
-        event: Event,
+        event: &Event,
         layout: Layout<'_>,
         cursor: mouse::Cursor,
         renderer: &Renderer,
         clipboard: &mut dyn Clipboard,
         shell: &mut Shell<'_, Message>,
         viewport: &Rectangle,
-    ) -> event::Status {
+    ) {
         let menu = tree.state.downcast_mut::<Menu<T>>();
 
         let started_focused = self.state.is_focused();
@@ -427,9 +427,9 @@ where
 
         // Provide it to the widget
         let mut tree = self.state.text_input_tree();
-        let mut event_status = self.text_input.on_event(
+        self.text_input.update(
             &mut tree,
-            event.clone(),
+            event,
             layout,
             cursor,
             renderer,
@@ -439,13 +439,22 @@ where
         );
         self.state.update_text_input(tree);
 
+        if local_shell.is_event_captured() {
+            shell.capture_event();
+        }
+
+        match local_shell.redraw_request() {
+            window::RedrawRequest::NextFrame => shell.request_redraw(),
+            window::RedrawRequest::At(at) => shell.request_redraw_at(at),
+            window::RedrawRequest::Wait => {}
+        }
+
         // Then finally react to them here
         for message in local_messages {
             let TextInputEvent::TextChanged(new_value) = message;
 
             if let Some(on_input) = &self.on_input {
                 shell.publish((on_input)(new_value.clone()));
-                published_message_to_shell = true;
             }
 
             // Couple the filtered options with the `ComboBox`
@@ -462,6 +471,7 @@ where
                 );
             });
             shell.invalidate_layout();
+            shell.request_redraw();
         }
 
         if self.state.is_focused() {
@@ -488,7 +498,8 @@ where
                                 }
                             }
 
-                            event_status = event::Status::Captured;
+                            shell.capture_event();
+                            shell.request_redraw();
                         }
                         (keyboard::Key::Named(keyboard::key::Named::ArrowUp), _)
                         | (keyboard::Key::Named(keyboard::key::Named::Tab), true) => {
@@ -513,7 +524,8 @@ where
                                 }
                             }
 
-                            event_status = event::Status::Captured;
+                            shell.capture_event();
+                            shell.request_redraw();
                         }
                         (keyboard::Key::Named(keyboard::key::Named::ArrowDown), _)
                         | (keyboard::Key::Named(keyboard::key::Named::Tab), false) => {
@@ -541,7 +553,8 @@ where
                                 }
                             }
 
-                            event_status = event::Status::Captured;
+                            shell.capture_event();
+                            shell.request_redraw();
                         }
                         _ => {}
                     }
@@ -563,9 +576,9 @@ where
 
                 // Unfocus the input
                 let mut tree = state.text_input_tree();
-                let _ = self.text_input.on_event(
+                self.text_input.update(
                     &mut tree,
-                    Event::Mouse(mouse::Event::ButtonPressed(mouse::Button::Left)),
+                    &Event::Mouse(mouse::Event::ButtonPressed(mouse::Button::Left)),
                     layout,
                     mouse::Cursor::Unavailable,
                     renderer,
@@ -587,8 +600,6 @@ where
         if started_focused != self.state.is_focused() {
             shell.invalidate_widgets();
         }
-
-        event_status
     }
 
     fn mouse_interaction(
