@@ -3,7 +3,7 @@ use std::slice;
 use iced::advanced::widget::{operation, tree, Operation};
 use iced::advanced::{self, layout, overlay, renderer, widget, Clipboard, Layout, Shell, Widget};
 use iced::widget::{column, container};
-use iced::{event, mouse, Element, Event, Length, Point, Rectangle, Size, Task, Vector};
+use iced::{mouse, Element, Event, Length, Point, Rectangle, Size, Task, Vector};
 
 pub use iced::widget::container::{Style, StyleFn};
 
@@ -60,7 +60,7 @@ impl State {
     }
 }
 
-#[derive(Debug, Clone, Copy)]
+#[derive(Debug, Clone, Copy, PartialEq)]
 pub enum Status {
     Closed,
     Open(Point),
@@ -121,7 +121,7 @@ where
             layout,
             cursor,
             viewport,
-        )
+        );
     }
 
     fn tag(&self) -> tree::Tag {
@@ -152,25 +152,26 @@ where
     ) {
         let state = tree.state.downcast_mut::<State>();
 
-        operation.custom(state, None);
+        operation.custom(None, layout.bounds(), state);
 
         self.base
             .as_widget()
             .operate(&mut tree.children[0], layout, renderer, operation);
     }
 
-    fn on_event(
+    fn update(
         &mut self,
         tree: &mut widget::Tree,
-        event: Event,
+        event: &Event,
         layout: Layout<'_>,
         cursor: mouse::Cursor,
         renderer: &Renderer,
         clipboard: &mut dyn Clipboard,
         shell: &mut Shell<'_, Message>,
         viewport: &Rectangle,
-    ) -> event::Status {
+    ) {
         let state = tree.state.downcast_mut::<State>();
+        let prev_status = state.status;
 
         let position = match self.activation_button {
             mouse::Button::Left => {
@@ -194,7 +195,14 @@ where
             state.status = Status::Open(position);
         }
 
-        self.base.as_widget_mut().on_event(
+        match (state.status, prev_status) {
+            (Status::Closed, Status::Open(_)) | (Status::Open(_), Status::Closed) => {
+                shell.request_redraw();
+            }
+            _ => {}
+        }
+
+        self.base.as_widget_mut().update(
             &mut tree.children[0],
             event,
             layout,
@@ -203,7 +211,7 @@ where
             clipboard,
             shell,
             viewport,
-        )
+        );
     }
 
     fn mouse_interaction(
@@ -336,10 +344,15 @@ pub fn close<Message: 'static + Send>(f: fn(bool) -> Message) -> Task<Message> {
             _bounds: Rectangle,
             operate_on_children: &mut dyn FnMut(&mut dyn Operation<T>),
         ) {
-            operate_on_children(self)
+            operate_on_children(self);
         }
 
-        fn custom(&mut self, state: &mut dyn std::any::Any, _id: Option<&widget::Id>) {
+        fn custom(
+            &mut self,
+            _id: Option<&widget::Id>,
+            _bounds: Rectangle,
+            state: &mut dyn std::any::Any,
+        ) {
             if let Some(state) = state.downcast_mut::<State>() {
                 if let Status::Open(_) = state.status {
                     state.status = Status::Closed;
@@ -379,8 +392,8 @@ struct Overlay<'a, 'b, Message, Theme, Renderer> {
     position: Point,
 }
 
-impl<'a, 'b, Message, Theme, Renderer> overlay::Overlay<Message, Theme, Renderer>
-    for Overlay<'a, 'b, Message, Theme, Renderer>
+impl<Message, Theme, Renderer> overlay::Overlay<Message, Theme, Renderer>
+    for Overlay<'_, '_, Message, Theme, Renderer>
 where
     Renderer: advanced::Renderer,
 {
@@ -447,15 +460,15 @@ where
             .operate(&mut self.state.menu_tree, layout, renderer, operation);
     }
 
-    fn on_event(
+    fn update(
         &mut self,
-        event: Event,
+        event: &Event,
         layout: Layout<'_>,
         cursor: mouse::Cursor,
         renderer: &Renderer,
         clipboard: &mut dyn Clipboard,
         shell: &mut Shell<'_, Message>,
-    ) -> event::Status {
+    ) {
         if let Event::Mouse(mouse::Event::ButtonPressed(_)) = &event {
             if cursor.position_over(layout.bounds()).is_none() {
                 self.state.status = Status::Closed;
@@ -468,7 +481,11 @@ where
             }
         }
 
-        self.menu.as_widget_mut().on_event(
+        if matches!(self.state.status, Status::Closed) {
+            shell.request_redraw();
+        }
+
+        self.menu.as_widget_mut().update(
             &mut self.state.menu_tree,
             event,
             layout,
@@ -477,7 +494,7 @@ where
             clipboard,
             shell,
             &layout.bounds(),
-        )
+        );
     }
 
     fn mouse_interaction(
