@@ -8,7 +8,6 @@ use iced::advanced::widget::tree::{self, Tree};
 use iced::advanced::widget::Operation;
 use iced::advanced::{Clipboard, Layout, Shell, Widget};
 use iced::alignment;
-use iced::event;
 use iced::mouse;
 use iced::widget;
 use iced::widget::container;
@@ -54,7 +53,7 @@ where
     width: Length,
     height: Length,
     font: Option<Renderer::Font>,
-    align_x: alignment::Horizontal,
+    align_x: text::Alignment,
     align_y: alignment::Vertical,
     class: Theme::Class<'a>,
     on_link: Option<Box<dyn Fn(Link) -> Message + 'a>>,
@@ -83,7 +82,7 @@ where
             width: Length::Shrink,
             height: Length::Shrink,
             font: None,
-            align_x: alignment::Horizontal::Left,
+            align_x: text::Alignment::Left,
             align_y: alignment::Vertical::Top,
             class: Theme::default(),
             on_link: None,
@@ -139,7 +138,7 @@ where
     }
 
     /// Sets the [`alignment::Horizontal`] of the [`Rich`] text.
-    pub fn align_x(mut self, alignment: impl Into<alignment::Horizontal>) -> Self {
+    pub fn align_x(mut self, alignment: impl Into<text::Alignment>) -> Self {
         self.align_x = alignment.into();
         self
     }
@@ -199,8 +198,8 @@ where
     }
 }
 
-impl<'a, Message, Link, Entry, Theme, Renderer> Default
-    for Rich<'a, Message, Link, Entry, Theme, Renderer>
+impl<Message, Link, Entry, Theme, Renderer> Default
+    for Rich<'_, Message, Link, Entry, Theme, Renderer>
 where
     Link: self::Link + 'static,
     Theme: Catalog,
@@ -223,6 +222,8 @@ struct State<Link, P: Paragraph> {
     spans: Vec<Span<'static, Link, P::Font>>,
     span_pressed: Option<usize>,
     paragraph: P,
+    hovered: bool,
+    link_hovered: bool,
     interaction: Interaction,
     shown_spoiler: Option<(usize, Color, Highlight)>,
 
@@ -230,6 +231,39 @@ struct State<Link, P: Paragraph> {
     context_menu: context_menu::State,
 }
 
+struct Snapshot {
+    hovered: bool,
+    link_hovered: bool,
+    span_pressed: Option<usize>,
+    interaction: Interaction,
+    context_menu_status: context_menu::Status,
+    shown_spoiler: Option<(usize, Color, Highlight)>,
+}
+
+impl<Link, P: Paragraph> From<&State<Link, P>> for Snapshot {
+    fn from(value: &State<Link, P>) -> Self {
+        Snapshot {
+            hovered: value.hovered,
+            link_hovered: value.link_hovered,
+            span_pressed: value.span_pressed,
+            interaction: value.interaction,
+            context_menu_status: value.context_menu.status,
+            shown_spoiler: value.shown_spoiler,
+        }
+    }
+}
+
+impl Snapshot {
+    fn is_changed(&self, other: &Self) -> bool {
+        self.hovered != other.hovered
+            || self.link_hovered != other.link_hovered
+            || self.span_pressed != other.span_pressed
+            || self.interaction != other.interaction
+            || self.context_menu_status != other.context_menu_status
+            || self.shown_spoiler != other.shown_spoiler
+    }
+}
+
 impl<'a, Message, Link, Entry, Theme, Renderer> Widget<Message, Theme, Renderer>
     for Rich<'a, Message, Link, Entry, Theme, Renderer>
 where
@@ -253,6 +287,8 @@ where
             shown_spoiler: None,
             context_menu_link: None,
             context_menu: context_menu::State::new(),
+            hovered: false,
+            link_hovered: false,
         })
     }
 
@@ -285,30 +321,48 @@ where
         )
     }
 
-    fn on_event(
+    fn update(
         &mut self,
         tree: &mut Tree,
-        event: Event,
+        event: &Event,
         layout: Layout<'_>,
         cursor: mouse::Cursor,
         renderer: &Renderer,
         _clipboard: &mut dyn Clipboard,
         shell: &mut Shell<'_, Message>,
         viewport: &Rectangle,
-    ) -> event::Status {
+    ) {
         let state = tree
             .state
             .downcast_mut::<State<Link, Renderer::Paragraph>>();
+        let prev_snapshot = Snapshot::from(&*state);
 
         let bounds = layout.bounds();
 
         if viewport.intersection(&bounds).is_none()
             && matches!(state.interaction, Interaction::Idle)
         {
-            return event::Status::Ignored;
+            return;
         }
 
-        let mut status = event::Status::Ignored;
+        state.hovered = false;
+        state.link_hovered = false;
+
+        if let Some(position) = cursor.position_in(layout.bounds()) {
+            state.hovered = true;
+
+            if self.on_link.is_some() {
+                if let Some(span) = state
+                    .paragraph
+                    .hit_span(position)
+                    .and_then(|span| self.spans.get(span))
+                {
+                    if span.link.is_some() {
+                        state.link_hovered = true;
+                    }
+                }
+            }
+        }
 
         match event {
             iced::Event::Mouse(mouse::Event::ButtonPressed(mouse::Button::Left))
@@ -316,8 +370,7 @@ where
                 if let Some(position) = cursor.position_in(bounds) {
                     if let Some(span) = state.paragraph.hit_span(position) {
                         state.span_pressed = Some(span);
-
-                        status = event::Status::Captured;
+                        shell.capture_event();
                     }
                 }
 
@@ -375,10 +428,10 @@ where
                     size,
                     line_height: self.line_height,
                     font,
-                    horizontal_alignment: self.align_x,
-                    vertical_alignment: self.align_y,
+                    align_x: self.align_x,
+                    align_y: self.align_y,
                     shaping: Shaping::Advanced,
-                    wrapping: Default::default(),
+                    wrapping: text::Wrapping::default(),
                 };
 
                 // Check spoiler
@@ -460,7 +513,9 @@ where
             _ => {}
         }
 
-        status
+        if prev_snapshot.is_changed(&Snapshot::from(&*state)) {
+            shell.request_redraw();
+        }
     }
 
     fn draw(
@@ -571,11 +626,8 @@ where
             .and_then(|raw| raw.resolve(bounds))
         {
             let line_height = f32::from(
-                self.line_height.to_absolute(
-                    self.size
-                        .map(Pixels::from)
-                        .unwrap_or_else(|| renderer.default_size()),
-                ),
+                self.line_height
+                    .to_absolute(self.size.unwrap_or_else(|| renderer.default_size())),
             );
 
             let baseline_y =
@@ -629,29 +681,21 @@ where
     fn mouse_interaction(
         &self,
         tree: &Tree,
-        layout: Layout<'_>,
-        cursor: mouse::Cursor,
+        _layout: Layout<'_>,
+        _cursor: mouse::Cursor,
         _viewport: &Rectangle,
         _renderer: &Renderer,
     ) -> mouse::Interaction {
-        if let Some(position) = cursor.position_in(layout.bounds()) {
-            let state = tree
-                .state
-                .downcast_ref::<State<Link, Renderer::Paragraph>>();
+        let state = tree
+            .state
+            .downcast_ref::<State<Link, Renderer::Paragraph>>();
 
-            if self.on_link.is_some() {
-                if let Some(span) = state
-                    .paragraph
-                    .hit_span(position)
-                    .and_then(|span| self.spans.get(span))
-                {
-                    if span.link.is_some() {
-                        return mouse::Interaction::Pointer;
-                    }
-                }
+        if state.hovered {
+            if state.link_hovered {
+                mouse::Interaction::Pointer
+            } else {
+                mouse::Interaction::Text
             }
-
-            mouse::Interaction::Text
         } else {
             mouse::Interaction::None
         }
@@ -675,12 +719,12 @@ where
             .selection()
             .and_then(|raw| selection(raw, bounds, &state.paragraph, &value))
         {
-            let content = value.select(selection.start, selection.end).to_string();
-            operation.custom(&mut (bounds.y, content), None);
+            let mut content = value.select(selection.start, selection.end).to_string();
+            operation.custom(None, bounds, &mut content);
         }
 
         // Context menu
-        operation.custom(&mut state.context_menu, None);
+        operation.custom(None, bounds, &mut state.context_menu);
     }
 
     fn overlay<'b>(
@@ -734,8 +778,8 @@ fn layout<Link, Renderer>(
     line_height: LineHeight,
     size: Option<Pixels>,
     font: Option<Renderer::Font>,
-    horizontal_alignment: alignment::Horizontal,
-    vertical_alignment: alignment::Vertical,
+    align_x: text::Alignment,
+    align_y: alignment::Vertical,
 ) -> layout::Node
 where
     Link: Clone,
@@ -753,10 +797,10 @@ where
             size,
             line_height,
             font,
-            horizontal_alignment,
-            vertical_alignment,
+            align_x,
+            align_y,
             shaping: Shaping::Advanced,
-            wrapping: Default::default(),
+            wrapping: text::Wrapping::default(),
         };
 
         if state.spans != spans {
@@ -779,10 +823,10 @@ where
                 size,
                 line_height,
                 font,
-                horizontal_alignment,
-                vertical_alignment,
+                align_x,
+                align_y,
                 shaping: Shaping::Advanced,
-                wrapping: Default::default(),
+                wrapping: text::Wrapping::default(),
             }) {
                 text::Difference::None => {}
                 text::Difference::Bounds => {
