@@ -1,12 +1,11 @@
 use iced::advanced::renderer::Quad;
-use iced::advanced::text::{paragraph, Paragraph};
-use iced::advanced::widget::{operation, tree, Operation, Tree};
-use iced::advanced::{layout, mouse, renderer, text, widget, Layout, Widget};
+use iced::advanced::text::{Paragraph, paragraph};
+use iced::advanced::widget::{Operation, Tree, operation, tree};
+use iced::advanced::{Layout, Widget, layout, mouse, renderer, text, widget};
 use iced::widget::text::{Fragment, IntoFragment, Wrapping};
 use iced::widget::text_input::Value;
 use iced::{
-    alignment, event, touch, Border, Color, Element, Length, Pixels, Point, Rectangle, Shadow,
-    Size, Task,
+    Border, Color, Element, Length, Pixels, Point, Rectangle, Shadow, Size, Task, alignment, touch,
 };
 
 pub use self::selection::selection;
@@ -34,8 +33,8 @@ where
     line_height: LineHeight,
     width: Length,
     height: Length,
-    horizontal_alignment: alignment::Horizontal,
-    vertical_alignment: alignment::Vertical,
+    align_x: text::Alignment,
+    align_y: alignment::Vertical,
     font: Option<Renderer::Font>,
     shaping: Shaping,
     wrapping: Wrapping,
@@ -55,13 +54,13 @@ where
             font: None,
             width: Length::Shrink,
             height: Length::Shrink,
-            horizontal_alignment: alignment::Horizontal::Left,
-            vertical_alignment: alignment::Vertical::Top,
+            align_x: text::Alignment::Left,
+            align_y: alignment::Vertical::Top,
             #[cfg(debug_assertions)]
             shaping: Shaping::Basic,
             #[cfg(not(debug_assertions))]
             shaping: Shaping::Advanced,
-            wrapping: Default::default(),
+            wrapping: Wrapping::default(),
             class: Theme::default(),
         }
     }
@@ -99,13 +98,13 @@ where
         self
     }
 
-    pub fn horizontal_alignment(mut self, alignment: alignment::Horizontal) -> Self {
-        self.horizontal_alignment = alignment;
+    pub fn align_x(mut self, alignment: text::Alignment) -> Self {
+        self.align_x = alignment;
         self
     }
 
-    pub fn vertical_alignment(mut self, alignment: alignment::Vertical) -> Self {
-        self.vertical_alignment = alignment;
+    pub fn align_y(mut self, alignment: alignment::Vertical) -> Self {
+        self.align_y = alignment;
         self
     }
 
@@ -120,7 +119,7 @@ where
     }
 }
 
-impl<'a, Message, Theme, Renderer> Widget<Message, Theme, Renderer> for Text<'a, Theme, Renderer>
+impl<Message, Theme, Renderer> Widget<Message, Theme, Renderer> for Text<'_, Theme, Renderer>
 where
     Renderer: text::Renderer,
     Theme: Catalog,
@@ -151,10 +150,7 @@ where
         layout::sized(limits, self.width, self.height, |limits| {
             let bounds = limits.max();
 
-            let size = self
-                .size
-                .map(Pixels::from)
-                .unwrap_or_else(|| renderer.default_size());
+            let size = self.size.unwrap_or_else(|| renderer.default_size());
             let font = self.font.unwrap_or_else(|| renderer.default_font());
 
             state.paragraph.update(text::Text {
@@ -163,8 +159,8 @@ where
                 line_height: self.line_height,
                 bounds,
                 font,
-                horizontal_alignment: self.horizontal_alignment,
-                vertical_alignment: self.vertical_alignment,
+                align_x: self.align_x,
+                align_y: self.align_y,
                 shaping: self.shaping,
                 wrapping: self.wrapping,
             });
@@ -173,19 +169,25 @@ where
         })
     }
 
-    fn on_event(
+    fn update(
         &mut self,
         tree: &mut Tree,
-        event: iced::Event,
-        _layout: Layout<'_>,
+        event: &iced::Event,
+        layout: Layout<'_>,
         cursor: mouse::Cursor,
         _renderer: &Renderer,
         _clipboard: &mut dyn iced::advanced::Clipboard,
-        _shell: &mut iced::advanced::Shell<'_, Message>,
+        shell: &mut iced::advanced::Shell<'_, Message>,
         _viewport: &Rectangle,
-    ) -> event::Status {
+    ) {
         let state = tree.state.downcast_mut::<State<Renderer::Paragraph>>();
 
+        let bounds = layout.bounds();
+
+        let prev_hovered = state.hovered;
+        let prev_interaction = state.interaction;
+        state.hovered = cursor.is_over(bounds);
+
         match event {
             iced::Event::Mouse(mouse::Event::ButtonPressed(mouse::Button::Left))
             | iced::Event::Touch(touch::Event::FingerPressed { .. }) => {
@@ -218,7 +220,9 @@ where
             _ => {}
         }
 
-        event::Status::Ignored
+        if prev_hovered != state.hovered || prev_interaction != state.interaction {
+            shell.request_redraw();
+        }
     }
 
     fn draw(
@@ -247,11 +251,8 @@ where
             .and_then(|raw| raw.resolve(bounds))
         {
             let line_height = f32::from(
-                self.line_height.to_absolute(
-                    self.size
-                        .map(Pixels::from)
-                        .unwrap_or_else(|| renderer.default_size()),
-                ),
+                self.line_height
+                    .to_absolute(self.size.unwrap_or_else(|| renderer.default_size())),
             );
 
             let baseline_y =
@@ -365,13 +366,15 @@ where
 
     fn mouse_interaction(
         &self,
-        _state: &Tree,
-        layout: Layout<'_>,
-        cursor: mouse::Cursor,
+        tree: &Tree,
+        _layout: Layout<'_>,
+        _cursor: mouse::Cursor,
         _viewport: &Rectangle,
         _renderer: &Renderer,
     ) -> mouse::Interaction {
-        if cursor.position_over(layout.bounds()).is_some() {
+        let state = tree.state.downcast_ref::<State<Renderer::Paragraph>>();
+
+        if state.hovered {
             mouse::Interaction::Text
         } else {
             mouse::Interaction::default()
@@ -394,8 +397,8 @@ where
             .selection()
             .and_then(|raw| selection(raw, bounds, state.paragraph.raw(), &value))
         {
-            let content = value.select(selection.start, selection.end).to_string();
-            operation.custom(&mut (bounds.y, content), None);
+            let mut content = value.select(selection.start, selection.end).to_string();
+            operation.custom(None, bounds, &mut content);
         }
     }
 }
@@ -413,13 +416,13 @@ fn draw<Renderer>(
     let State { paragraph, .. } = &state;
     let bounds = layout.bounds();
 
-    let x = match paragraph.horizontal_alignment() {
-        alignment::Horizontal::Left => bounds.x,
-        alignment::Horizontal::Center => bounds.center_x(),
-        alignment::Horizontal::Right => bounds.x + bounds.width,
+    let x = match paragraph.align_x() {
+        text::Alignment::Default | text::Alignment::Left | text::Alignment::Justified => bounds.x,
+        text::Alignment::Center => bounds.center_x(),
+        text::Alignment::Right => bounds.x + bounds.width,
     };
 
-    let y = match paragraph.vertical_alignment() {
+    let y = match paragraph.align_y() {
         alignment::Vertical::Top => bounds.y,
         alignment::Vertical::Center => bounds.center_y(),
         alignment::Vertical::Bottom => bounds.y + bounds.height,
@@ -448,9 +451,10 @@ where
 pub struct State<P: Paragraph> {
     paragraph: paragraph::Plain<P>,
     interaction: Interaction,
+    hovered: bool,
 }
 
-#[derive(Debug, Clone, Copy, Default)]
+#[derive(Debug, Clone, Copy, Default, PartialEq)]
 pub enum Interaction {
     #[default]
     Idle,
@@ -495,12 +499,17 @@ pub fn selected<Message: Send + 'static>(f: fn(Vec<(f32, String)>) -> Message) -
             _bounds: Rectangle,
             operate_on_children: &mut dyn FnMut(&mut dyn Operation<T>),
         ) {
-            operate_on_children(self)
+            operate_on_children(self);
         }
 
-        fn custom(&mut self, state: &mut dyn std::any::Any, _id: Option<&widget::Id>) {
-            if let Some(content) = state.downcast_ref::<(f32, String)>() {
-                self.contents.push(content.clone());
+        fn custom(
+            &mut self,
+            _id: Option<&widget::Id>,
+            bounds: Rectangle,
+            state: &mut dyn std::any::Any,
+        ) {
+            if let Some(content) = state.downcast_ref::<String>() {
+                self.contents.push((bounds.y, content.clone()));
             }
         }
 
