@@ -1,7 +1,7 @@
 use data::shortcut;
 use iced::advanced::widget::Tree;
 use iced::advanced::{Clipboard, Layout, Shell};
-use iced::{event, keyboard, mouse, Event};
+use iced::{keyboard, mouse, Event};
 
 pub use data::shortcut::Command;
 
@@ -16,11 +16,11 @@ where
     Message: 'a,
 {
     decorate(base)
-        .on_event(
+        .update(
             move |modifiers: &mut shortcut::Modifiers,
                   inner: &mut Element<'a, Message>,
                   tree: &mut Tree,
-                  event: iced::Event,
+                  event: &iced::Event,
                   layout: Layout<'_>,
                   cursor: mouse::Cursor,
                   renderer: &Renderer,
@@ -36,7 +36,8 @@ where
                             .find_map(|shortcut| shortcut.execute(&key_bind))
                         {
                             shell.publish((on_press)(command));
-                            return event::Status::Captured;
+                            shell.capture_event();
+                            return;
                         }
                     }
                     Event::Keyboard(keyboard::Event::ModifiersChanged(new_modifiers)) => {
@@ -45,9 +46,9 @@ where
                     _ => {}
                 }
 
-                inner.as_widget_mut().on_event(
+                inner.as_widget_mut().update(
                     tree, event, layout, cursor, renderer, clipboard, shell, viewport,
-                )
+                );
             },
         )
         .into()
