@@ -1,8 +1,8 @@
 use futures::{stream::BoxStream, Stream, StreamExt};
-use iced::{advanced::graphics::futures::subscription, Point, Size, Subscription};
+use iced::{advanced::graphics::futures::subscription, Point, Size, Subscription, Task};
 
-pub use data::window::{default_size, Error, MIN_SIZE};
-pub use iced::window::{close, gain_focus, open, Id, Position, Settings};
+pub use data::window::{Error, MIN_SIZE};
+pub use iced::window::{close, gain_focus, get_latest, open, Id, Position, Settings};
 
 #[derive(Debug, Clone, Copy)]
 pub struct Window {
@@ -38,6 +38,22 @@ impl From<Window> for data::Window {
     }
 }
 
+pub fn toggle_fullscreen<Message: 'static + Send>() -> Task<Message> {
+    get_latest().and_then(move |window| {
+        iced::window::get_mode(window).then(move |mode| {
+            iced::window::set_mode(
+                window,
+                match mode {
+                    iced::window::Mode::Windowed => iced::window::Mode::Fullscreen,
+                    iced::window::Mode::Fullscreen => iced::window::Mode::Windowed,
+                    // Do nothing.
+                    iced::window::Mode::Hidden => iced::window::Mode::Hidden,
+                },
+            )
+        })
+    })
+}
+
 #[derive(Debug, Clone, Copy)]
 pub enum Event {
     Moved(Point),
@@ -151,18 +167,46 @@ impl subscription::Recipe for Events {
         events: subscription::EventStream,
     ) -> BoxStream<'static, Self::Output> {
         use futures::stream;
-        const TIMEOUT: std::time::Duration = std::time::Duration::from_millis(500);
 
-        let window_events = events.filter_map(|event| {
+        const TIMEOUT: std::time::Duration = std::time::Duration::from_millis(500);
+        const INITIAL_SKIP_THRESHOLD: std::time::Duration = std::time::Duration::from_secs(2);
+
+        let start_time = std::time::Instant::now();
+
+        let window_events = events.filter_map(move |event| {
+            let elapsed = start_time.elapsed();
+
             futures::future::ready(match event {
                 subscription::Event::Interaction {
                     window: id,
                     event: iced::Event::Window(window_event),
                     status: _,
                 } => match window_event {
-                    iced::window::Event::Moved(point) => Some((id, Event::Moved(point))),
-                    iced::window::Event::Resized(Size { width, height }) => {
-                        Some((id, Event::Resized(Size::new(width, height))))
+                    iced::window::Event::Moved(point) => {
+                        let point_is_positive =
+                            point.x.is_sign_positive() && point.y.is_sign_positive();
+
+                        if point_is_positive && elapsed >= INITIAL_SKIP_THRESHOLD {
+                            Some((
+                                id,
+                                Event::Moved(Point {
+                                    x: point.x.max(0.0),
+                                    y: point.y.max(0.0),
+                                }),
+                            ))
+                        } else {
+                            None
+                        }
+                    }
+                    iced::window::Event::Resized(size) => {
+                        let is_bigger_than_min_allowed =
+                            size.width >= MIN_SIZE.width && size.height >= MIN_SIZE.height;
+
+                        if is_bigger_than_min_allowed && elapsed >= INITIAL_SKIP_THRESHOLD {
+                            Some((id, Event::Resized(size.max(MIN_SIZE))))
+                        } else {
+                            None
+                        }
                     }
                     iced::window::Event::Focused => Some((id, Event::Focused)),
                     iced::window::Event::Unfocused => Some((id, Event::Unfocused)),
